#!/usr/bin/env python3
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2014, 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import deb822
import getpass
import glob
import json
import logging
import os
import socket
import subprocess
import sys
import tempfile

from argparse import ArgumentParser
from ubuntu_system_tests import (
    config,
    results
)
from ubuntu_system_tests.helpers.file_system import clean_dir

logger = logging.getLogger(__name__)
_ADT_RUN_SHORT_TIMEOUT = 600  # 10 minutes.


def main(argv=None):
    if argv is None:
        argv = sys.argv[1:]
    args = _parse_arguments(argv)
    return run_system_tests(args.config, args.suites,
                            args.update_apt, args.network_ready)


def _parse_arguments(argv=None):
    parser = ArgumentParser('Run system tests for Ubuntu Touch.')
    parser.add_argument('--config', '-c',
                        help='The config file to use for this run.',
                        default='ubuntu-system-tests.conf')
    parser.add_argument(
        '--update-apt', '-u', action='store_true',
        help='Update apt lists before installing the dependencies',
        default=False)
    parser.add_argument(
        '--network-ready', '-n', action='store_true',
        help='Assume that the network connection is already setup.')
    parser.add_argument(
        'suites', nargs='*', help='The list of test suites to run.')
    return parser.parse_args(args=argv)


def run_system_tests(conf_name, tests_to_run, update_apt=False,
                     network_ready=False):
    """Run system tests in a device connected via USB and report the results.
    :param conf_name: The name of the configuration file that will be used
        for the system tests.
    :param tests_to_run: A list of tests to run.
    :param update_apt: Boolean representing whether apt-get update should be
        run on the device before installing dependencies.
    :param network: Boolean representing whether phablet-network should be run
        to provision the network on the device.
    :returns: 0 when successful, other value when failed.

    """
    config_stack = _get_config_stack(conf_name, tests_to_run)
    create_temporal_config_file(config_stack)
    print_warnings()
    clean_output_directory(config_stack)

    enable_autopilot_introspection(config_stack)
    return_code = run_autopkg_test(config_stack, update_apt, network_ready)
    config.delete_temporal_config_file()

    if return_code <= 6:
        # Return codes higher than 6 mean that there was a problem with adt or
        # the test bed, not a failure in the tests. So there will be no file
        # with results.
        generate_user_report(config_stack)

    return return_code


def get_camera_warning():
    return '- Please ensure device is raised from surface to allow rear ' \
           'camera to take a photograph during camera tests.'


def print_warnings():
    print_separator('*')
    logger.warn(get_camera_warning())
    print_separator('*')


def print_separator(char):
    logger.warning(char * 125)


def clean_output_directory(config_stack):
    """
    Clean the output directory specified in config_stack.

    This is required so that adt-run does not fail due to existing content
    in the output directory from a previous test run.

    """
    path = os.path.abspath(config_stack.get('output_dir'))
    if os.path.isdir(path):
        clean_dir(path)


def _get_config_stack(conf_name, tests_to_run):
    """Get the configuration values for the system tests.

    This function registers the declared options, opens the config store
    using the specified file name, adds the options coming from the command
    line, and asks the user for the missing options.

    """
    config.register_test_options()

    config_stack = config.UbuntuSystemTestsStack(conf_name)

    # We set the values passed from the command line to send them from the host
    # to the testbed.
    config_stack.set('tests_to_run', ' '.join(tests_to_run or []))
    config_stack.set('ssh_ip', _get_local_ip_address())
    config_stack.set('ssh_user', getpass.getuser())
    config_stack.get_missing_options_from_stdin()

    return config_stack


def _get_local_ip_address():
    """ Retrieve the local ip address
    :return: The ip obtained from the socket connection
    """
    ip = '127.0.0.1'
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(("8.8.8.8", 80))
        ip = s.getsockname()[0]
    finally:
        s.close()
    return ip


def _get_serial_argument(config_stack):
    device_serial = config_stack.get('device_serial')
    if device_serial:
        return ['-s', device_serial]
    else:
        # No serial in the config, run the commands as if only one device is
        # connected.
        return []


def enable_autopilot_introspection(config_stack):
    command = _build_enable_autopilot_introspection_command(config_stack)
    subprocess.check_output(command)


def _build_enable_autopilot_introspection_command(config_stack):
    command = (
        ['phablet-config'] +
        _get_serial_argument(config_stack) +
        ['autopilot', '--dbus-probe', 'enable']
    )
    return command


def run_autopkg_test(config_stack, update_apt, network_ready):
    extra_depends = _get_extra_dependencies(
        config_stack.get('tests_to_run', '').split()
    )
    adt_run_cmd = _build_adt_run_test_command(
        config_stack, extra_depends,
        update_apt, network_ready)
    return subprocess.call(adt_run_cmd)


def _get_extra_dependencies(tests_to_run):
    """
    Get any extra dependencies required to run the list of
    tests provided

    :param tests_to_run: The list of tests to run
    :returns: A list of the extra dependencies
    """
    depends_map = _read_dependency_files(
        os.path.realpath('ubuntu_system_tests/external_tests')
    )
    extra_depends = []
    for test in tests_to_run:
        extra_depends.extend(_extra_dependencies_for_test(test, depends_map))
    return list(set(extra_depends))


def _read_dependency_files(deps_path):
    """
    Read the information about how dependencies map
    to test names from the .json files in the path
    provided.

    :param deps_path: The path to the directory containing the files
    :returns: A dictionary associating test prefixes with lists of dependencies
    """
    depends_map = {}
    for file in glob.glob(deps_path + '/*.json'):
        with open(file) as deps_file:
            for depends in json.loads(deps_file.read())['test_dependencies']:
                depends_map[depends['id']] = depends['depends']
    return depends_map


def _extra_dependencies_for_test(test, depends_map):
    """
    Calculates the extra dependencies required for a particular test

    :param test: The test to get the dependencies for
    :param depends_map: The dependency mapping
    :returns: A list of the extra dependencies
    """
    extra_dependencies = []
    for id in depends_map.keys():
        if test.startswith(id):
            extra_dependencies.append(depends_map[id])
    return extra_dependencies


def _create_temp_control_file(dependencies,
                              control_file='debian/tests/control'):
    """Create a copy of the test suite control file, appending the list of
    dependencies

    :param dependencies: List of dependencies to append to file
    :returns: File path to the new control file
    """
    # create a temporary file
    _, tmp_path = tempfile.mkstemp(text=True)
    # open existing control file
    with open(os.path.realpath(control_file), 'r') as tests_control:
        new_content = _append_dependencies(
            tests_control, dependencies)
        # write the new content to the temp file
        with open(tmp_path, 'w') as tmp_file:
            tmp_file.write(new_content)
    # save the path in config
    return tmp_path


def _append_dependencies(tests_control, dependencies):
    """Append the list of dependencies to the file content and return"""
    content = ''
    for pkg in deb822.Packages.iter_paragraphs(tests_control):
        if pkg['Tests'] == 'systemtests':
            current_deps = pkg['Depends']
            for dep in dependencies:
                current_deps += '\n         {},'.format(dep)
            pkg['Depends'] = current_deps
        content += pkg.dump() + '\n'
    return content.rstrip()


def _add_copy_to_temp_config_file(run_cmd):
    """ Add the copy parameter for the temp config file to the command line"""
    if os.path.isfile(config.get_device_config_temp_file_path()):
        run_cmd.extend(['--copy', '{src}:{dest}'.format(
            src=config.get_device_config_temp_file_path(),
            dest=config.get_device_config_temp_file_path())])


def _build_adt_run_test_command(config_stack, extra_depends=None,
                                update_apt=False, network_ready=False):
    output_dir = config_stack.get('output_dir')
    device_password = config_stack.get('device_password')

    run_cmd = ['adt-run']
    if extra_depends:
        run_cmd.append('--override-control={}'.format(
            _create_temp_control_file(extra_depends))
        )
    run_cmd.extend([
        # Updating and installing can take a while.
        '--timeout-short={}'.format(_ADT_RUN_SHORT_TIMEOUT),
        '-B',
        '--unbuilt-tree={}'.format(os.getcwd()),
        '--output-dir',
        output_dir,
        # Even if the config file on the host has a different name, on the
        # testbed it's copied with the default name.
        '--copy',
        '{src}:{dest}'.format(
            src=config_stack.store.path,
            dest=config.get_device_config_file_path()),
    ])

    _add_copy_to_temp_config_file(run_cmd)

    if not network_ready:
        # The network needs to be configured on the device
        # This setup-command should be the first to run so that wi-fi is ready
        network_cmd = _get_device_network_command(
            config_stack.get('wifi_ssid'),
            config_stack.get('wifi_password'))
        run_cmd.extend(['--setup-commands', network_cmd])

    ppa_setup_command = _get_ppa_setup_command()

    if update_apt:
        run_cmd.extend(['--setup-commands', 'ro-apt-update'])

    ssh_cmd = (
        ['ssh', '-s', 'adb', '--', '-p', device_password] +
        _get_serial_argument(config_stack)
    )
    return run_cmd + ppa_setup_command + ['---'] + ssh_cmd


def _get_device_network_command(wifi_ssid, wifi_password):
    """Return command to configure wifi network on device.

    :param wifi_ssid: SSID of required wifi network
    :param wifi_password: Password of required wifi network

    """
    cmd_fmt = ('nmcli device wifi connect '
               '\'{ssid}\' password \'{password}\' 2>/dev/null || true')
    return cmd_fmt.format(ssid=wifi_ssid, password=wifi_password)


def _get_ppa_setup_command():
    return [
        '--setup-commands',
        'mount -o remount,rw /',
        '--setup-commands',
        'apt-add-repository -y ppa:canonical-platform-qa/selenium',
        '--setup-commands',
        'apt-get --no-list-cleanup update -o Dir::Etc::SourceList=/dev/null',
        '--setup-commands',
        'sync; sleep 2; mount -o remount,ro /'
    ]


def create_temporal_config_file(config_stack):
    """Create the temporal config stack"""
    ssh_password_key = 'ssh_password'
    ssh_password_value = ''
    if config_stack.get(ssh_password_key) is None:
        ssh_password_value = collect_user_password()
    config.create_temporal_config_file({ssh_password_key: ssh_password_value})


def collect_user_password():
    """ Collect user password to run ssh test. The password is stored in
        memory and deleted when the execution is finished"""
    print_separator('-')
    logger.warning('Please enter your password for user {u}.\nThis is required'
                   ' by MTP tests to create an SSH connection from the device '
                   'to the host. Leave blank if SSH not required by '
                   'tests.'.format(u=getpass.getuser()))
    user_password = getpass.getpass('Password: ')
    return user_password


def generate_user_report(config_stack):
    """Create a human readable report from the subunit results."""
    artifacts_directory = results.get_artifacts_directory(config_stack)
    results.convert_subunit_results_to_pyunit(artifacts_directory)
    results.print_results(artifacts_directory)


if __name__ == '__main__':
    sys.exit(main())
