# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import address_book_app

from autopilot.exceptions import StateNotFoundError
from ubuntuuitoolkit import UbuntuUIToolkitCustomProxyObjectBase


class ABContactListPage(address_book_app.pages.ABContactListPage):

    def expand_contact(self, index):
        """Click the contact to show more information.

        :param index: The index of the contact to expand.

        """
        contact_delegate = self._get_contact_delegate(index)
        self.pointing_device.click_object(contact_delegate)
        contact_delegate.visible.wait_for(False)

    def open_contact_by_name(self, display_name):
        """
        Open the contact specified by display_name
        :param display_name: Name of contact to open
        :return: Page with contact information
        """
        return self.open_contact(self.get_contacts().index(display_name))


class ABContactViewPage(address_book_app.pages.ABContactViewPage):

    def get_contact_mobile_phone_number(self):
        """Return the first mobile phone listed for the contact"""
        mobile_fields = self.select_many('BasicFieldView',
                                         typeLabel='Mobile')
        if not mobile_fields:
            raise Exception('Mobile fields not found')

        return mobile_fields[0].\
            select_single('Label', objectName='label_phoneNumber_0.0').text

    def _get_contact_phone_number(self, phone_number):
        """
        Find the contact using phone number.

        :param phone_number: Phone number for required contact
        :return: ContactDetailPhoneNumberView matching search requirements
                 None if match is not found

        """
        contacts = self.select_many(ContactDetailPhoneNumberView, visible=True)
        for contact in contacts:
            try:
                # try to find matching phone number
                contact.select_single(
                    'Label', objectName='label_phoneNumber_0.0',
                    text=phone_number, visible=True)
            except StateNotFoundError:
                # move onto the next one
                pass
            else:
                # found the required contact
                return contact

        # No matching contact found
        raise Exception('Contact with phone number {n} not found'.format(
            n=phone_number))

    def call(self, phone_number):
        """
        Press the call button for the required phone number
        :param phone_number: The specified phone number to search for
        """
        phone_number = self._get_contact_phone_number(phone_number)
        phone_number.click_call_button()

    def message(self, phone_number):
        """
        Press the message button for the required phone number
        :param phone_number: The specified phone number to search for
        """
        phone_number = self._get_contact_phone_number(phone_number)
        phone_number.click_message_button()


class ContactDetailPhoneNumberView(UbuntuUIToolkitCustomProxyObjectBase):
    """Class to represent a phone number view with call and message buttons"""

    def _get_call_button(self):
        """Return the call button ActionButton object"""
        return self.select_single('ActionButton', iconName='call-start')

    def _get_message_button(self):
        """Return the message button ActionButton object"""
        return self.select_single('ActionButton', iconName='message')

    def click_call_button(self):
        """Click on the call button for this phone number"""
        self.pointing_device.click_object(self._get_call_button())

    def click_message_button(self):
        """Click on the message button for this phone number"""
        self.pointing_device.click_object(self._get_message_button())
