# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2014 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import configparser
import os
import subprocess
import systemimage.config as sys_info
import sys
import time

from autopilot import introspection
from autopilot.display import is_point_on_any_screen
from autopilot.exceptions import StateNotFoundError
from contextlib import contextmanager
from ubuntu_system_tests.helpers import wait_until
from ubuntu_system_tests.helpers import click

CO_ORD_MAX = (sys.maxsize, sys.maxsize)


@contextmanager
def override_proxy_timeout(proxy, timeout_seconds):
    original_timeout = proxy._poll_time
    try:
        proxy._poll_time = timeout_seconds
        yield proxy
    finally:
        proxy._poll_time = original_timeout


def get_proxy_object_for_existing_process(
        process_name, base_custom_proxy_object):
    """Return the proxy object for a currently running process."""
    return introspection.get_proxy_object_for_existing_process(
        pid=_get_pid(process_name),
        emulator_base=base_custom_proxy_object
    )


# Pulled from the UX tests
def _get_pid(process_name):
    """Returns pid for currently running browser process."""
    for i in range(10):
        try:
            return int(
                subprocess.check_output(
                    ['pidof', process_name]).strip()
            )
        except subprocess.CalledProcessError:
            time.sleep(1)
    else:
        raise RuntimeError(
            'Could not find autopilot interface for {} after 10 '
            'seconds'.format(process_name)
        )


def order_by_y_coord(object_list, include_off_screen=False):
    """
    Return an ordered list of objects ordered by y co-ordinate first
    and then x co-ordinate second if y co-ordinates match.

    :param object_list: List of objects containing globalRect property.
    :param include_off_screen: Whether to include objects that are not within
                               the bounds of any screen.
    :return: List of sorted objects.

    """
    return _order_by_key(object_list, _get_y_and_x, include_off_screen)


def order_by_x_coord(object_list, include_off_screen=False):
    """
    Return an ordered list of objects ordered by x co-ordinate first
    and then y co-ordinate second if x co-ordinates match.

    :param object_list: List of objects containing globalRect property.
    :param include_off_screen: Whether to include objects that are not within
                               the bounds of any screen.
    :return: List of sorted objects.

    """
    return _order_by_key(object_list, _get_x_and_y, include_off_screen)


def _order_by_key(object_list, sort_key, include_off_screen):
    """
    Return an ordered list of objects ordered by key.

    :param object_list: List of objects containing globalRect property.
    :param sort_key: Sorting key to use.
    :param include_off_screen: Whether to include objects that are not within
                               the bounds of any screen.
    :return: List of sorted objects.

    """
    objects = []
    for obj in object_list:
        try:
            # If obj is no longer valid this can cause excpetion
            point = obj.globalRect.x, obj.globalRect.y
            if include_off_screen:
                objects.append(obj)
            elif is_point_on_any_screen(point):
                objects.append(obj)
        except StateNotFoundError:
            pass
    return sorted(objects, key=sort_key)


def _get_y_and_x(item):
    """
    Return y and x co-ordinates for specified object.

    :param item: Item to check
    :return: (y, x) co-ordinates
    """
    try:
        # If item is no longer valid this can cause excpetion
        co_ords = item.globalRect.y, item.globalRect.x
    except StateNotFoundError:
        # Trying to sort an object that no longer exists
        # return a dummy key value so this item is sorted last
        co_ords = CO_ORD_MAX
    return co_ords


def _get_x_and_y(item):
    """
    Return x and y co-ordinates for specified object.

    :param item: Item to check
    :return: (x, y) co-ordinates
    """
    try:
        # If item is no longer valid this can cause excpetion
        co_ords = item.globalRect.x, item.globalRect.y
    except StateNotFoundError:
        # Trying to sort an object that no longer exists
        # return a dummy key value so this item is sorted last
        co_ords = CO_ORD_MAX
    return co_ords


"""

The following methods are for performing platform feature detection.

These methods may in future be moved into the autopilot.platform module.
Whilst the API will remain the same, the implementation of the methods may
change depening on the specific requirements of the supported devices.

"""


def get_device_name():
    """Return the device's model name.'"""
    return sys_info.config.device


def get_channel_name():
    """
    Return the build channel name. In case there is not channel defined, it
    returns the default value
    """
    file = '/etc/system-image/channel.ini'
    default = 'ubuntu-touch/rc-proposed/bq-aquaris.en'
    if not os.path.isfile(file):
        return default
    parser = configparser.ConfigParser()
    parser.read(file)
    return parser['service']['channel']


def is_agps_supported():
    """Return True if a agps is supported, False if it is not."""
    agps_supported = True
    try:
        subprocess.check_call(
            ['dpkg-query --status ubuntu-location-provider-here'],
            shell=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
    except subprocess.CalledProcessError:
        agps_supported = False
    return agps_supported


def is_camera_supported():
    """Return True if a camera is supported, False if it is not."""
    return click.is_package_installed('com.ubuntu.camera')


def is_wifi_supported():
    """Return True if wifi is supported, False if it is not."""
    wifi_supported = True
    try:
        subprocess.check_call(
            ['nmcli -t -f type device | grep wifi'],
            shell=True, stdout=subprocess.DEVNULL)
    except subprocess.CalledProcessError:
        wifi_supported = False
    return wifi_supported


def is_element(predicate, *args, **kwargs):
    """
    Evaluate the predicate with the args and Indicate it raises
    StateNotFoundError
     :param: predicate: The method to be evaluated
     :param: *args: The predicate positional parameters
     :param: **kwargs: The predicate optional parameters
     :return: True if the predicate raise StateNotFoundError, False otherwise
    """
    try:
        predicate(*args, **kwargs)
        return True
    except StateNotFoundError:
        return False


def wait_select_many(root, number=1, timeout=10, period=0.5, type_name='*',
                     **kwargs):
    """
    Get a list of proxy objects matching some search criteria from the
    introspection tree, with type equal to *type_name* and (optionally)
    matching the keyword filters present in *kwargs*. This method will retry
    until either the objects matching the criteria are >= number or the timeout
    is reached
    :param: root: The root object to make the query
    :param: number: The number of objects that have to match to return
    :param: timeout: The timeout for the polling
    :param: period: The retry period of time
    :param: type_name: Either a string naming the type you want, or a class
    of the appropriate type
    :param: **kwargs: The optional parameters used to match objects
    :raise: StateNotFoundError: When the number of objects matching the query
    are less than the number passed as parameter
    :return: A list of proxy objects
    """

    if wait_until(
            lambda: len(root.select_many(type_name, **kwargs)) >= number,
            timeout=timeout, period=period):
        return root.select_many(type_name, **kwargs)
    else:
        raise StateNotFoundError("Not found the number of elements requested")
