# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3,
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging

from autopilot import introspection, exceptions

from ubuntuuitoolkit import pickers
import ubuntuuitoolkit

logger = logging.getLogger(__name__)


class ClockApp(ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase):
    """Autopilot helper object for clock app."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        name = introspection.get_classname_from_path(path)
        return name == b'comubuntuclock'

    @property
    def main_view(self):
        return self.wait_select_single(MainView)


class MainView(ubuntuuitoolkit.MainView):

    @classmethod
    def get_type_query_name(cls):
        return 'MainView12'

    def get_main_page(self):
        """ Open and return the Clock Page """
        return self.wait_select_single(MainPage)

    def open_alarm(self):
        """ Open and return the Alarm Page """
        main_page = self.get_main_page()
        main_page.reveal_bottom_edge_page()
        self.get_header().visible.wait_for(True)
        return self.wait_select_single(AlarmPage)

    def click_add_alarm_button(self):
        """ Click the add alarm button and return the Edit Alarm Page """
        header = self.get_header()
        header.click_action_button('addAlarmAction')
        return self.wait_select_single(EditAlarmPage)

    def click_save_action(self):
        """ Click the save action header button """
        header = self.get_header()
        header.click_action_button('saveAction')

    def click_save_alarm(self):
        """ Click the save timer alarm action header button """
        header = self.get_header()
        header.click_action_button('saveAlarmAction')


class Page(ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase):
    """Autopilot helper for Pages."""

    @property
    def main_view(self):
        return self.get_root_instance().select_single(MainView)


class MainPage(Page):
    """ Autopilot helper for the Clock page """

    def reveal_bottom_edge_page(self):
        """ Bring the bottom edge page to the screen """
        self.bottomEdgePageLoaded.wait_for(True)
        try:
            action_item = self.wait_select_single(objectName='bottomEdgeTip')
            action_item.visible.wait_for(True)
            action_item.isAnimating.wait_for(False)
            start_x = (action_item.globalRect.x +
                       (action_item.globalRect.width * 0.5))
            start_y = (action_item.globalRect.y +
                       (action_item.height * 0.5))
            stop_y = start_y - (self.height * 0.7)
            self.pointing_device.drag(start_x, start_y,
                                      start_x, stop_y, rate=2)
            self.isReady.wait_for(True)
        except exceptions.StateNotFoundError:
            logger.error('BottomEdge element not found.')
            raise


class AlarmPage(Page):
    """Autopilot helper for the Alarm page."""

    def add_single_alarm(self, name, days, time_to_set, test_sound_name):
        """ Add a single type alarm
        :param name: name of alarm
        :param days: days on which the alarm should be triggered
        :param time_to_set: time to set alarm to
        :param test_sound_name: sound to set in alarm
        """
        # Set time
        edit_alarm_page = self.main_view.click_add_alarm_button()
        edit_alarm_page.set_alarm_time(time_to_set)

        # Set days
        alarm_repeat_page = edit_alarm_page.open_alarm_repeat_page()
        alarm_repeat_page.set_alarm_days(days)
        self.main_view.click_save_action()

        # Change Label
        alarm_label_page = edit_alarm_page.open_alarm_label_page()
        alarm_label_page.set_alarm_label(name)
        self.main_view.click_save_action()

        # Select sound
        alarm_sound_page = edit_alarm_page.open_alarm_sound_page()
        alarm_sound_page.set_alarm_sound(test_sound_name)
        self.main_view.click_save_action()

        # Save alarm
        self.main_view.click_save_alarm()


class EditAlarmPage(Page):
    """ Autopilot helper for the Add Alarm page """

    def set_alarm_time(self, time_to_set):
        """Set alarm time on datepicker.
        :param time_to_set: time to set on datepicker

        """
        hours_picker = self.wait_select_single(
            "Picker", objectName="PickerRow_HoursPicker")
        self._set_picker(hours_picker, 'time', time_to_set)

    def _set_picker(self, field, mode, value):
        # open picker
        self.pointing_device.click_object(field)

        # valid options are date or time; assume date if invalid/no option
        mode_value = 'Hours|Minutes'
        picker = self.wait_select_single(
            pickers.DatePicker, mode=mode_value, visible=True)
        picker.pick_time(value)

        # close picker
        self.pointing_device.click_object(field)

    def _open_alarm_page(self, obj):
        alarm_item = self.wait_select_single(
            "SubtitledListItem", objectName=obj)
        self.pointing_device.click_object(alarm_item)

    def open_alarm_repeat_page(self):
        """ Open the alarm repeat page """
        self._open_alarm_page("alarmRepeat")
        return self.main_view.wait_select_single(AlarmRepeat)

    def open_alarm_label_page(self):
        """ Open the alarm label page """
        self._open_alarm_page("alarmLabel")
        return self.main_view.wait_select_single(AlarmLabel)

    def open_alarm_sound_page(self):
        """ Open the alarm sound page """
        self._open_alarm_page("alarmSound")
        return self.main_view.wait_select_single(AlarmSound)


class AlarmRepeat(Page):
    """Autopilot helper for the  Alarm Repeat page."""

    def set_alarm_days(self, days):
        """
        :param days: array of days to select
        """
        self.unselect_selected_days()
        for day in days:
            obj_name = self.wait_select_single('Label', text=day).objectName
            self._select_single_alarm_day(obj_name.replace('alarmDay', ''))

    def _get_num_of_days(self):
        return int(self.wait_select_single(
            'QQuickRepeater', objectName='alarmDays').count)

    def _select_single_alarm_day(self, index):
        """ function for selecting the day passed to the function.
        :param index: the day to be selected
        """
        day_checkbox = self.wait_select_single(
            'CheckBox', objectName='daySwitch{}'.format(index))
        day_checkbox.check()

    def unselect_selected_days(self):
        """ Unselect the already selected days """
        for index in range(self._get_num_of_days()):
            day_checkbox = self.wait_select_single(
                'CheckBox', objectName='daySwitch{}'.format(index))
            day_checkbox.uncheck()


class AlarmSound(Page):
    """ Autopilot helper for the  Alarm Sound page. """

    def set_alarm_sound(self, test_sound_name):
        """Set alarm sound.
        :param test_sound_name: sound to set for alarm
        """
        for index in range(self._get_num_of_sounds()):
            if self.wait_select_single(
                    'Label', objectName='soundName{}'.format(index)).\
                    text == test_sound_name:
                self._select_alarm_sound(index)
                break

    def _get_num_of_sounds(self):
        return int(self.wait_select_single(
            'QQuickRepeater', objectName='alarmSounds').count)

    def _select_alarm_sound(self, index):
        """ function for selecting the sound passed to the function.
        :param index: the sound to be selected
        """
        sound_checkbox = self.wait_select_single(
            'CheckBox', objectName='soundStatus{}'.format(index))
        sound_checkbox.check()


class AlarmLabel(Page):
    """Autopilot helper for the  Alarm Label page."""

    def set_alarm_label(self, name):
        """ Set alarm label.
        :param name: label for alarm to set
        """
        alarm_textfield = self.wait_select_single("TextField",
                                                  objectName='labelEntry')
        alarm_textfield.visible.wait_for(True)
        alarm_textfield.write(name)
