# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import threading

from dbus.mainloop.glib import DBusGMainLoop
from gi.repository import GLib


class DBusObserver():
    """Base class for a DBus observer object."""

    def __init__(self, stop, result):
        """
        Construct the Observer object.
        :param stop: Event object to signal when to stop waiting.
        :param result: Event object to signal when result is matched.

        """
        self._stop = stop
        self.result = result
        DBusGMainLoop(set_as_default=True)
        self._mainloop = GLib.MainLoop()

    def start(self):
        """
        Start the observer by starting the callback timer and the glib
        mainloop.

        """
        self._stop.clear()
        self.result.clear()
        self.start_callback_timer()
        self._mainloop.run()

    def stop(self):
        """Stop the glib mainloop to stop observing."""
        self._cleanup()
        self._mainloop.quit()

    def _stop_if_signalled(self):
        """
        Timer callback event. If _stop event is set then stop observing,
        otherwise continue to wait by starting a new callback timer.

        """
        if self._stop.is_set():
            self.stop()
        else:
            self.start_callback_timer()

    def _cleanup(self):
        """
        Execute cleanup actions before the observer is stopped.

        This should be over-ridden in a derived class where specific cleanup
        actions are required. In this class the function is empty.

        """
        pass

    def start_callback_timer(self):
        """
        Start a new callback timer which is used to check whether to stop
        or continue waiting.

        """
        timer = threading.Timer(0.5, self._stop_if_signalled)
        timer.start()
