
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging

from autopilot import introspection, logging as autopilot_logging
import ubuntuuitoolkit as uuitk
from unity8 import indicators

from ubuntu_system_tests.helpers.autopilot import wait_select_many
from ubuntu_system_tests.helpers.ubuntuuitoolkit._qquicklistview import QQuickListView  # NOQA
from ubuntu_system_tests.helpers.ubuntuuitoolkit.switchmenu import SwitchMenu
from ubuntu_system_tests.helpers.ubuntuuitoolkit import slider

SOUND_INDICATOR = 'indicator-sound'
SOUND_INDICATOR_PAGE = 'indicator-sound-page'

logger = logging.getLogger(__name__)


class IndicatorPage(indicators.IndicatorPage):
    """Extend IndicatorPage from Unity8."""

    def _get_listview(self):
        """Return the list view object from the indicator."""
        return self.select_single(QQuickListView, objectName='mainMenu')

    def press_label(self, text):
        """Press the label in the list view of the indicator."""
        self._get_listview().press_label(text)


class NotificationsIndicatorItem(uuitk.UbuntuUIToolkitCustomProxyObjectBase):
    """Class to represent 'Notifications' indicator shown in top panel."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        name = introspection.get_classname_from_path(path)
        return (name == b'IndicatorItem' and
                state['objectName'][1] == 'indicator-messages-panelItem')

    def is_new_message_displayed(self):
        return 'indicator-messages-new' in self.icons[0]


class ModemInfoItem(uuitk.UbuntuUIToolkitCustomProxyObjectBase):

    def is_offline(self):
        return self.statusText == 'Offline'

    def is_inserted(self):
        """Return False if SIM ModemInfoItem reads 'No SIM', True otherwise.

        The SIM list is never empty--a dummy ModemInfoItem with text
        'No SIM' appears if no SIMs are inserted.  This function may
        be used for counting of *actual* SIMs, e.g.

        """
        return self.statusText != 'No SIM'

    def get_sim_name(self):
        """Returns the name of the SIM, is mostly the provider name."""
        return self.statusText


class Indicator:

    def __init__(self, main_window, name, page):
        super().__init__()
        self._main_window = main_window
        self.name = name
        self.page = page

    def open(self):
        """Open the requested indicator page.

        :return: The custom proxy object for the requested indicator page.

        """
        return self._main_window.open_indicator_page(self.name)

    def close(self):
        """Close the indicator page."""
        self._main_window.close_indicator_page()

    def ensure_closed(self):
        """Ensure the indicator page is closed."""
        indicator_page = self._main_window.select_single(objectName=self.page)
        if indicator_page.visible:
            self.close()


class MessageIndicator(Indicator):
    """ Autopilot helper for the message indicator """

    def get_last_message(self, number):
        """ Get the last message displayed in the indicator """
        return wait_select_many(self._main_window, number,
                                objectName='title')[-1].text

    def wait_while_indicator_visible(self):
        """ Wait until the indicator message is not displayed any more """
        messages = self._main_window.select_many(objectName='title')
        for message in messages:
            if not message.truncated:
                message.truncated.wait_for(True)


class NetworkIndicatorPage(IndicatorPage):

    """Autopilot helper for the network indicator page."""
    @classmethod
    def validate_dbus_object(cls, path, state):
        name = introspection.get_classname_from_path(path)
        return (name == b'IndicatorPage' and
                state['objectName'][1] == 'indicator-network-page')

    def is_flight_mode_enabled(self):
        """Return True if flight mode is on, False otherwise."""
        return self._get_flight_mode_switch().checked

    def is_flight_mode_switch_visible(self):
        """Return True if flight mode switch is visible, False otherwise."""
        return self._get_flight_mode_switch().visible

    def _get_flight_mode_switch(self):
        return self.select_single(
            'SwitchMenu',
            objectName='indicator.airplane.enabled')

    def turn_on_flight_mode(self):
        """Actuate the flight mode switch to turn on as necessary."""
        if not self.is_flight_mode_enabled():
            self.pointing_device.click_object(
                self._get_flight_mode_switch())

    def turn_off_flight_mode(self):
        """Actuate the flight mode switch to turn off as necessary."""
        if self.is_flight_mode_enabled():
            self.pointing_device.click_object(
                self._get_flight_mode_switch())

    def is_wifi_enabled(self):
        """Return True if WiFi is enabled, False otherwise."""
        return self._get_wifi_switch().checked

    def is_wifi_switch_visible(self):
        """Return True if WiFi switch is visible, False otherwise."""
        return self._get_wifi_switch().visible

    def _get_wifi_switch(self):
        """Return the WiFi switch."""
        return self.select_single(
            'SwitchMenu',
            objectName='indicator.wifi.enable')

    def turn_on_wifi(self):
        """Actuate the WiFi switch to turn on as necessary."""
        if not self.is_wifi_enabled():
            self.pointing_device.click_object(
                self._get_wifi_switch())

    def turn_off_wifi(self):
        """Actuate the WiFi switch to turn off as necessary."""
        if self.is_wifi_enabled():
            self.pointing_device.click_object(
                self._get_wifi_switch())

    def _get_cellular_settings_menu_item(self):
        """Return the menu item which opens cellular settings."""
        return self.select_single(
            'StandardMenu',
            objectName='indicator.cellular.settings')

    def is_cellular_settings_visible(self):
        return self._get_cellular_settings_menu_item().visible

    def open_cellular_settings(self):
        """Tap the cellular settings list item to open settings."""
        self.pointing_device.click_object(
            self._get_cellular_settings_menu_item())

    def _get_wifi_settings_menu_item(self):
        """Return the menu item which opens WiFi settings."""
        return self.select_single(
            'StandardMenu',
            objectName='indicator.wifi.settings')

    def is_wifi_settings_visible(self):
        return self._get_wifi_settings_menu_item().visible

    def open_wifi_settings(self):
        """Tap the wifi settings list item to open settings."""
        self.pointing_device.click_object(self._get_wifi_settings_menu_item())

    def get_list_of_sims(self):
        """Return a list of ModemInfoItems, or empty list if none found.

        Note that we don't count the 'No SIM' item which appears if no
        SIM is found.

        """
        return sorted(
            [sim for sim in self.select_many('ModemInfoItem')
             if sim.is_inserted()],
            key=lambda x: x.globalRect.y
        )

    def is_sim_list_offline(self):
        """Return True if all SIMs are offline, False otherwise."""
        return all([sim.is_offline() for sim in self.get_list_of_sims()])

    def get_list_of_wifi_access_points(self):
        """Return a list of WiFi access points, empty list if none found."""
        return self.select_many('AccessPointMenu')


class NotificationsIndicatorPage(IndicatorPage):
    """Autopilot helper for the messages indicator page."""
    @classmethod
    def validate_dbus_object(cls, path, state):
        name = introspection.get_classname_from_path(path)
        return (name == b'IndicatorPage' and
                state['objectName'][1] == 'indicator-messages-page')

    def get_message(self, sender, **kwargs):
        """Return the message with specified sender and arguments."""
        return self.select_single(
            MessageHeader, title=sender, visible=True, **kwargs)

    def clear_all(self):
        """Press the 'Clear All' label to clear all existing notifications."""
        self.press_label(text='Clear All')

    def _get_reply_input_field(self):
        """Return the sms reply entry."""
        return self.select_single('TextArea', objectName='replyText')

    def _type_message(self, message):
        """Focus the reply entry and type the reply message."""
        reply_field = self._get_reply_input_field()
        reply_field.write(message)

    def _get_send_button(self):
        """Return the reply button."""
        return self.select_single('Button', objectName='sendButton')

    def reply_sms(self, message_body, message_item):
        """
        Expand the message item in the indicator, type reply message and
        send.
        :param message_body: the message text to send
        :param message_item: the message object for the message to reply to.
        """
        self.pointing_device.click_object(message_item)
        self._type_message(message_body)
        self.pointing_device.click_object(self._get_send_button())
        message_item.wait_until_destroyed()


class MessageHeader(uuitk.UbuntuUIToolkitCustomProxyObjectBase):
    """Autopilot helper for message object displayed in indicator."""

    def get_call_icon(self):
        """Return the call icon object."""
        return self.select_single(
            'Icon', objectName='icon',
            source='image://theme/telephony-service-call')

    def get_contact_avatar(self):
        """Return the contact's avatar icon."""
        return self.select_single('Icon', objectName='avatar', visible=True)

    @autopilot_logging.log_action(logger.info)
    def call(self):
        """Press the call icon."""
        self.pointing_device.click_object(self.get_call_icon())

    @autopilot_logging.log_action(logger.info)
    def press_avatar(self):
        """Press the contact's avatar to display snap decision menu."""
        self.pointing_device.click_object(self.get_contact_avatar())


class BluetoothIndicatorPage(IndicatorPage):
    """Autopilot helper for the bluetooth indicator page."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        name = introspection.get_classname_from_path(path)
        return (name == b'IndicatorPage' and
                state['objectName'][1] == 'indicator-bluetooth-page')

    def _get_bluetooth_switch(self):
        """Return the bluetooth switch."""
        return self.select_single(
            'SwitchMenu',
            objectName='indicator.bluetooth-enabled')

    def is_bluetooth_switch_checked(self):
        """Return true if bluetooth switch is checked, otherwise return
        false."""
        return self._get_bluetooth_switch().checked

    def enable_bluetooth(self):
        """Tap on the bluetooth checkbox to turn it on."""
        if not self.is_bluetooth_switch_checked():
            self.pointing_device.click_object(self._get_bluetooth_switch())

    def disable_bluetooth(self):
        """Tap on the bluetooth checkbox to turn it off."""
        if self.is_bluetooth_switch_checked():
            self.pointing_device.click_object(self._get_bluetooth_switch())

    def _get_bluetooth_settings_menu_item(self):
        """Return the bluetooth settings list item."""
        return self.select_single('StandardMenu',
                                  objectName='indicator.phone-show-settings')

    def open_bluetooth_settings(self):
        """Tap the bluetooth settings list item to open settings."""
        self.pointing_device.click_object(
            self._get_bluetooth_settings_menu_item())


class DateTimeIndicatorPage(IndicatorPage):
    """Autopilot helper for the date time indicator page."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        name = introspection.get_classname_from_path(path)
        return (name == b'IndicatorPage' and
                state['objectName'][1] == 'indicator-datetime-page')

    def _get_datetime_settings_menu_item(self):
        """Return the date time settings list item."""
        return self.select_single(
            'StandardMenu',
            objectName='indicator.phone.open-settings-app')

    def open_date_time_settings(self):
        """Tap the Date time settings list item to open settings."""
        self.pointing_device.click_object(
            self._get_datetime_settings_menu_item())


class SoundIndicatorPage(IndicatorPage):
    """Autopilot helper for the sound indicator page."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        name = introspection.get_classname_from_path(path)
        return (name == b'IndicatorPage' and
                state['objectName'][1] == 'indicator-sound-page')

    def _get_sound_settings_menu_item(self):
        """Return the sound settings list item."""
        return self.select_single(
            'StandardMenu',
            objectName='indicator.phone-settings')

    def get_silent_mode_switcher(self):
        menu = self.select_single(
            SwitchMenu, objectName='indicator.silent-mode', visible=True)
        return menu.get_checkbox()

    def toggle_silent_mode_state(self):
        """Change the silent mode setting from it's current state and return
        its new checked state."""
        switcher = self.get_silent_mode_switcher()
        switcher.change_state()
        return switcher.checked

    def enable_silent_mode(self):
        """Enable silent mode if it is not already enabled."""
        switcher = self.get_silent_mode_switcher()
        if not switcher.checked:
            switcher.change_state()

    def disable_silent_mode(self):
        """Disable silent mode if it is already enabled."""
        switcher = self.get_silent_mode_switcher()
        if switcher.checked:
            switcher.change_state()

    def open_sound_settings(self):
        """Tap the sound settings list item to open settings."""
        self.pointing_device.click_object(self._get_sound_settings_menu_item())


class BatteryIndicatorPage(IndicatorPage):
    """Autopilot helper for the battery indicator page."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        name = introspection.get_classname_from_path(path)
        return (name == b'IndicatorPage' and
                state['objectName'][1] == 'indicator-power-page')

    def _get_battery_settings_menu_item(self):
        """Return the battery settings list item."""
        return self.select_single(
            'StandardMenu',
            objectName='indicator.activate-phone-settings')

    def open_battery_settings(self):
        """Tap the battery settings menu item to open settings."""
        self.pointing_device.click_object(
            self._get_battery_settings_menu_item())

    def get_brightness_slider_widget(self):
        """Return the slider widget on the battery indicator page."""
        return self.select_single(slider.Slider, objectName='slider')

    def get_current_brightness(self):
        return float(self.get_brightness_slider_widget().value)

    def _get_brightness_switch(self):
        """Return the auto brightness switch."""
        return self.select_single('SwitchMenu',
                                  objectName='indicator.auto-brightness')

    def enable_auto_brightness(self):
        """Enable auto brightness."""
        if not self._get_brightness_switch().checked:
            self.pointing_device.click_object(self._get_brightness_switch())

    def disable_auto_brightness(self):
        """Disable auto brightness."""
        if self._get_brightness_switch().checked:
            self.pointing_device.click_object(self._get_brightness_switch())
