# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import dbus

from enum import IntEnum

from ubuntu_system_tests.helpers.dbus_observer import DBusObserver
from ubuntu_system_tests.helpers import processes
from ubuntu_system_tests.helpers.threads import StoppingThread

APP = 'mediaplayer-app'


class PlaybackStatus(IntEnum):
    """Media player playback states."""
    Null = 0
    Ready = 1
    Playing = 2
    Paused = 3
    Stopped = 4


class MediaPlayerObserver(DBusObserver):
    """Class to observe the state of the media player."""

    def __init__(self, stop, result):
        """
        Construct the MediaPlayerObserver object.
        :param stop: Event object to signal when to stop waiting.
        :param result: Event object to signal when a track is added.

        """
        super().__init__(stop, result)
        self.bus = dbus.SessionBus()
        self.bus.add_signal_receiver(
            self.on_playback_status_changed,
            dbus_interface='org.mpris.MediaPlayer2.Player',
            signal_name='PlaybackStatusChanged')

    def _cleanup(self):
        """Execute cleanup actions before the observer stops."""
        self.bus.remove_signal_receiver(self.on_playback_status_changed)

    def on_playback_status_changed(self, status):
        """
        Callback for when media player playback status is changed.
        If new state is Playing then set the result event to signal this event
        has occured.

        :param status: Int status value.

        """
        if status == PlaybackStatus.Playing:
            self.result.set()


def is_mediaplayer_running():
    """ Indicate if the mediaplayer is running """
    return processes.is_process_running(APP)


def _start_media_player_observer(stop, result):
    """
    Start a media player observer.

    :param stop: Event object to signal when to stop observing.
    :param result: Event object to signal when playback state changes to
                   Playing.

    """
    observer = MediaPlayerObserver(stop, result)
    observer.start()


def start_media_player_observer(result):
    """
    Listen for when media player changes state.

    :param result: Event object to indicate if state has changed to Playing.
    :return: StoppingThread object which should be stopped by caller when
             required.

    """
    launch_thread = StoppingThread(
        target=_start_media_player_observer, args=[result])
    launch_thread.start()
    return launch_thread
