# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging
import time

import autopilot
import ubuntuuitoolkit as uitk

from autopilot.exceptions import StateNotFoundError
from urllib.parse import urlparse
from messaging_app import emulators as messaging_emulators

logger = logging.getLogger(__name__)


class MainView(messaging_emulators.MainView):

    @autopilot.logging.log_action(logger.info)
    def type_contact_name_and_select_num(self, name, number):
        self.type_contact_name(name)
        # Wait for the list to settle
        time.sleep(1)
        # Now select the right number from the list
        self.pointing_device.click_object(
            self.get_mobile_number_label(number)
        )

    def type_contact_name(self, name):
        text_entry = self.get_newmessage_multirecipientinput()
        text_entry.enter_text(name)

    def get_mobile_number_label(self, number):
        return self.wait_select_single(
            'Label', text='Mobile {}'.format(number)
        )

    @autopilot.logging.log_action(logger.info)
    def select_sim(self, sim_number):
        """Select default sim for messages, expecting dialog to be shown."""
        self.wait_select_single('Dialog')
        self.pointing_device.click_object(self.get_sim_label(int(sim_number)))

    @autopilot.logging.log_action(logger.info)
    def detect_select_sim_prompt(self):
        """Detect if the select SIM dialog is shown."""
        try:
            self.wait_select_single('Dialog')
            return True
        except StateNotFoundError:
            return False

    # TODO: Remove this when https://bugs.launchpad.net/bugs/1482205
    # is fixed.
    def get_send_button(self):
        """Return Button with text Send"""
        return self.get_messages_page().select_single(
            'Icon', objectName='sendButton'
        )

    def get_sim_label(self, sim_number):
        return self.wait_select_single(
            'Label', text='SIM {}'.format(sim_number)
        )

    def get_thread_list(self):
        return self.select_single(objectName='threadList')

    def _get_attachment_button(self):
        """Returns the 'attach media' button."""
        return self.get_messages_page().select_single(
            'Icon', name='camera-app-symbolic', visible=True)

    def click_attach_button(self):
        self.pointing_device.click_object(self._get_attachment_button())
        return self._get_content_picker()

    def _get_content_picker(self):
        return self.wait_select_single(ContentPeerPicker10, visible=True)

    def type_message(self, message):
        """Select and type message in new message text area in messages page

        :parameter message: the message to type
        """
        text_entry = self.get_newmessage_textarea()
        text_entry.write(message)
        if text_entry.text != message:
            self.logger.info('typed: "{}" expected: "{}"'.format(
                text_entry.text, message
            ))

    def _get_thread_delegates(self):
        """Get all thread delegates in main screen."""
        return self.get_thread_list().select_many('ThreadDelegate')

    def get_visible_threads_count(self):
        """Return the number of visible threads."""
        return len([t for t in self._get_thread_delegates() if t.height != 0])

    def _is_search_mode(self):
        """Return bool indicating if the App header is in search mode."""
        return self.get_main_page().state == 'search'

    def _get_search_box(self):
        """Return the message searchbox."""
        """Return the contacts search box."""
        return self.select_single(uitk.TextField, objectName='searchField')

    def search(self, text):
        """Tap the search icon and write the requestion text."""
        if not self._is_search_mode():
            self.click_header_action('searchAction')
        search_box = self._get_search_box()
        search_box.clear()
        search_box.write(text)

    def clear_search(self):
        """Clear the search box."""
        if self._is_search_mode():
            self._get_search_box().clear()

    def is_thread_for_number_visible(self, number):
        """
        Return bool indicating the thread for `number` is visible on screen.
        """
        thread = self.get_thread_from_number(number)
        return thread.height > 0


class MultiRecipientInput(uitk.UbuntuUIToolkitCustomProxyObjectBase):

    def enter_text(self, text):
        contact_search_input = self.select_single(
            'TextField', objectName='contactSearchInput'
        )
        contact_search_input.write(text)


class MessageDelegateFactory(messaging_emulators.MessageDelegateFactory):
    """Class to get all the messages from the app."""

    def get_mms_content_file_path(self):
        """Return MMSBase for MMS message."""
        file_url = self.select_single(MMSBase).get_media_file_url()
        return urlparse(file_url).path

    def get_message_bubble(self):
        return self.wait_select_single(MessageBubble, visible=True)


class MessageBubble(uitk.UbuntuUIToolkitCustomProxyObjectBase):
    """Class to represent the message bubble for a displayed message."""

    def wait_until_sent(self, timeout):
        sent_time = self.select_single('Label', objectName='messageDate')
        sent_time.visible.wait_for(True, timeout=timeout)


class MMSBase(uitk.UbuntuUIToolkitCustomProxyObjectBase):
    """Class to examine MMS content."""

    def get_media_file_url(self):
        """Return the file url of the media content."""
        image = self.get_media_container().select_single('QQuickImage')
        return image.get_properties()['source']

    def get_media_container(self):
        """Return container object for media content."""
        return self.select_single('UCUbuntuShape', visible='True')


class Messages(messaging_emulators.Messages):
    """Class to examine content of messages displayed in editor."""

    def get_attachments(self):
        """Return list of displayed attachment objects."""
        attachments = []
        for shape in self.select_many('UCUbuntuShape', visible=True):
            if 'filePath' in shape.get_properties():
                if len(shape.filePath) > 0:
                    attachments.append(shape)
        return attachments

    def get_attachment_count(self):
        """Return a count of the displayed attachments in message editor."""
        return len(self.get_attachments())

    def get_matching_messages(self, text, sender):
        """Return matching messages for specified sender in thread."""
        return self.select_many(
            'RegularMessageDelegate', textMessage=text, senderId=sender,
            visible=True)

    def is_message_displayed(self, text, phone_number):
        """
        Check if message with phone_number and text is displayed in thread list

        :param text: The body of the message we are expecting
        :param number: The sending phone number
        :return: True if received, otherwise False

        """
        return bool(self.get_matching_messages(text, phone_number))

    def how_many_messages_displayed(self, text, phone_number):
        """
        Count how many messages are displayed with specified text and
        phone number in thread list.

        :param text: The body of the message we are expecting
        :param number: The sending phone number
        :return: Number of matching messages

        """
        return len(self.get_matching_messages(text, phone_number))


class ContentPeerPicker10(uitk.UbuntuUIToolkitCustomProxyObjectBase):
    """Class to pick a content source application."""

    def pick_app(self, app_name):
        """
        Taps the requested app's icon from the content picker screen
        :param app_name: name of the app to tap

        """
        app_icon = self.wait_select_single('Label', text=app_name,
                                           visible=True).get_parent()
        self.pointing_device.click_object(app_icon)
