# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import configparser
import fixtures
import os
import shutil
import subprocess

from messaging_app import fixture_setup

from ubuntu_system_tests.helpers import file_system as fs
from ubuntu_system_tests.helpers.system_settings import (
    get_default_sim_for_messages, set_default_sim_for_messages)

MESSAGING_APP_CONF = 'com.ubuntu.messaging-app/MessagingApp.conf'

KEY_GENERAL = 'General'
KEY_HINT = 'hintNecessary'
FALSE = 'false'
TRUE = 'true'


class MessagingTestEnvironment(fixtures.Fixture):
    """Fixture used to clean up the phone and messaging environment"""

    def setUp(self):
        super(MessagingTestEnvironment, self).setUp()
        self.useFixture(RespawnMessagingApp())
        self.useFixture(UseEmptySmsHistory())
        self.useFixture(fixture_setup.RespawnService())
        self.useFixture(DisableSMSGestureHint())
        self.useFixture(SetDefaultSIM())


class DisableSMSGestureHint(fixtures.Fixture):
    """Fixture used to disable SMS gesture hint"""

    CONFIG_FILE = os.path.join(fs.DIR_HOME_CONFIG, MESSAGING_APP_CONF)

    def setUp(self):
        super().setUp()
        original_value = self.get_hint_value()
        self.addCleanup(self.set_hint_value, original_value)
        self.set_hint_value(FALSE)

    def get_hint_value(self):
        try:
            hint = self._get_config()[KEY_GENERAL][KEY_HINT]
        except KeyError:
            hint = TRUE
        return hint

    def set_hint_value(self, value):
        config = self._get_config()
        if KEY_GENERAL not in config:
            config.add_section(KEY_GENERAL)
        config[KEY_GENERAL][KEY_HINT] = value
        dir_path = os.path.dirname(self.CONFIG_FILE)
        if not os.path.exists(dir_path):
            os.makedirs(dir_path)
        with open(self.CONFIG_FILE, 'w') as configfile:
            config.write(configfile)

    def _get_config(self):
        config = configparser.ConfigParser(strict=False)
        # tell it preserve case of key names
        config.optionxform = str
        config.read(self.CONFIG_FILE)
        return config


class SetDefaultSIM(fixtures.Fixture):
    """Fixture to set the default sim for messaging as sim 1"""

    def setUp(self):
        super().setUp()
        original_value = get_default_sim_for_messages()
        self.addCleanup(set_default_sim_for_messages, original_value)
        set_default_sim_for_messages('1')


class UseEmptySmsHistory(fixture_setup.FillCustomSmsHistory):
    """Fixture used to make the history service run with a clean data db"""
    def setUp(self):
        super(UseEmptySmsHistory, self).setUp()

    def _prepare_history_data(self):
        pass


class RespawnMessagingApp(fixtures.Fixture):
    """ As the Messaging app is not stopped properly by using the
    ubuntu-app-stop command, this fixture ensures the app will be stopped at
    the end of the test
    """
    app_name = 'messaging-app'

    def setUp(self):
        super(RespawnMessagingApp, self).setUp()
        self.addCleanup(self._kill_app_to_respawn)
        self._kill_app_to_respawn()

    def _kill_app_to_respawn(self):
        """ This method is used to kill either a process or an app which is not
        stopped properly by using ubuntu-app-stop"""
        subprocess.call(['pkill', '-f', self.app_name])


class FillCustomSmsHistory(fixtures.Fixture):

    history_db = "history.sqlite"
    prefilled_history = os.path.join(fs.DIR_TEST_DATA_DATABASES, history_db)
    database_path = '/tmp/' + history_db

    def setUp(self):
        super().setUp()
        self.addCleanup(self._clear_test_data)
        self.addCleanup(self._kill_service_to_respawn)
        self._clear_test_data()
        self._prepare_history_data()
        self._kill_service_to_respawn()
        self._start_service_with_custom_data()

    def _prepare_history_data(self):
        if os.path.exists(self.prefilled_history):
            shutil.copy(self.prefilled_history, self.database_path)

    def _clear_test_data(self):
        if os.path.exists(self.database_path):
            os.remove(self.database_path)

    def _kill_service_to_respawn(self):
        subprocess.call(['pkill', 'history-daemon'])

    def _start_service_with_custom_data(self):
        os.environ['HISTORY_SQLITE_DBPATH'] = self.database_path
        with open(os.devnull, 'w') as devnull:
            subprocess.Popen(['history-daemon'], stderr=devnull)
