# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import json
import os
import sys

from autopilot import exceptions
from collections import namedtuple
import ubuntuuitoolkit

from ubuntu_system_tests.helpers import autopilot
from ubuntu_system_tests.helpers import file_system
from ubuntu_system_tests.helpers import timedate
from ubuntu_system_tests.helpers.autopilot import order_by_y_coord
from ubuntu_system_tests.helpers.ubuntuuitoolkit.pageheader import PageHeader
from ubuntu_system_tests.helpers.unity8.dash import get_dash
from ubuntu_system_tests.helpers import url_dispatcher

SCOPES_FILE = os.path.join(file_system.DIR_TEST_DATA_SCOPES, 'scopes.json')
KEY_DEFAULT_CONDITION = 'default_condition'
KEY_PARENT_ID = 'parent_id'
KEY_SECTIONS = 'sections'
KEY_CONDITION = 'condition'
KEY_FORMAT_METHODS = 'format_methods'
CONDITION_DEFAULT = 'none'
SECTION_HEADER = 'header'
SECTION_LABEL = 'label'
SECTION_IMAGE = 'image'

THIS_MODULE = sys.modules[__name__]

SCOPE_ID_TODAY = 'today'
SCOPE_ID_NEARBY = 'nearby'
SCOPE_ID_APPS = 'apps'
SCOPE_ID_NEWS = 'news'
SCOPE_ID_PHOTOS = 'photos'
SCOPE_ID_MUSIC = 'music'
SCOPE_ID_VIDEO = 'video'

ScopeDataItem = namedtuple('ScopeDataItem', ['data', 'globalRect'])

channel = autopilot.get_channel_name()
device = autopilot.get_device_name()
scopes_list = None


class Scope(object):

    """
    Scope base class object.

    This class should be derived from GenericScopeView. However, because of
    Autopilot CPO inheritance problems this is not reliable currently.

    So this class is used as temporary solution.

    """

    def __init__(self, scope):
        self.scope = scope

    def _get_data_from_object(self, object):
        """
        Extract the required data from object.

        For 'Label' object this is 'text' property.
        For 'QQuickImage' object this is 'source' property.

        :param object: The object to examine
        :return: The extracted data

        """
        if 'text' in object.get_properties():
            data = object.text
        else:
            data = object.source
        return data

    def _add_sorted_new_items_to_existing(self, new_items, existing_items):
        """
        Sort the new items in order of y-coord and add to
        existing_items list.

        """
        for item in order_by_y_coord(new_items):
            existing_items.append(item.data)
        return existing_items

    def _get_content_of_types(self, types, existing_items):
        """
        Extract the required types from the scope. Add all new items to the
        existing_items list and return it.

        :param types: A list of object types to look for.
        :param existing_items: A list of ScopeDataItems. This represents any
                              items that have already been found. No duplicate
                              items will be added to the list.
        :return: List of ScopeDataItems including any new items found.

        """
        # find any new items not already in existing_items
        new_items = []
        for type in types:
            for item in self.scope.select_many(type, visible=True):
                item_data = self._get_data_from_object(item)
                if item_data and item_data not in existing_items:
                    new_items.append(ScopeDataItem(item_data, item.globalRect))

        return self._add_sorted_new_items_to_existing(
            new_items, existing_items)

    def _get_content_labels(self, existing_items, include_images):
        """Return list of data items from all Label items displayed."""
        types = ['Label']
        if include_images:
            types.append('QQuickImage')
        return self._get_content_of_types(types, existing_items)

    def _get_content_headers(self, existing_items):
        """Return list of data items from all Header items displayed."""
        return self._get_content_of_types(['Header'], existing_items)

    def _get_listview(self):
        return self.scope.select_single(
            'ListViewWithPageHeader', objectName='categoryListView')

    def get_all_content(self, include_images, scroll_to_end=True):
        """
        Return all data content from the current scope, including Headers,
        Labels and optionally QQuickImages. The data items returned will be
        orderd according to the order that they appear on the screen
        veritcally.

        :param include_images: Whether to include image sources in the results
        :param scroll_to_end: Whether to scroll to the end of the scope to
                              find the results.
        :return: List of data items ordered by y co-ordinate.

        """
        list_view = self._get_listview()
        all_headers = []
        all_labels = []
        at_bottom = False
        while not at_bottom:
            all_headers = self._get_content_headers(all_headers)
            all_labels = self._get_content_labels(all_labels, include_images)
            if not list_view.atYEnd and scroll_to_end:
                list_view.swipe_to_show_more_below()
            else:
                at_bottom = True
        self.swipe_quickly_to_top()
        return all_headers, all_labels

    def get_title(self):
        """Return title of current scope."""
        return self.scope.select_single(objectName="scopePageHeader").title

    def refresh(self):
        """Swipe to the top to the middle to refresh the scope."""
        list_view = self._get_listview()
        list_view.swipe_to_top()
        list_view._swipe_to_middle()

    def swipe_quickly_to_top(self):
        """Swipe the scope quickly to the top."""
        list_view = self._get_listview()
        start_x = list_view.globalRect.x + (list_view.globalRect.width // 2)
        stop_x = start_x
        from ubuntuuitoolkit._custom_proxy_objects import _flickable
        containers = list_view._get_containers()
        start_y = _flickable._get_visible_container_top(containers) + 5
        stop_y = _flickable._get_visible_container_bottom(containers) - 50
        while not list_view.atYBeginning:
            list_view._slow_drag(start_x, stop_x, start_y, stop_y, rate=20)
            list_view.dragging.wait_for(False)
            list_view.moving.wait_for(False)

    def _get_setup_skip_button(self):
        try:
            section = self.scope.select_single(
                'CardVerticalJournal', objectName='skip')
            return section.select_single(
                'AbstractButton', objectName='delegate0', visible=True)
        except exceptions.StateNotFoundError:
            return None

    def _press_skip_setup_button(self):
        """Press the skip setup button and wait for the header animation to
        stop moving.
        """
        self.scope.pointing_device.click_object(self._get_setup_skip_button())
        self.wait_for_header_to_stop_moving()

    def _get_first_visible_object(self, objects):
        """ Retrieve the object that is currently being displayed based on its
        position
        :param objects: the list of objects to compare
        :return: First visible item from list

        """
        return order_by_y_coord(objects)[0]

    def has_skip_button(self):
        """ Indicates if the skip button is present"""
        return self._get_setup_skip_button() is not None

    def dismiss_setup(self):
        """
        Skip the accounts setup by clicking in the button 'Skip & set up
        later' just the first time.
        This method should be used just for the scopes: Today and Photos
        """
        self.scroll_to_bottom()
        self._press_skip_setup_button()
        get_dash().get_current_page_header().go_back()

    def scroll_to_bottom(self):
        """ Scroll to the bottom of the scope """
        list_view = self._get_listview()
        list_view.swipe_to_bottom()

    def get_search_button(self):
        return self.scope.select_single(objectName='search_header_button')

    def get_search_text_field(self):
        return self.scope.select_single(objectName='searchTextField')

    def search(self, text):
        """ Search for a specific text in the scope and waits until the results
        are displayed
        :param text: The text to search
        """
        self.scope.pointing_device.click_object(self.get_search_button())
        self.wait_for_header_to_stop_moving()

        self.get_search_text_field().write(text)
        self.wait_for_processing_to_complete()

    def _get_scope_page_header(self):
        return self.scope.select_single(
            PageHeader, objectName='scopePageHeader')

    def wait_for_header_to_stop_moving(self):
        self._get_scope_page_header().wait_until_not_moving()

    def wait_for_processing_to_complete(self):
        """Wait for the processing indicator to disappear."""
        get_dash().wait_for_processing_to_complete()


class ScopeData(object):

    """Class to store all expected data about a scope."""

    def __init__(self, id, title, url, check_header_count='true',
                 default_condition=CONDITION_DEFAULT, parent_id=None,
                 sections=None):
        self.id = id
        self.title = title
        self.url = url
        self.check_header_count = check_header_count == 'true'
        self.default_condition = default_condition
        self.parent_id = parent_id
        self.sections = sections


def get_scope_list():
    """Return a list of ScopeData objects for current channel and device."""
    global scopes_list
    if scopes_list:
        return scopes_list

    required_scopes = _get_required_scope_list(channel, device)
    scopes_list = _get_scope_object_list(required_scopes)
    return scopes_list


def get_scope_data(scope_id):
    """ Get the scope predefined information for the specified scope id """
    for scope in get_scope_list():
        if scope.id == scope_id:
            return scope
    raise RuntimeError("Scope {} not found".format(scope_id))


def go_to_scope(scope_id, scope_class=Scope, dismiss_setup=True):
    """
    Go to the scope received as parameter and wait until it is loaded
    :param scope_id: The desired scope id
    :param scope_class: The class of Scope required to return
    :param dismiss_setup: Whether to dismiss setup of the scope if required
    :return: The current scope object
    """
    url_dispatcher.go_to_url(get_scope_data(scope_id).url)
    scope = get_dash().get_current_scope(scope_class)
    if dismiss_setup and scope.has_skip_button():
        scope.dismiss_setup()
    scope.wait_for_header_to_stop_moving()
    return scope


def _get_scopes_file_path():
    return os.path.realpath(SCOPES_FILE)


def _read_json_from_file(file_path):
    with open(file_path) as f:
        data_json = json.load(f)
    return data_json


def _get_scopes_json():
    return _read_json_from_file(_get_scopes_file_path())


def _get_scopes_list():
    return _get_scopes_json().get('scopes', None) or []


def _get_channels_list():
    return _get_scopes_json().get('channels', None) or []


def _get_scope_from_id(scopes_list_json, scope_id):
    """Return scope json for scope matching scope_id."""
    for scope_json in scopes_list_json:
        if scope_json['id'] == scope_id:
            return scope_json
    return None


def _get_format_current_day_of_month():
    """Return current day of month 0-31."""
    return timedate.get_current_day_of_month()


def _get_format_current_day_name():
    """Return current day name."""
    return timedate.get_current_day_name()


def _get_format_current_month_name():
    """Return current month name."""
    return timedate.get_current_month_name()


def _get_format_current_week_number():
    """Return current week number."""
    return timedate.get_current_week_number()


def _construct_scope(scopes_list_json, scope_id):
    """
    Construct a ScopeData object using the specified scope_id value.

    1) Look up the the scope json definition from the scope id in the list of
       all scopes defined in scopes_list_json.
    2) If this definition includes a parent id then look up that parent
       scope until no other parent scopes are found.
    3) Construct the ScopeData object using the base parent scope.
    4) Add all section titles from the child scope definitions to the
       created ScopeData object, so all required section titles are present.
    5) Return the constructed ScopeData object.

    """
    scope_json = _get_scope_from_id(scopes_list_json, scope_id)
    if KEY_PARENT_ID in scope_json:
        # Look up the base scope re-cursively
        scope = _construct_scope(scopes_list_json, scope_json[KEY_PARENT_ID])
    else:
        # There is no other base scope defined, so create the ScopeData object
        # here using the json for the current scope.
        return ScopeData(**scope_json)

    # To get here there must have been a parent scope defined.
    # Combine the parent scope and child scope by adding all sections from
    # the child scope to the parent scope
    for section in scope_json[KEY_SECTIONS]:
        scope.sections.append(section)

    return scope


def _get_conditional_section_list_from_scope(scope):
    """
    Using the provided scope object, create a list of expected section
    titles for each condition specified.

    The result will be a dictionary of condtions, each containing a list of
    expected sections for that condition.

    :param scope: The ScopeData object to check.
    :return: A dictionary of conditions with a list of expected sections for
             each condition.

    """
    conditions_dict = {}
    conditions = _get_condition_list_from_scope(scope)
    for condition in conditions:
        # create a list of required sections for each condition
        conditions_dict[condition] = _get_sections_for_condition(
            condition, scope.sections)
    return conditions_dict


def _get_condition_list_from_scope(scope):
    """Return a list of all conditions specified in the scope."""
    # use dictionary to keep track of all conditions and avoid duplicates.
    conditions = {}
    conditions[scope.default_condition] = 1
    # Now check through each section to see if it has a condition specified
    for section in scope.sections:
        if KEY_CONDITION in section:
            conditions[section[KEY_CONDITION]] = 1
    return conditions.keys()


def _get_sections_for_condition(condition, sections):
    """Return a list of sections for each specified condition."""
    section_list = []
    for section in sections:
        if KEY_CONDITION in section:
            # this section has a condition specified, check its value
            if section[KEY_CONDITION] == condition:
                # this is a matching condition, so add to list,
                # otherwise just ignore it
                section_list.append(section)
        else:
            # no condition specified for this section, so its required
            section_list.append(section)
    return section_list


def get_section_names_from_list(section_list):
    """
    Return list of section names from the sections list.

    These could be defined as 'header', 'label' or 'image'.

    """
    labels = []
    for section in section_list:
        labels.append(get_section_name_from_section(section))
    return labels


def get_section_name_from_section(section):
    """
    Return section name from the section object.

    The name could be defined as 'header', 'label' or 'image'.

    """
    if SECTION_LABEL in section:
        key = SECTION_LABEL
    elif SECTION_IMAGE in section:
        key = SECTION_IMAGE
    else:
        key = SECTION_HEADER
    value = section[key]

    if KEY_FORMAT_METHODS in section:
        # custom formatting is required on the value returned
        value = _apply_format_methods_to_string(value,
                                                section[KEY_FORMAT_METHODS])

    return value


def get_section_header_names_from_list(section_list):
    """Return a list of Headers items from the section_list."""
    headers = []
    for section in section_list:
        if SECTION_HEADER in section:
            headers.append(section[SECTION_HEADER])
    return headers


def _apply_format_methods_to_string(string, format_methods):
    """
    Apply the specified format methods to the format string.
    :param string: The string to apply the format methods to
    :param format_methods: A string representation of a list of format method
                           names to apply.
                           e.g. "['format_method_1', 'format_method_2']"
    :return: The formatted string

    """
    format_results = []
    for format_method in format_methods.split(','):
        result = getattr(
            THIS_MODULE, '_get_format_{m}'.format(m=format_method.strip()))()
        format_results.append(result)
    return string.format(*format_results)


def _get_device_scope_list(devices, device_name):
    """Return a list of scopes required by a device."""
    scopes = []
    for device in devices:
        if device['name'] == device_name:
            scopes = device['scopes']
    return scopes


def _get_required_scope_list(channel_name, device_name):
    """Return a list of required scope ids for specified channel and device."""
    channels_list_json = _get_channels_list()
    # Find the matching channel and device combination
    # Then store the list of required scope titles
    required_scopes = []
    for channel in channels_list_json:
        if channel['name'] == channel_name:
            devices = channel.get('devices', None) or []
            required_scopes = _get_device_scope_list(devices, device_name)
    return required_scopes


def _get_scope_object(scope_id):
    """Get a ScopeData object for required scope_id."""
    scopes_list_json = _get_scopes_list()
    for scope_json in scopes_list_json:
        # Loop through every scope defined in json
        if scope_json['id'] == scope_id:
            # The required scope id and the json definition match so
            # convert to ScopeData object
            scope = _construct_scope(scopes_list_json, scope_json['id'])
            # Now get a list of sections for each condition specified in the
            # scope. Store the list of conditional sections in the ScopeData
            # object, over-writing previous sections list
            scope.sections = _get_conditional_section_list_from_scope(scope)
            return scope
    # There was no scope matching the required id
    # raise an exception as this implies the json definition is wrong
    ERR_MSG = ('No scope found with id: {s}\nDefinition file: {f}')
    raise NameError(ERR_MSG.format(
        s=scope_id,
        f=_get_scopes_file_path()))


def _get_scope_object_list(required_scopes):
    """Return a list of ScopeData objects from required list."""
    scope_list = []
    for required_scope in required_scopes:
        scope_list.append(_get_scope_object(required_scope['id']))
    return scope_list


def is_image_expected_in_sections(section_list):
    """Return True if an image is expected in the section_list."""
    for section in section_list:
        if SECTION_IMAGE in section:
            return True
    return False


def is_scope_included(scopes, scope_id):
    """Return True when the scope is part of the scopes in scopes list"""
    for scope in scopes:
        if scope.id == scope_id:
            return True
    return False


class ScopesList(ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase):

    """The 'Manage' page for managing scopes.

    Owned by the dash, retrieved by unity8.dash.open_manage_scopes().

    """

    def _is_list_item_visible(self, list_item):
        """Return the visibility of the list item on the screen.

        Is list item origin within screen bounds?  (Doesn't factor
        partial visibility, etc.)

        """
        y = list_item.globalRect[1]
        return y >= 0 and y <= self.height

    def _get_qquickflickable(self):
        """Return the scrollable/flickable for the scopes list."""
        return self.get_children_by_type('QQuickFlickable')[0]

    def _get_scopes_list_by_category(self, category_name):
        """Return ScopesListCategoryItems under the given category name.

        :param category_name: probably 'favorites' or 'other'

        """
        return self.wait_select_single(
            'ScopesListCategory',
            objectName='scopesListCategory{}'.format(category_name))

    def get_scopes_list(self):
        """Return a list of all scopes."""
        return self.select_many('ScopesListCategoryItem')

    def get_favorite_scopes_list(self):
        """Return a list of scopes under the 'Home' section."""
        scopes_list = self._get_scopes_list_by_category('favorites')
        return scopes_list.select_many('ScopesListCategoryItem')

    def get_other_scopes_list(self):
        """Return a list of scopes under the 'Also installed' section."""
        scopes_list = self._get_scopes_list_by_category('other')
        return scopes_list.select_many('ScopesListCategoryItem')

    def favorite_scopes_are_visible(self):
        """Validate that all favorite scopes are visible."""
        favorite_scopes = self.get_favorite_scopes_list()
        for scope in favorite_scopes:
            # weird 'dragItem' interferes; only consider items with text
            if scope.text == '':
                continue
            if self._is_list_item_visible(scope) is False:
                return False
        return True

    def other_scopes_are_visible(self):
        """Validate that at least one 'other' scope is visible."""
        return len(self.get_other_scopes_list()) > 0

    def scrolling_down_shows_more_scopes(self):
        """Scroll down, validate that more scopes are shown.

        The list of scopes changes when we scroll, i.e. the list
        visible at the bottom is different from that at the top.  For
        a small list of scopes we expect this to be false :) .

        """
        scopes_top = set(
            [scope.text for scope in
             filter(self._is_list_item_visible,
                    self.get_scopes_list())])
        flickable = self._get_qquickflickable()
        flickable.swipe_to_bottom()
        scopes_bottom = set(
            [scope.text for scope in
             filter(self._is_list_item_visible,
                    self.get_scopes_list())])
        return scopes_top != scopes_bottom
