# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import time

from autopilot import exceptions

from ubuntu_system_tests.helpers.autopilot import is_element
from ubuntu_system_tests.helpers.scopes import Scope
from ubuntu_system_tests.helpers.ubuntuuitoolkit import pageheader
from ubuntu_system_tests.helpers.unity8.dash import get_dash


HEADER_MY_MUSIC = 'dashSectionHeadermymusic'
CATEGORY_MY_MUSIC = 'dashCategorymymusic'
HEADER_SOUND_CLOUD = 'dashSectionHeadersoundcloud'
CATEGORY_SOUND_CLOUD = 'dashCategorysoundcloud'
HEADER_7DIGITAL = 'dashSectionHeader7digital'
CATEGORY_7DIGITAL = 'dashCategory7digital'


class GenericMusicScope(Scope):
    """ This class contains generic methods used by the music scope classes """

    def get_displayed_titles(self, section=None):
        """
        Retrieve all the titles displayed for all the songs.
        The titles displayed could be the artist, album, genre, etc.
        :param section: If not None, the titles will be retrieved from there,
        otherwise will be retrieved from the scope
        """
        if section:
            buttons = section.select_many('AbstractButton')
        else:
            buttons = self.scope.select_many('AbstractButton')
        return [button.title for button in buttons if
                'delegate' in button.objectName and button.title]


class MusicScope(GenericMusicScope):
    """
    This class extends Scope class and implement the methods specific to
    the Music scope.
    """

    def is_artist(self, artist):
        """ Determines if there is an album with the desired name in the
        current scope
        :param artist: The name of the desired artist
        :return: True is there is an artist with the desired name, False
        otherwise
        """
        try:
            self.get_artist(artist)
            return True
        except exceptions.StateNotFoundError:
            return False

    def get_artist(self, artist):
        """ Retrieve an artist with the desired name """
        return self.scope.wait_select_single('AbstractButton', title=artist)

    def select_artist(self, artist):
        """
        Click on the album with the desired name and wait until the new
        screen is loaded
        :return: A category preview object for the new screen displayed
        """
        self.scope.pointing_device.click_object(self.get_artist(artist))
        self.wait_for_processing_to_complete()
        return CategoryPreview(self.scope.get_root_instance().select_single(
            objectName='dashTempScopeItem'))

    def select_my_music(self):
        """
        Click on the my music category header, wait until the page is loaded
        and return the category preview object
        """
        return self._select_category(HEADER_MY_MUSIC,
                                     CATEGORY_MY_MUSIC)

    def get_my_music_titles(self, count=None):
        """
        Retrieve all the titles displayed for all the songs in my music
        :param count: The number of expected titles
        :raise StateNotFoundError: When the my music section is not displayed
        """
        return self._get_titles(CATEGORY_MY_MUSIC, count)

    def is_my_music_displayed(self):
        """ Indicate if the my music section is being displayed """
        is_element(self._get_category, CATEGORY_MY_MUSIC)

    def select_sound_cloud(self):
        """
        Click on the sound cloud category header, wait until the page is loaded
        and return the category preview object
        """
        return self._select_category(HEADER_SOUND_CLOUD,
                                     CATEGORY_SOUND_CLOUD)

    def get_sound_cloud_titles(self, count=None):
        """
        Retrieve all the titles displayed for all the songs in sound cloud
        :param count: The number of expected titles
        """
        return self._get_titles(CATEGORY_SOUND_CLOUD, count)

    def select_7digital(self):
        """
        Click on the 7digital category header, wait until the page is loaded
        and return the category preview object
        """
        return self._select_category(HEADER_7DIGITAL,
                                     CATEGORY_7DIGITAL)

    def get_7digital_titles(self, count=None):
        """
        Retrieve all the titles displayed for all the songs in 7digital
        :param count: The number of expected titles
        """
        return self._get_titles(CATEGORY_7DIGITAL, count)

    def _get_titles(self, category_name, count=None):
        """
        Retrieve all the titles displayed for all the songs in my music
        :param count: The number of expected titles
        """
        category = self._get_category(category_name)
        if count:
            category.select_single('QQuickListView').count.wait_for(count)
        return self.get_displayed_titles(section=category)

    def _get_section_header(self, header_name):
        return self.scope.wait_select_single(objectName=header_name)

    def _get_category(self, category_name):
        return self.scope.wait_select_single(objectName=category_name)

    def _select_category(self, header_name, category_name):
        section = self._get_section_header(header_name)
        self.scope.pointing_device.click_object(section)
        self.wait_for_processing_to_complete()
        return CategoryPreview(self._get_category(category_name))


class PageHeader(pageheader.PageHeader):
    """
    This class represents the header section that is part of the Dash
    and implement the methods specific to the Music Header.
    """

    MY_MUSIC_ALBUMS_LABEL = 'Albums'
    DIGITAL_ALL_CATEGORIES_LABEL = 'All categories'
    DIGITAL_TOP_ALBUMS_LABEL = 'Top albums'

    def get_navigation_button(self):
        return self.select_single('DashNavigationButton',
                                  objectName='navigationButton',
                                  visible=True)

    def _select_display(self, display):
        """ Click on the dropdown list icon and select the desired label """
        navigation = self.get_navigation_button()
        option_button = navigation.select_single('Icon', name='down',
                                                 visible=True)
        self.pointing_device.click_object(option_button)
        label = navigation.wait_select_single('Label', text=display,
                                              visible=True)
        self.pointing_device.click_object(label)

    def display_by(self, new_display):
        """ Select the display option in the header """
        self._select_display(new_display)
        get_dash().wait_for_processing_to_complete()
        return CategoryPreview(
            self.get_root_instance().select_single(
                objectName='dashTempScopeItem'))


class CategoryPreview(GenericMusicScope):
    """
    This class represents the preview screen that is part of the
    GenericScopeView and implement the methods specific to the Artist Preview.
    """

    def get_track(self, track):
        return self.scope.wait_select_single('AbstractButton', title=track)

    def select_track(self, track):
        """ Click on the track with the desired name """
        self.scroll_to_bottom()
        self.scope.pointing_device.click_object(self.get_track(track))
        return TrackPreview(self.scope)


class TrackPreview(Scope):
    """
    This class represents the preview screen that is part of the
    GenericScopeView and implement the methods specific to the track Preview.
    """

    def get_play_button(self):
        """ Retrieve the play button once it is not moving any more """
        button = self.scope.wait_select_single(objectName='playButton',
                                               visible=True)
        for i in range(10):
            (x, y, w, h) = button.globalRect
            time.sleep(0.2)
            if (x, y, w, h) == button.globalRect:
                return button
        return button

    def play(self):
        """ Click on the play button and wait until the progress var is
        displayed """
        play_button = self.get_play_button()
        self.scope.pointing_device.click_object(play_button)
        self.get_progress_bar().visible.wait_for(True)

    def get_progress_bar(self):
        return self.scope.select_single(objectName='progressBarFill')

    def is_progress_bar_moving(self):
        """ Indicates if the progress bar is moving forward. This process will
        wait half second to see if the progress was has moved """
        try:
            first = self.get_progress_bar().width
            time.sleep(0.5)
            second = self.get_progress_bar().width
            return second > first
        except exceptions.StateNotFoundError:
            return False

    def pause(self):
        """ Pause the song """
        self.scope.pointing_device.click_object(self.get_play_button())

    def play_in_music_app(self):
        """ Play the song in the music-app """
        play = self.scope.select_single(objectName='buttonplay')
        self.scope.pointing_device.click_object(play)


def get_page_header():
    return get_dash().get_current_page_header(header_class=PageHeader)


def get_artists(songs):
    """
    Get a sorted list without duplicates with all the artists in songs
    """
    return sorted(list(set([song.artist for song in songs])))


def get_albums(songs):
    """
    Get a sorted list without duplicates with all the albums in songs
    """
    return sorted(list(set([song.album for song in songs])))
