# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from datetime import datetime

from autopilot import exceptions

from ubuntu_system_tests.helpers.autopilot import wait_select_many
from ubuntu_system_tests.helpers.scopes import Scope


class PhotosScope(Scope):
    """
    This class extends Scope class and implement the methods specific to
    the Photos scope.
    """

    SEARCH_FORMAT = '%d %m %Y'

    def get_picture_from_carousel(self, pic_path):
        """ Retrieve the picture from the carousel
        :param pic_path: Path of the picture
        :return: The picture object when the picture is in the carousel and
        None otherwise
        """
        try:

            imgs = wait_select_many(self.scope,
                                    type_name='CroppedImageMinimumSourceSize',
                                    objectName='artImage')
            for img in imgs:
                if img.source == pic_path:
                    return img
            return None
        except exceptions.StateNotFoundError:
            return None

    def select_picture_from_carousel(self, img):
        """ Select the picture in the carousel """
        self.scope.pointing_device.click_object(img)

    def get_current_photo_preview(self):
        """
        Retrieve the photo preview object that is currently being displayed
        :return: The PhotoPreview object
        """
        previews = wait_select_many(self.scope, type_name='Preview')
        return PhotoPreview(self._get_first_visible_object(previews))

    def is_in_search_result(self, path):
        """
        Indicate if the picture is in the search result based on its path
        """
        try:
            self.scope.select_single('QQuickImage', source=path)
            return True
        except exceptions.StateNotFoundError:
            return False


class PhotoPreview():
    """
    This class represents the preview screen that is part of the
    GenericScopeView and implement the methods specific to the Photos
    Preview.
    """

    def __init__(self, preview):
        self.preview = preview

    def get_open_button(self):
        return self.preview.select_single('PreviewActionButton',
                                          objectName='buttonopen')

    def _get_labels_with_text(self, text):
        labels = self.preview.select_many('Label', objectName='textLabel')
        for label in labels:
            if text in label.text:
                return label
        return None

    def get_date(self):
        """ Retrieve a date following the format used in the preview """
        text = self._get_labels_with_text('Date').text.split(':')[1].strip()
        return datetime.strptime(text, '%A, %B %d, %Y').date()

    def get_time(self):
        """ Retrieve a time following the format used in the preview """
        text = ' '.join(self._get_labels_with_text('Time').text.split(' ')[1:])
        return datetime.strptime(text, '%I:%M:%S %p %Z').time()

    def get_thumbnail(self):
        return self.preview.select_single('LazyImage', objectName='lazyImage')

    def click_open_button(self):
        self.preview.pointing_device.click_object(
            self.get_open_button())
