# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from autopilot import exceptions
import ubuntuuitoolkit as uitk

from ubuntu_system_tests.helpers.scopes import Scope


class VideoScope(Scope):
    """
    This class represents the preview screen that is part of the
    GenericScopeView and implement the methods specific to the Video Scope.
    """

    def is_in_search_result(self, text):
        """ Indicate if a text is part of the search result as video label """
        try:
            self.scope.select_single(objectName='titleLabel', text=text)
            return True
        except exceptions.StateNotFoundError:
            return False

    def _get_my_videos_section(self):
        return self.scope.select_single(
            objectName='dashCategorylocal').select_single('QQuickGridView')

    def get_my_videos_thumbnails(self):
        """
        Retrieve all the titles and thumbnails displayed for all the songs in
        my music section
        """
        labels = []
        sources = []

        videos = self.get_my_videos()
        for video in videos:
            labels.append(video.select_single(objectName='titleLabel').text)
            sources.append(video.select_single(objectName='artImage').source)
        return labels, sources

    def get_my_videos(self):
        """ Retrieve all the youtube videos """
        self.wait_for_processing_to_complete()
        buttons = self._get_videos(self._get_my_videos_section())
        return buttons

    def _get_youtube_category(self):
        return self.scope.select_single(
            'DashCategoryBase', category='com.ubuntu.scopes.youtube_youtube')

    def _get_videos(self, category):
        buttons = category.select_many('AbstractButton')
        return [button for button in buttons if
                'delegate' in button.objectName and button.title]

    def get_youtube_videos(self):
        """ Retrieve all the youtube videos """
        self.wait_for_processing_to_complete()
        return self._get_videos(self._get_youtube_category())

    def select_video(self, video):
        """ Select a video in the youtube section """
        self.scope.pointing_device.click_object(video)
        self.wait_for_processing_to_complete()
        return self.scope.select_single(Preview, isCurrent=True)


class Preview(uitk.UbuntuUIToolkitCustomProxyObjectBase):
    """
    This class represents the preview Widget that is part of the
    GenericScopeView and implement the methods specific to the Video
    Preview.
    """

    def _get_current(self, objname):
        return self.select_single('PreviewWidget', isCurrentPreview=True,
                                  objectName=objname)

    def get_statistics(self):
        """ Retrieve a string with the likes and dislikes """
        statistics = self._get_current('statistics')
        return statistics.select_single(objectName='titleLabel').text

    def get_summary(self):
        """ Retrieve the summary displayed """
        summary = self._get_current('summary')
        return summary.select_single(objectName='textLabel').text

    def get_title(self):
        return self._get_subheader().title

    def get_subtitle(self):
        return self._get_subheader().subtitle

    def _get_subheader(self):
        """ Retrieve a pair with the title and subtitle displayed """
        header = self._get_current('header')
        return header.select_single(objectName='innerPreviewHeader')

    def _get_video(self):
        return self._get_current('video')

    def play_video(self):
        """
        Click on the play button to start the video playback in the webbrowser
        """
        video = self._get_video()
        play = video.select_single(objectName='playButton')
        self.pointing_device.click_object(play)

    def get_thumbnail(self):
        """ Retrieve the thumbnail url for the current video """
        video = self._get_video()
        thumbnail = video.select_single(objectName='screenshot')
        return thumbnail.source

    def get_external_id(self):
        """ Retrieve the id used to link to the external video """
        video = self._get_video()
        screenshot = video.select_single(objectName='screenshot')
        return screenshot.source.split('/')[-2]
