
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
import os
import paramiko

from ubuntu_system_tests import config

logger = logging.getLogger(__name__)


class SSH():
    """This class manages the paramiko ssh client"""

    def __init__(self, hostname, user, password):
        self.client = paramiko.SSHClient()
        self.client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        self.client.connect(hostname, username=user, password=password)

    def close(self):
        """Close the ssh connection"""
        self.client.close()

    def run(self, cmd):
        """Run a command in the remote host and returns the stout"""
        (stdin, stdout, stderr) = self.client.exec_command(cmd)

        for line in stderr.readlines():
            logger.warning(line)

        return "".join(stdout.readlines())

    def put(self, local_path, remote_path):
        """Copy a file through sftp from the local_path to the remote_path"""
        if not os.path.isfile(local_path):
            raise RuntimeError("File to copy does not exist")

        sftp = self.client.open_sftp()
        sftp.put(local_path, remote_path)
        sftp.close()


class LinuxRemoteHelper(SSH):
    """This class executes remote linux commands through the ssh connection"""

    def __init__(self):
        config_stack = config.get_device_config_stack()
        host = config_stack.get('ssh_ip')
        user = config_stack.get('ssh_user')
        password = config_stack.get('ssh_password')
        super(LinuxRemoteHelper, self).__init__(host, user, password)

    def remove_dir(self, dir_name):
        self.run('rm -rf {}'.format(dir_name))

    def create_dir(self, dir_name):
        self.run('mkdir {}'.format(dir_name))

    def remove_file(self, file_name):
        self.run('rm -f {}'.format(file_name))

    def copy_file(self, remote_from, remote_to):
        self.run('cp {rfrom} {rto}'.format(rfrom=remote_from, rto=remote_to))

    def copy_dir(self, remote_from, remote_to):
        """ Copy a dir in the remote host
        :param remote_from: the source remote directory
        :param remote_to: the destination remote directory
        """
        self.run('cp -rf {rfrom} {rto}'.format(rfrom=remote_from,
                                               rto=remote_to))

    def create_random_file(self, file_path, kb=0, mb=0, gb=0):
        """ Create a file with random data in the remote host
        :param file_path: The path to store the file
        :param kb: The kilobytes of the file
        :param mb: The megabytes of the file
        :param gb: The gigabytes fo the file
        :return: The file path to the created file with random content and size
        equals to kb kilobytes + mb megabytes + gb gigabytes
        """
        count = kb + mb * 1024 + gb * 1024 * 1024
        self.run('dd if=/dev/urandom of={file} bs=1024 count={count}'.
                 format(file=file_path, count=count))

    def get_sha1(self, file_path):
        """Calculate the sha1 digest to the specified file"""
        res = self.run("sha1sum {file}".format(file=file_path))
        return res.split()[0]
