# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2014, 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import ubuntuuitoolkit

from retrying import retry

from autopilot import introspection
from autopilot.input._common import get_center_point

from ubuntu_system_tests.helpers.autopilot import order_by_x_coord
from ubuntu_system_tests.helpers.ubuntuuitoolkit.pageheader import PageHeader
from unity8 import process_helpers
from unity8.shell.emulators import dash as unity8_dash


def get_dash():
    """Return the Unity8 Dash autopilot custom proxy object."""
    pid = process_helpers.get_job_pid('unity8-dash')
    dash_proxy = introspection.get_proxy_object_for_existing_process(
        pid=pid,
        emulator_base=ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase,
    )
    return dash_proxy.select_single(Dash)


class Dash(unity8_dash.Dash):

    def wait_for_processing_to_complete(self):
        """Wait until the dash is fully loaded"""
        self.select_single(
            objectName='processingIndicator').visible.wait_for(False,
                                                               timeout=90)

    def open_scope(self, scope):
        """Open the scope once the dash is ready to start the scroll
        :parameter scope: The name of the scope to open
        :return: The opened scope.
        """
        self.wait_for_dash_loaded()
        return super().open_scope(scope)

    def open_apps_scope(self):
        """Open the Apps scope once the dash is ready to start the scroll"""
        self.wait_for_dash_loaded()
        return self.open_scope('clickscope')

    def open_music_scope(self):
        """Open the Music scope once the dash is ready to start the scroll"""
        self.wait_for_dash_loaded()
        return self.open_scope('musicaggregator')

    def wait_for_dash_loaded(self):
        """Wait until the dash content is ready"""
        dash_content = self.select_single(objectName='dashContent')
        dash_content.pageHeaderTotallyVisible.wait_for(True)

    @retry(stop_max_attempt_number=3,
           wait_fixed=500,
           retry_on_exception=lambda exception: (
               isinstance(exception, AssertionError)))
    def _scroll_to_left_scope(self):
        """
        Call parent implementation, retrying in case it raises
        an exception due to a failed  scrolling
        """
        super()._scroll_to_left_scope()

    @retry(stop_max_attempt_number=3,
           wait_fixed=500,
           retry_on_exception=lambda exception: (
               isinstance(exception, AssertionError)))
    def _scroll_to_right_scope(self):
        """
        Call parent implementation, retrying in case it raises
        an exception due to a failed  scrolling
        """
        super()._scroll_to_right_scope()

    def scroll_to_left_scope(self):
        self._scroll_to_left_scope()
        self.dash_content_list.moving.wait_for(False)

    def scroll_to_right_scope(self):
        self._scroll_to_right_scope()
        self.dash_content_list.moving.wait_for(False)

    def get_current_scope(self, scope_class=None):
        """ Retrieve the current scope object based on the scope class desired
        :param scope_class: The scope class to load the object. If this
        parameter is not set, the scope retrieved will be the generic one
        :return: The scope
        """
        if not scope_class:
            # Delay import of Scope class to avoid circular depenedency
            from ubuntu_system_tests.helpers.scopes import Scope
            scope_class = Scope
        scope_loader = self.dash_content_list.select_single(
            'QQuickLoader', isCurrent=True)
        return scope_class(self._get_scope_from_loader(scope_loader))

    def get_scope_count(self):
        """Count how many scopes are present."""
        return len(self.dash_content_list.select_many('GenericScopeView'))

    def get_current_page_header(self, header_class=PageHeader):
        """
        Return the PageHeader object representing the current header
        being displayed.

        :return: PageHeader object

        """
        self.wait_for_dash_loaded()

        # Get a list of headers displayed on the screen
        headers = self.select_many(header_class, visible=True)

        # Return the first visible header ordered by x co-ordinate
        return order_by_x_coord(headers)[0]

    def swipe_screen_from_left(self):
        """Move a focused application to the background and return to dash."""
        width = self.width
        height = self.height
        start_x = 0
        start_y = int(height/2)
        end_x = width
        end_y = start_y
        self.pointing_device.drag(start_x, start_y, end_x, end_y)

    def open_manage_scopes(self):
        """Open and return a helper for the scope management page."""
        overview_drag_handle = self.wait_select_single(
            objectName='overviewDragHandle')
        start_x, start_y = get_center_point(overview_drag_handle)
        stop_x = start_x
        stop_y = 0
        self.pointing_device.drag(start_x, start_y, stop_x, stop_y)
        self.wait_for_dash_loaded()
        return self.wait_select_single(
            'ScopesList',
            objectName='scopesList',
            visible=True)


class GenericScopeView(unity8_dash.GenericScopeView):

    def swipe_to_top(self):
        """Swipe to the top of the scope."""
        list_view = self.select_single(
            'ListViewWithPageHeader', objectName='categoryListView')
        list_view.swipe_to_top()

    def click_scope_item(self, category, title, press_duration=0.10):
        """Override parent implementation making it click at the top of the
        application button to avoid issue when the button is not displayed
        completely

        :parameter category: The name of the category where the item is.
        :parameter title: The title of the item.

        """

        category_element = self._get_category_element(category)
        icon = category_element.wait_select_single('AbstractButton',
                                                   title=title)
        list_view = self.select_single(
            'ListViewWithPageHeader', objectName='categoryListView')
        list_view.swipe_child_into_view(icon)

        x, y, width, height = icon.globalRect
        self.pointing_device.move(x + 1, y + 1)
        self.pointing_device.click(press_duration=press_duration)
