# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import dbus

from ubuntu_system_tests.helpers.dbus_observer import DBusObserver
from ubuntu_system_tests.helpers.threads import StoppingThread

MATCH_STRING = ("type='method_call',path='/com/canonical/usensord/haptic',"
                "interface='com.canonical.usensord.haptic',"
                "destination='com.canonical.usensord',eavesdrop='true'")


class VibrationObserver(DBusObserver):
    """Class to observe when vibrator is turned on."""

    def __init__(self, stop, result):
        """
        Construct the VibrationObserver object.
        :param stop: Event object to signal when to stop waiting.
        :param result: Event object to signal when vibrator is activated.

        """
        super().__init__(stop, result)
        self.bus = dbus.SessionBus()
        self.bus.add_match_string_non_blocking(MATCH_STRING)
        self.bus.add_message_filter(self.on_event)

    def _cleanup(self):
        """Execute cleanup actions before the observer stops."""
        self.bus.remove_match_string_non_blocking(MATCH_STRING)
        self.bus.remove_message_filter(self.on_event)

    def on_event(self, bus, message):
        """
        Callback for when dbus event is observed.
        Set the result to True if it is the required method.

        :param bus: received dbus message bus object.
        :param message: received dbus message object.

        """
        if message.get_interface() == 'com.canonical.usensord.haptic':
            if (message.has_member('Vibrate') or
               message.has_member('VibratePattern')):
                    if message.get_args_list()[0] > 1:
                        self.result.set()


def _start_vibration_observer(stop, result):
    """
    Start a vibration observer.

    :param stop: Event object to signal when to stop observing.
    :param result: Event object to signal when vibration is observed.

    """
    observer = VibrationObserver(stop, result)
    observer.start()


def start_vibration_observer(result):
    """
    Listen for when vibration is activated.

    :param result: Event object to indicate if vibrator has been enabled.
    :return: StoppingThread object which should be stopped by caller when
             required.

    """
    launch_thread = StoppingThread(
        target=_start_vibration_observer, args=[result])
    launch_thread.start()
    return launch_thread
