# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging

from webbrowser_app.emulators import browser

logger = logging.getLogger(__name__)


class Browser(browser.Browser):

    def get_page_loaded(self, url):
        """Wait until the page is fully loaded and returns the text in the
        address bar
        :param url The url to wait
        :return the text displayed in the address bar
        """

        self.wait_until_page_loaded(url)
        return self.address_bar.text

    def get_current_url(self):
        return self.address_bar.text_field.text

    def go_to_url(self, url):
        """Make the browser go to the url passed as parameter
        :param url The url to wait
        :raises RuntimeError when the url could not be correctly written

        The clear method and the retry mechanism are are executes to avoid an
        sporadic error that happen when is being written the url
        """
        self.pointing_device.click_object(self.address_bar.text_field)
        for i in range(3):
            if not self.get_current_url() == url:
                self.address_bar.clear()
                self.address_bar.write(url)
            else:
                self.address_bar.text_field.keyboard.press_and_release('Enter')
                return

        raise RuntimeError("The url couldn't be correctly written in the "
                           "address bar")

    def open_history_view(self):
        """Open the history page where the visited pages are shown"""
        drawer_button = self.select_single("ChromeButton",
                                           objectName="drawerButton")
        self.pointing_device.click_object(drawer_button)
        drawer = self.wait_select_single("QQuickItem",
                                         objectName="drawer",
                                         clip=False)
        tabs_action = drawer.select_single("AbstractButton",
                                           objectName='history',
                                           visible=True)
        self.pointing_device.click_object(tabs_action)

    def get_history_urls(self):
        """Return all the pages (urls) that are shown in the history page"""
        return self.select_many('UrlDelegate')

    def get_tab_preview_urls(self):
        """Return all the pages (urls) that are shown in the history page"""
        return self._get_urls_delegates()

    def _get_urls_delegates(self):
        return self.select_many('UrlDelegate')

    def open_new_tab(self):
        """Open a new tab from the history view and return the tab"""
        self.open_history_view()
        new_tab_button = self.select_single("ToolbarAction",
                                            iconName="tab-new")
        self.pointing_device.click_object(new_tab_button)
        return self.get_new_tab_view()

    def get_popover_foreground_dialog(self):
        self.select_single('UCStyledItemBase', objectName='popover_foreground')

    def get_popover_foreground_dialog_item(self, text):
        elems = self.select_many('Empty', visible=True)
        return next(elem for elem in elems if elem.text == text)

    def get_gallery_button(self):
        dialog = self.wait_select_single('ContentDownloadDialog')
        buttons = dialog.select_many('AbstractButton', visible=True)
        for button in buttons:
            gallery = self.get_gallery(button)
            if gallery is not None:
                return button

        return None

    def get_gallery(self, button):
        images = button.select_many('QQuickImage', objectName='image')
        for img in images:
            if img.source.find('gallery') >= 0:
                return img
        return None

    def open_page_and_accept_location_permission_prompt(self, url):
        """Open the provided url in the browser and accept the location
        access prompt."""
        self.go_to_url(url)
        dialog = self.get_geolocation_dialog()
        accept_button = dialog.get_allow_button()
        self.pointing_device.click_object(accept_button)
        self.wait_page_loaded()

    def wait_page_loaded(self):
        """Wait for the webbrowser to stop loading the current page."""
        webview = self.get_current_webview()
        # loadProgress == 100 ensures that a page has actually loaded
        webview.loadProgress.wait_for(100, timeout=20)
        webview.loading.wait_for(False)
