#!/usr/bin/python3

#
# Ubuntu System Tests
# Copyright (C) 2014, 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import subprocess

import ubuntu_system_tests
from ubuntu_system_tests import (
    config,
    external_tests
)

DEFAULT_TEST_TIMEOUT = '1200'  # 20 minutes.


def main():
    artifacts_directory = get_artifacts_directory()

    config_stack = config.get_device_config_stack()

    test_ids = ubuntu_system_tests.get_tests_ids_to_run(config_stack)
    result = run_autopilot_tests(config_stack, artifacts_directory, *test_ids)
    ensure_temp_stack_deleted()

    if result is False:
        exit(1)


def get_artifacts_directory():
    """Returns path to the directory in which results should be written to."""
    return os.environ.get('ADT_ARTIFACTS')


def run_autopilot_tests(config_stack, artifacts_directory, *test_ids):
    """Run autopilot tests. Return True if all tests pass, False otherwise."""
    autopilot_cmd = _build_autopilot_command_list(config_stack)

    subunit_destination_file_path = os.path.join(
        artifacts_directory, 'test-results.subunit')

    return_code = 0

    for test_id in test_ids:
        _ensure_unity_in_required_state(test_id)
        results_file_path = os.path.join(
            artifacts_directory, test_id + '.subunit')
        test_cmd = autopilot_cmd + ['-o', results_file_path, test_id]
        return_code += subprocess.call(test_cmd)
        if os.path.exists(results_file_path):
            append_subunit_results(
                subunit_destination_file_path, results_file_path)

    return return_code == 0


def _build_autopilot_command_list(config_stack):
    """Return a list containing the base autopilot command to run."""
    test_config_string = config.get_test_config_values(config_stack)

    return [
        'python3',
        '-m',
        'autopilot.run',
        'run',
        '-v',
        '-f',
        'subunit',
        '--config',
        test_config_string,
        '--test-timeout',
        DEFAULT_TEST_TIMEOUT
    ]


def _ensure_unity_in_required_state(test_id):
    if not test_id.startswith('ubuntu_system_tests'):
        external_tests.ensure_unity_in_required_state(test_id)


def append_subunit_results(destination_file_path, results_file_path):
    """Append the subunit results into another subunit file."""
    with open(destination_file_path, 'ab') as destination_file:
        with open(results_file_path, 'rb') as results_file:
            destination_file.write(results_file.read())


def ensure_temp_stack_deleted():
    """Delete the temporal config file after the tests are executed"""
    config.delete_temporal_config_file()


if __name__ == '__main__':
    main()
