# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2014, 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging
import os
import psutil
from retrying import retry
import stat
import subprocess
from tempfile import mkstemp
import threading

from autopilot import (
    input,
    introspection,
    logging as autopilot_logging,
    testcase
)
from autopilot.matchers import Eventually
import ubuntuuitoolkit
from autopilot._fixtures import OSKAlwaysEnabled
from autopilot.introspection import _object_registry as object_registry
from testtools.matchers import Equals
from ubuntuuitoolkit import fixture_setup
from unity8 import process_helpers
from unity8.sensors import FakePlatformSensors

from ubuntu_system_tests.helpers import autopilot as autopilot_helpers
from ubuntu_system_tests.helpers import click
from ubuntu_system_tests.helpers import images
from ubuntu_system_tests.helpers import mir
from ubuntu_system_tests.helpers import processes
from ubuntu_system_tests.helpers import unity8
from ubuntu_system_tests.helpers import wait_until
from ubuntu_system_tests.helpers.processes import ensure_application_closed
from ubuntu_system_tests.helpers.unity8.dash import get_dash
from ubuntu_system_tests.helpers.unity8.shell import Unity8  # NOQA

# Any test name ending with this will leave device locked
TEST_ID_DEVICE_LOCKED = 'device_locked'

logger = logging.getLogger(__name__)


class BaseUbuntuSystemTestCase(testcase.AutopilotTestCase):

    @classmethod
    def setUpClass(cls):
        unity8.ensure_unity_stopped()
        super().setUpClass()

    @classmethod
    def tearDownClass(cls):
        super().tearDownClass()
        unity8.ensure_unity_running_and_greeter_hidden()

    def setUp(self):
        super().setUp()
        self._reset_autopilot_registry()

    @autopilot_logging.log_action(logger.info)
    def launch_unity(self, fake_sensors=False):
        """Launch Unity8 with testability enabled.

        :return: The Unity8 autopiliot custom proxy object.

        """
        self.useFixture(fixture_setup.InitctlEnvironmentVariable(
            QT_LOAD_TESTABILITY='1'))
        if fake_sensors:
            self.useFixture(fixture_setup.InitctlEnvironmentVariable(
                UBUNTU_PLATFORM_API_TEST_OVERRIDE='sensors')
            )
            # This has to be done in a seperate thread to avoid blocking
            # when creating the sensors
            launch_thread = threading.Thread(
                target=self._start_unity_with_testability
            )
            launch_thread.start()
            self._create_sensors()
            launch_thread.join()
        else:
            self._start_unity_with_testability()

        unity = self.get_job_proxy_object('unity8')
        unity.wait_for_greeter_content()
        self._reset_keyboard()
        # Workaround for bug lp:1474444
        self.useFixture(OSKAlwaysEnabled())
        return unity

    def _start_unity_with_testability(self):
        process_helpers.start_job('unity8', 'QT_LOAD_TESTABILITY=1')

    # Retry added until fix for bug 1427946 in QtMir is landed
    @retry(stop_max_delay=30000,
           retry_on_exception=lambda exception: (
               isinstance(exception, psutil.NoSuchProcess) or
               isinstance(exception, ValueError)))
    def get_job_proxy_object(self, pname):
        pid = process_helpers.get_job_pid(pname)
        return self.get_proxy_object(pid)

    def get_proxy_object(self, pid):
        return introspection.get_proxy_object_for_existing_process(
            pid=pid,
            emulator_base=ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase,
        )

    def _reset_autopilot_registry(self):
        """Reset autopilot's registry between tests to avoid CPO errors."""
        object_registry.patch_registry({})

    def _reset_keyboard(self):
        """Reset the Autopilot connection to the OSK"""
        input.Keyboard.create('OSK')._keyboard.reset()

    # Retry added until fix for bug 1427946 in QtMir is landed
    @retry(stop_max_attempt_number=3,
           retry_on_exception=lambda exception: (
               isinstance(exception, FileNotFoundError) or
               isinstance(exception, AssertionError)))
    def _create_sensors(self):
        """Create fake sensors for testing"""
        Eventually(Equals(True)).match(
            lambda: process_helpers.is_job_running('unity8')
        )
        unity_pid = process_helpers._get_unity_pid()
        fifo_path = '/tmp/sensor-fifo-{}'.format(unity_pid)
        Eventually(Equals(True)).match(
            lambda: os.path.exists(fifo_path))
        assert(stat.S_ISFIFO(os.stat(fifo_path).st_mode))
        with open(fifo_path, 'w') as fifo:
            fifo.write('create accel 0 1000 0.1\n')
            fifo.write('create light 0 10 1\n')
            fifo.write('create proximity\n')
        self.sensors = FakePlatformSensors(unity_pid)

    def restart_unity(self, fake_sensors=False):
        """
        Restart and unlock unity8

        :param fake_sensors: Whether to use the fake sensors
        :returns: The proxy object for the Unity8 shell

        """
        unity8.ensure_unity_stopped()
        unity_proxy = self.launch_unity(fake_sensors=fake_sensors)
        if self.device_unlock_required():
            unity_proxy.unlock()
            get_dash().wait_for_dash_loaded()
        return unity_proxy

    def device_unlock_required(self):
        """Return True if test requires device unlocking, False otherwise."""
        test_id = self.id().split('.')[-1]
        return not test_id.endswith(TEST_ID_DEVICE_LOCKED)

    def ensure_application_closed(self, app_name):
        """Stop the app before unity8 is restarted so it's not orphaned."""
        subprocess.call(['ubuntu-app-stop', app_name])

    def get_scree_size(self):
        """ Calculate the screen size base on the unity8 size """
        unity = self.get_job_proxy_object('unity8')
        return unity.get_width(), unity.get_height()

    def get_screenshot_image(self):
        """ Take an screenshot and return a PIL Image file. The screenshot
        will be automatically deleted after the test execution
        :return: A PIL Image file which contains the screenshot
        """
        file_path = mkstemp()[1]
        self.addCleanup(os.remove, file_path)
        mir.get_rgba_screenshot(file_path)
        return images.get_image_from_rgba(file_path, self.get_scree_size())

    def _launch_application_from_apps_scope(self, app_name,
                                            category='predefined'):
        """
        Launch an app from apps scope and return proxy object.

        :param app_name: Name of the displayed application.
        :param category: Category name app is displayed in.
        :return: Proxy object for application.

        """
        apps_scope = get_dash().open_apps_scope()
        apps_scope.click_scope_item(category, app_name)

    def _get_proxy_object_for_existing_app(self, process_name,
                                           cleanup_process=None):
        """
        Return proxy object from existing application process.

        Add cleanup action to close the application using cleanup_process
        parameter. If cleanup_process is None then process_name is used as the
        cleanup target using ensure_application_closed().

        :param process_name: Name of required application process.
        :param cleanup_process: Name of process to clean up.
        :return: Proxy object for the desired application.

        """
        if not cleanup_process:
            cleanup_process = process_name
        proxy = autopilot_helpers.get_proxy_object_for_existing_process(
            process_name,
            ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase)
        self.addCleanup(ensure_application_closed, cleanup_process)
        return proxy

    def _get_proxy_object_for_existing_qmlscene_process(self, qmlfile):
        """
        Return proxy object from existing qmlscene process.

        Add cleanup action to close the application using cleanup_process
        parameter. If cleanup_process is None then process_id is used as the
        cleanup target using processes.stop_qmlscene_process().

        :param qmlfile: Thje name of the QML file for the required process
        :return: Proxy object for the desired qmlscene process

        """
        process_id = processes.get_qmlscene_process_id(qmlfile)
        proxy = introspection.get_proxy_object_for_existing_process(
            pid=process_id,
            emulator_base=ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase
        )
        self.addCleanup(processes.stop_qmlscene_process, qmlfile)
        return proxy

    def launch_address_book(self):
        """
        Launch the address book app from apps scope and return proxy object.

        :return: Proxy object for address book application.

        """
        self._launch_application_from_apps_scope('Contacts')
        return self.get_address_book_app_proxy()

    def get_address_book_app_proxy(self):
        """
        Return address book app proxy object from existing process.

        :return: Proxy object for address book application.

        """
        from ubuntu_system_tests.helpers.addressbook import _cpo  # NOQA
        proxy = self._get_proxy_object_for_existing_app('address-book-app')
        return proxy.main_window

    def launch_camera_app(self):
        """
        Launch the camera app from dash and return proxy object.

        :return: Proxy object for camera application.

        """
        self._launch_application_from_apps_scope('Camera')
        return self.get_camera_app_proxy()

    def get_camera_app_proxy(self):
        """
        Return camera app proxy object from existing process and add
        cleanup actions to close the application.

        :return: Proxy object for camera application.

        """
        from ubuntu_system_tests.helpers.camera._cpo import MainWindow
        proxy = self._get_proxy_object_for_existing_app(
            'camera-app', click.get_click_app_identifier('com.ubuntu.camera'))
        return MainWindow(proxy)

    def launch_dialer_app(self):
        """
        Launch the dialer app from apps scope and return proxy object.

        :return: Proxy object for dialer application.

        """
        self._launch_application_from_apps_scope('Phone')
        return self.get_dialer_app_proxy()

    def get_dialer_app_proxy(self):
        """
        Return dialer app proxy object from existing process.

        :return: Proxy object for dialer application.

        """
        from ubuntu_system_tests.helpers.dialer_app import _cpo  # NOQA
        proxy = self._get_proxy_object_for_existing_app('dialer-app')
        return proxy.main_view

    def launch_gallery_app(self):
        """
        Launch the gallery app from apps scope and return proxy object.

        :return: Proxy object for gallery application.

        """
        from ubuntu_system_tests.helpers.gallery._cpo import EventsView
        self._launch_application_from_apps_scope('Gallery')
        return EventsView(self.get_gallery_app_proxy())

    def get_gallery_app_proxy(self):
        """
        Return gallery app proxy object from existing process.

        :return: Proxy object for gallery application.

        """
        return self._get_proxy_object_for_existing_app('gallery-app')

    def get_gallery_events_view(self):
        """Return a gallery_app EventsView custom proxy object."""
        from ubuntu_system_tests.helpers.gallery._cpo import EventsView
        return EventsView(self.get_gallery_app_proxy())

    def get_gallery_photo_viewer(self):
        """Return a gallery_app PhotoViewer custom proxy object."""
        from ubuntu_system_tests.helpers.gallery._cpo import PhotoViewer
        return PhotoViewer(self.get_gallery_app_proxy())

    def get_media_player_proxy(self):
        """
        Return media player app proxy object from existing process.

        :return: Proxy object for media player application.

        """
        from ubuntu_system_tests.helpers.mediaplayer._cpo import MainWindow
        proxy = self._get_proxy_object_for_existing_app('mediaplayer-app')
        return MainWindow(proxy)

    def launch_messaging_app(self):
        """
        Launch the messaging app from apps scope and return proxy object.

        :return: Proxy object for messaging application.

        """
        self._launch_application_from_apps_scope('Messaging')
        return self.get_messaging_app_proxy()

    def get_messaging_app_proxy(self):
        """
        Return messaging app proxy object from existing process.

        :return: Proxy object for messaging application.

        """
        from ubuntu_system_tests.helpers.messaging._cpo import MainView
        proxy = self._get_proxy_object_for_existing_app('messaging-app')
        return proxy.wait_select_single(MainView)

    def launch_system_settings(self):
        """
        Launch system settings from apps scope and return proxy object.

        :return: Proxy object for system settings.

        """
        self._launch_application_from_apps_scope('System Settings', 'local')
        return self.get_system_settings_proxy()

    def get_system_settings_proxy(self):
        """
        Return system settings proxy object from existing process.

        :return: Proxy object for system settings.

        """
        from ubuntu_system_tests.helpers.system_settings import _cpo  # NOQA
        proxy = self._get_proxy_object_for_existing_app(
            'system-settings', 'ubuntu-system-settings')
        return proxy.main_view

    def launch_webbrowser_app(self):
        """
        Launch the webbrowser app from apps scope and return proxy object.

        :return: Proxy object for webbrowser application.

        """
        self._launch_application_from_apps_scope('Browser')
        return self.get_webbrowser_app_proxy()

    def get_webbrowser_app_proxy(self):
        """
        Return webbrowser app proxy object from existing process.

        :return: Proxy object for webbrowser application.

        """
        from ubuntu_system_tests.helpers.webbrowser import _cpo  # NOQA
        proxy = self._get_proxy_object_for_existing_app('webbrowser-app')
        return proxy.main_window

    def launch_calendar_app(self):
        """
        Launch the calendar app from apps scope and return proxy object.

        :return: Proxy object for calendar application.

        """
        self._launch_application_from_apps_scope('Calendar', 'local')
        wait_until(processes.is_qmlscene_running_with_qmlfile, 'calendar')
        return self.get_calendar_app_proxy()

    def get_calendar_app_proxy(self):
        """
        Return calendar app proxy object from existing process.

        :return: Proxy object for calendar application.

        """
        proxy = self._get_proxy_object_for_existing_qmlscene_process(
            'calendar')
        return proxy.main_window

    def launch_clock_app(self):
        """
        Launch the clock app from apps scope and return proxy object.

        :return: Proxy object for clock application.

        """
        self._launch_application_from_apps_scope('Clock')
        wait_until(processes.is_qmlscene_running_with_qmlfile,
                   'ubuntu-clock-app')
        return self.get_clock_app_proxy()

    def get_clock_app_proxy(self):
        """
        Return clock app proxy object from existing process.

        :return: Proxy object for calendar application.

        """
        from ubuntu_system_tests.helpers.clock import _cpo  # NOQA
        proxy = self._get_proxy_object_for_existing_qmlscene_process(
            'ubuntu-clock-app')
        return proxy.main_view
