# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3,
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os

from datetime import datetime

from ubuntu_system_tests.helpers.addressbook import(
    Contact,
    fixture_setup as address_book_fixture
)
from ubuntu_system_tests.helpers import file_system as fs
from ubuntu_system_tests.helpers.messaging import fixture_setup
from ubuntu_system_tests.helpers import messaging as messaging_helpers
from ubuntu_system_tests.helpers import system_settings
from ubuntu_system_tests.helpers.system_settings import SETTINGS_PROCESS
from ubuntu_system_tests.helpers.unity8 import dash
from ubuntu_system_tests.helpers import wait_until
from ubuntu_system_tests.tests.base_telephony import BaseTelephonyTestCase

CAMERA_APP = 'Camera'
CAMERA_PROCESS = 'camera-app'
GALLERY_APP = 'Gallery'
GALLERY_PROCESS = 'gallery-app'

CONTACT_NAME_1 = 'Fulano de Tal'
REPLY_1 = 'ACK'

CONTACT_NAME_2 = 'Ubuntu Tester'
REPLY_2 = 'ACK2'


class MessagingBase(BaseTelephonyTestCase):

    def setUp(self):
        super().setUp()
        self.useFixture(fixture_setup.MessagingTestEnvironment())
        self.contact_1 = Contact(name=CONTACT_NAME_1,
                                 number=self.get_telephony_service_number1(),
                                 expected_reply=REPLY_1)
        self.contact_2 = Contact(name=CONTACT_NAME_2,
                                 number=self.get_telephony_service_number2(),
                                 expected_reply=REPLY_2)
        self.useFixture(
            address_book_fixture.AddTemporaryContacts([self.contact_1]))
        self.dash = dash.get_dash()

    def launch_messaging(self):
        """Launch the messaging app from apps scope."""
        self.main_view = self.launch_messaging_app()

    def get_messaging_app(self):
        """
        Get messaging app proxy object from existing process and add cleanup.
        Store proxy object in self.main_view.

        """
        self.main_view = self.get_messaging_app_proxy()

    def receive_message_and_get_notification(self, to, from_, message,
                                             media_url=None, timeout=60):
        """
        Receive a message from Telephony service and return notification.

        If media_url is None then an SMS will be sent, if media_url is
        specified the content will be attached to an MMS message.

        :param to: Phone number of recipient
        :param from_: Phone number of sender
        :param message: Body of the message to send
        :param media_url: Url of file to attach to message
        :param timeout: Timeout period to wait for notification in seconds
        :return: Notification message
        """
        notifications_list = self.shell.get_notifications_list()
        sent = self.request_incoming_message(to, from_, message, media_url)

        self.assertTrue(sent, 'The message was not sent succesfully by the '
                              'telephony service.')

        return notifications_list.get_new_message_notification(timeout)

    def request_incoming_message(self, to, from_, message, media_url=None):
        """Send an incoming message request for the DUT."""
        if media_url:
            return self.telephony.send_mms(to, from_, message, media_url)
        else:
            return self.telephony.send_sms(to, from_, message)

    def send_message_to_contacts_and_validate_replies(self, contacts, message,
                                                      sample_file=None):
        """
        Send message to each contact in contacts.
        1) From messaging app start a new message
        2) Add each contact to the message
        3) Press send button
        4) For each contact wait for a reply message
        5) Query Twilio to ensure the message was received

        :param contacts: A list of Contact objects to send message to.
        :param message: Message to send
        :param sample_file: Name of the media file that was sent in case
               of MMS.

        """
        self.compose_message_to_contacts(contacts, message)
        contact_count = len(contacts)
        if contact_count > 1:
            notifications = contact_count
        else:
            # If only sending a message to one contact, then a pop-up
            # notification is not shown when already in the contact's thread
            # view. Set notification observer to expect 0, so it will only
            # dismiss ussd notifications.
            notifications = 0
        self.start_notification_observer(expected_count=notifications)
        self.send_message(message)
        if contact_count > 1:
            self.wait_for_all_notifications()
        else:
            # Check the thread for reply messages as there is no pop up shown
            # when already in the contact's thread.
            reply_count = self.calculate_reply_count(message)
            self.wait_for_contact_replies(contacts[0], reply_count)
        self.stop_notification_observer()
        self.delete_message_from_thread(message)
        self.return_to_threads_view()
        self.wait_for_all_replies(contacts, message)
        self.validate_outgoing_message_received(contacts, message, sample_file)

    def send_message_to_number_and_validate_reply(self, contact, message):
        """
        Send message to each contact in contacts.
        1) From messaging app start a new message
        2) Add outgoing phone number for message
        3) Press send button
        4) Wait for a reply message
        5) Query Twilio to ensure the message was received

        :param contact: A contact object to send message to.
        :param message: Message to send

        """
        self.main_view.start_new_message()
        self.main_view.type_contact_name(contact.number)
        self.main_view.type_message(message.body)
        # When the required thread is displayed there is no pop-up shown,
        # so set expected_count to 0 to just dismiss any possible
        # balance notifications
        self.start_notification_observer(expected_count=0)
        self.send_message(message)
        # Check the thread for reply messages as there is no pop up shown
        # when already in the contact's thread.
        reply_count = self.calculate_reply_count(message)
        self.wait_for_contact_replies(contact, reply_count)
        self.stop_notification_observer()
        self.delete_message_from_thread(message)
        self.return_to_threads_view()
        self.wait_for_all_replies([contact], message)
        self.validate_outgoing_message_received([contact], message)

    def compose_message_to_contacts(self, contact_list, message):
        """
        Compose a message to each contact in contact_list.
        1) From messaging app start a new message
        2) Add each contact to the message
        3) Add each required attachment

        :param contact_list: A list of Contact objects to send message to.
        :param message: Message to send

        """
        self.main_view.start_new_message()
        for contact in contact_list:
            self.main_view.type_contact_name_and_select_num(contact.name,
                                                            contact.number)
        self.main_view.type_message(message.body)
        self.add_attachments_to_message(message.attachments)

    def send_message(self, message):
        """Press the send button and clear any network notifications."""
        # calculate a timeout based on the number of attachments to send
        timeout = 60 * (len(message.attachments) + 1)
        self.main_view.click_send_button()
        bubble = self.main_view.get_message(message.body).get_message_bubble()
        bubble.wait_until_sent(timeout)

    def delete_message_from_thread(self, message):
        """Delete the specified message from opened thread view."""
        self.main_view.delete_message(message.body)

    def choose_sim_from_prompt(self, sim):
        """Choose the selected sim from the prompt."""
        self.main_view.select_sim(sim)

    def return_to_threads_view(self):
        """Return to threads view."""
        self.main_view.go_back()
        self.main_view.get_thread_list().visible.wait_for(True)

    def calculate_reply_count(self, message):
        """
        Calculate the number of reply messages expected.

        The number of expected reply messages is based on length of
        message and whether the telephony service supports message
        concatenation or not. If concatenation is not supported then a reply
        is expected for each individual message sent, otherwise it will be a
        single reply for the whole message.

        :param message: Message to examine.
        :return: Number of reply messages expected.

        """
        concatenate = self.telephony.get_properties()['sms-concatenation']
        count = len(messaging_helpers.split_into_received_messages(
            message.body, concatenate))
        return count

    def wait_for_all_replies(self, contact_list, message):
        """
        From the main thread view, wait for the required number of replies
        from each contact in contact_list.

        1) For each contact open their thread
        2) Wait for required number of replies to be displayed in thread
        3) Return to application main thread view

        :param contact_list: A list of all contacts expecting replies from.
        :param message: The sent message. The required number of replies is
                        calculated from the number of messages required to
                        send the message body.

        """
        self.wait_for_contacts_replies(len(contact_list))
        count = self.calculate_reply_count(message)
        for contact in contact_list:
            self.main_view.get_main_page().open_thread(contact.number)
            self.wait_for_contact_replies(contact, count)
            self.main_view.go_back()

    def wait_for_contacts_replies(self, contact_count):
        """
        Wait until at least 1 reply has been sent by each contact and the
        contact is displayed in the main thread view.

        """
        all_threads = wait_until(
            self._are_all_expected_threads_displayed,
            timeout=180, period=5,
            expected_count=contact_count)
        self.assertTrue(
            all_threads,
            'Did not receive {n} expected replies'.format(n=contact_count))

    def wait_for_contact_replies(self, contact, replies):
        """
        Wait for expected number of replies for specified contact to be
        received in the contact's thread view.

        :param contact: The contact identifier
        :param replies: The number of expected replies from contact.

        """
        received = wait_until(
            self._are_all_expected_replies_displayed,
            timeout=180, period=5,
            text=contact.expected_reply,
            phone_number=contact.number,
            expected_replies=replies)
        msgs_page = self.main_view.get_messages_page()
        self.assertTrue(
            received,
            'Did not receive {n} expected responses from the telephony'
            ' service for contact: {c}\nOnly received {r} reply '
            'messages\nExpected reply: {e}'
            .format(
                n=replies,
                c=contact.name,
                r=msgs_page.how_many_messages_displayed(
                    contact.expected_reply, contact.number),
                e=contact.expected_reply))

    def _are_all_expected_threads_displayed(self, expected_count):
        """
        Return True if expected number of threads is displayed in main view.

        """
        threads = self.main_view.get_main_page().get_thread_count()
        return threads == expected_count

    def _are_all_expected_replies_displayed(self, text, phone_number,
                                            expected_replies):
        """
        Return True if the number of expected replies is displayed in
        the contact's thread view.

        """
        messages = self.main_view.get_messages_page()
        count = messages.how_many_messages_displayed(text, phone_number)
        return count == expected_replies

    def validate_outgoing_message_received(self, contact_list, message,
                                           sample_file_name=None):
        """
        Validate that the telephony service received the messages sent from
        the device.

        :param contact_list: List of contacts. Each contact should have a phone
                             number which is used by the telephony service.
        :param message: The expected message that should have been received
        :param sample_file_name: the sample file in case of an MMS.

        """
        for contact in contact_list:
            sent = self.telephony.get_sent_sms(
                contact.number,
                self.get_device_phone_number(),
                datetime.today(),
                message.body)
            self.assertTrue(sent)

            if sample_file_name:
                self._verify_mms_integrity(contact, message, sample_file_name)

    def _verify_mms_integrity(self, contact, message, sample_file_name):
        """Download MMS attachment from twilio and compare it with
        local sample file to verify integrity.
        """
        sample_file = os.path.join(fs.DIR_HOME_CAMERA_PICTURES,
                                   sample_file_name)
        downloaded_mms = self.telephony.download_mms_content(
            contact.number,
            self.get_device_phone_number(),
            datetime.today(),
            message.body)
        self.addCleanup(fs.delete_file, downloaded_mms)
        self.assertTrue(
            fs.compare_files(sample_file, downloaded_mms))

    def _open_app_to_attach_content(self, app_name):
        """
        Opens the requested app to pick content to attach

        :param app_name: name of the app to pick content from

        """
        self.main_view.get_messages_page().visible.wait_for(True)
        content_picker = self.main_view.click_attach_button()
        content_picker.pick_app(app_name)

    def add_attachments_to_message(self, attachments):
        """
        Add the specified attachements to the currently opened message.
        Verify that each attachment is added to the message editor.

        :param: List of attachements

        """
        counter = 0
        for attachment in attachments:
            # call the specified attach_method function with the attachment
            # as argument
            attachment.attach_method(attachment)
            counter += 1
            # This will wait if there is a delay in the attachment
            # being displayed in message editor
            ready = wait_until(
                self._are_expected_attachments_displayed, timeout=10, period=1,
                expected_count=counter)
            d = self.main_view.get_messages_page().get_attachment_count()
            # check that expected attachments are displayed
            self.assertTrue(ready,
                            'Number of displayed attachments {d} does not '
                            'match expected {e}'.format(d=d, e=counter))

    def _are_expected_attachments_displayed(self, expected_count):
        """Return true if all expected attachments are displayed in editor."""
        attachments = self.main_view.get_messages_page().get_attachment_count()
        return attachments == expected_count

    def _add_attachment_from_gallery(self, attachment):
        """Add an attachment using the gallery app."""
        self._open_app_to_attach_content(GALLERY_APP)
        gallery_app = self.get_gallery_events_view()
        self.addCleanup(self.ensure_application_closed, GALLERY_PROCESS)
        sample_photo_location = os.path.join(fs.DIR_HOME_CAMERA_PICTURES,
                                             attachment.identifier)
        gallery_app.pick_photo(sample_photo_location)

    def _add_attachment_from_camera(self, attachment):
        """Add an attachment using the camera app."""
        self._open_app_to_attach_content(CAMERA_APP)
        camera_app = self.get_camera_app_proxy()
        self.addCleanup(self.ensure_application_closed, CAMERA_PROCESS)
        camera_app.take_photo_and_select()

    def _add_attachment_from_camera_photoroll(self, attachment):
        """Add an attachment using the camera app photoroll."""
        self._open_app_to_attach_content(CAMERA_APP)
        camera_app = self.get_camera_app_proxy()
        self.addCleanup(self.ensure_application_closed, CAMERA_PROCESS)
        camera_app.swipe_to_gallery(self)
        camera_app.select_photo_from_roll()

    def set_default_sim_for_messages_with_uss(self, value):
        """
        1) Load Ubuntu System Settings and set the 'default sim for messages'
           setting to the specified value.
        2) Close Ubuntu System Settings
        3) Return to the top of the apps scope

        :param value: Value to set: 'ask', '1' or '2' (sim slot)

        """
        settings = self.launch_system_settings()
        cellular = settings.go_to_cellular_page()

        if value is '1' or value is '2':
            # convert to ril index, ril_0 or ril_1
            value = system_settings.get_ril_modem_for_sim(value)
        selector = cellular.get_default_sim_for_messages_selector(value)
        settings.scroll_to_and_click(selector)
        selector.selected.wait_for(True)
        dash.get_dash().swipe_screen_from_left()
        self.addCleanup(self.ensure_application_closed, SETTINGS_PROCESS)

    def click_message_url(self, message_text):
        """Click the message text in a message bubble."""
        message = self.main_view.get_message(message_text)
        text_item = message.select_single('Label', objectName='messageText')
        self.main_view.pointing_device.click_object(text_item)
