# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from collections import namedtuple
import subprocess

from autopilot.matchers import Eventually
from autopilot import platform
from testtools.matchers import Equals, LessThan, GreaterThan
from testtools import skipUnless

from ubuntu_system_tests.helpers import system_settings
from ubuntu_system_tests.helpers.power import fixture_setup as power_fixture

from ubuntu_system_tests.tests.indicators import BaseIndicatorTestCase


BATTERY_INDICATOR = 'indicator-power'
BATTERY_INDICATOR_PAGE = 'indicator-power-page'
BATTERY_SETTINGS_PAGE = 'powerPage'

DEVICE_ARALE = 'arale'
DEVICE_KRILLIN = 'krillin'
DEVICE_MAKO = 'mako'

BrightnessValues = namedtuple('BrightnessValues', ['brightness_file',
                                                   'brightness_min',
                                                   'brightness_max'])

test_data = {
    DEVICE_ARALE: BrightnessValues(
        brightness_file='/sys/class/backlight/lm3630_bled/brightness',
        brightness_min=60,
        brightness_max=1984
    ),
    DEVICE_KRILLIN: BrightnessValues(
        brightness_file='/sys/devices/platform/leds-mt65xx/leds/'
                        'lcd-backlight/brightness',
        brightness_min=10,
        brightness_max=255
    ),
    DEVICE_MAKO: BrightnessValues(
        brightness_file='/sys/devices/i2c-0/0-0038/backlight/lm3530/'
                        'brightness',
        brightness_min=2,
        brightness_max=114
    )
}

SUPPORTED_DEVICES = [DEVICE_ARALE, DEVICE_KRILLIN, DEVICE_MAKO]
# Max value for brightness in gsettings is 255 and minimum is 15
GSETTINGS_BRIGHTNESS_MAX = 255


class BatteryIndicatorTestCase(BaseIndicatorTestCase):

    def setUp(self):
        super().setUp(indicator=BATTERY_INDICATOR,
                      indicator_page=BATTERY_INDICATOR_PAGE)

    def test_battery_settings_item_open_correct_page(self):
        """
        1. Drag down the battery indicator
        2. Tap 'Battery settings...'
        3. Ensure battery settings open.
        """
        self.indicator_page.open_battery_settings()
        settings_app = self.get_system_settings_proxy()
        system_settings.ensure_power_settings_page_visible(settings_app)

    def get_current_brightness_from_backend(self):
        """Return the current brightness from the backlight driver."""
        output = subprocess.check_output(
            ['cat', test_data.get(platform.image_codename()).brightness_file],
            universal_newlines=True
        )

        return int(output.strip())

    @skipUnless(platform.image_codename() in SUPPORTED_DEVICES,
                'Supported devices: {}'.format(', '.join(SUPPORTED_DEVICES)))
    def test_brightness_slider_is_effective(self):
        """
        Prerequisite: ensure auto brightness is disabled and screen brightness
                      is maximum
        1. Drag down the battery indicator
        2. Drag the brightness handle to the left till the end
        3. Drag the brightness handle to the right till the end
        """
        max_brightness = test_data.get(
            platform.image_codename()).brightness_max
        min_brightness = test_data.get(
            platform.image_codename()).brightness_min

        self.useFixture(power_fixture.BrightnessEnvironmentFixture(
            auto_brightness='disable',
            brightness_value=GSETTINGS_BRIGHTNESS_MAX)
        )
        slider = self.indicator_page.get_brightness_slider_widget()

        while self.indicator_page.get_current_brightness() != 0.0:
            brightness_slider = self.indicator_page.get_current_brightness()
            brightness_backend = self.get_current_brightness_from_backend()
            slider.slide_one_step_left()
            self.assertThat(
                lambda: self.indicator_page.get_current_brightness(),
                Eventually(LessThan(brightness_slider)))
            self.assertThat(lambda: self.get_current_brightness_from_backend(),
                            Eventually(LessThan(brightness_backend)))

        self.assertThat(lambda: self.get_current_brightness_from_backend(),
                        Eventually(Equals(min_brightness)))

        while self.indicator_page.get_current_brightness() != 1.0:
            brightness_slider = self.indicator_page.get_current_brightness()
            brightness_backend = self.get_current_brightness_from_backend()
            slider.slide_one_step_right()
            self.assertThat(
                lambda: self.indicator_page.get_current_brightness(),
                Eventually(GreaterThan(brightness_slider)))
            self.assertThat(lambda: self.get_current_brightness_from_backend(),
                            Eventually(GreaterThan(brightness_backend)))

        self.assertThat(lambda: self.get_current_brightness_from_backend(),
                        Eventually(Equals(max_brightness)))
