
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from autopilot.matchers import Eventually
from testtools.matchers import Equals, NotEquals
from testtools import skipUnless

from ubuntu_system_tests.helpers import (
    autopilot,
    ofono,
    radio_killswitch,
    network,
)
from ubuntu_system_tests.helpers.ofono import fixture_setup
from ubuntu_system_tests.helpers import system_settings
from ubuntu_system_tests.tests.indicators import BaseIndicatorTestCase


NETWORK_INDICATOR = 'indicator-network'
NETWORK_INDICATOR_PAGE = 'indicator-network-page'
CELLULAR_SETTINGS_PAGE = 'cellularPage'
WIFI_SETTINGS_PAGE = 'wifiPage'


class NetworkIndicatorBase(BaseIndicatorTestCase):

    def setUp(self):
        super().setUp(indicator=NETWORK_INDICATOR,
                      indicator_page=NETWORK_INDICATOR_PAGE)


class FlightModeTestCase(NetworkIndicatorBase):

    @skipUnless(ofono.is_sim_inserted() and autopilot.is_wifi_supported(),
                'SIM and Wi-Fi support required.')
    def setUp(self):
        self.radio_killswitch = self.useFixture(
            radio_killswitch.RadioKillswitch(False))
        super().setUp()

    def assert_network_indicator_shows_flightmode_off(self,
                                                      network_indicator_page):
        """Network indicator UI elements reflect 'off' state of flight mode."""
        self.assertTrue(network_indicator_page.is_flight_mode_switch_visible())
        self.assertThat(network_indicator_page.is_sim_list_offline,
                        Eventually(Equals(False)))
        self.assertTrue(network_indicator_page.is_wifi_switch_visible())
        self.assertThat(network_indicator_page.is_wifi_enabled,
                        Eventually(Equals(True)))
        self.assertThat(network_indicator_page.get_list_of_wifi_access_points,
                        Eventually(NotEquals([])))
        self.assertTrue(network_indicator_page.is_cellular_settings_visible())
        self.assertTrue(network_indicator_page.is_wifi_settings_visible())

    def assert_network_indicator_shows_flightmode_on(self,
                                                     network_indicator_page):
        """Network indicator UI elements reflect 'on' state of flight mode."""
        self.assertThat(network_indicator_page.is_sim_list_offline,
                        Eventually(Equals(True)))
        self.assertThat(network_indicator_page.is_wifi_enabled,
                        Eventually(Equals(False)))
        self.assertThat(
            network_indicator_page.get_list_of_wifi_access_points,
            Eventually(Equals([])))

    def test_switching_on_flight_mode_must_engage_flight_mode(self):
        """Switching on 'flight mode' in the indicator must engage flight mode.

        Note that we assume we're starting in *off* mode; also we skip
        the test if no SIM is inserted.

        """
        self.assertFalse(self.radio_killswitch.is_flight_mode_on(),
                         'Test must be run with flight mode off.')
        self.assert_network_indicator_shows_flightmode_off(
            self.indicator_page)
        self.indicator_page.turn_on_flight_mode()
        self.assertThat(self.radio_killswitch.is_flight_mode_on,
                        Eventually(Equals(True)))
        self.assert_network_indicator_shows_flightmode_on(
            self.indicator_page)
        self.indicator_page.turn_off_flight_mode()
        self.assertThat(self.radio_killswitch.is_flight_mode_on,
                        Eventually(Equals(False)))
        self.assert_network_indicator_shows_flightmode_off(
            self.indicator_page)

    def assert_network_indicator_shows_wifi_enabled_flight_mode_on(
            self, network_indicator_page):
        self.assertThat(network_indicator_page.is_sim_list_offline,
                        Eventually(Equals(True)))
        self.assertThat(network_indicator_page.is_wifi_enabled,
                        Eventually(Equals(True)))
        self.assertThat(
            network_indicator_page.get_list_of_wifi_access_points,
            Eventually(NotEquals([])))

    def test_enable_wifi_in_flight_mode(self):
        """
        Enable flight mode and turn on wifi and check internet is working

        1. Switch to flight mode
        2. Turn on Wifi
        3. Ensure internet is working
        """
        self.assertFalse(self.radio_killswitch.is_flight_mode_on(),
                         'Test must be run with flight mode off.')
        self.assert_network_indicator_shows_flightmode_off(
            self.indicator_page)
        self.indicator_page.turn_on_flight_mode()
        self.addCleanup(self.indicator_page.turn_off_flight_mode)
        self.assertThat(self.radio_killswitch.is_flight_mode_on,
                        Eventually(Equals(True)))
        self.assert_network_indicator_shows_flightmode_on(
            self.indicator_page)
        self.indicator_page.turn_on_wifi()
        self.assert_network_indicator_shows_wifi_enabled_flight_mode_on(
            self.indicator_page)
        network.ensure_internet_connectivity(timeout=15)


class CellularProvidersTestCase(NetworkIndicatorBase):

    @skipUnless(ofono.is_sim_inserted(), 'At least one SIM card required')
    def setUp(self):
        self.useFixture(fixture_setup.UnlockSims())
        super().setUp()

    def test_cellular_providers_register(self):
        """
        Prerequisite: At least one SIM is inserted into the phone.
        1. Drag down the network indicator
           - The network menu opens and shows the currently
             active SIMs names.
        """
        sims = self.indicator_page.get_list_of_sims()
        for i in range(len(sims)):
            self.assertEqual(sims[i].get_sim_name(), ofono.get_sim_name(i))


class NetworkIndicatorTestCase(NetworkIndicatorBase):

    def setUp(self):
        super().setUp()

    def test_cellular_settings_item_open_correct_page(self):
        """
        1. Drag down the network indicator
        2. Tap 'Cellular settings...'
        3. Ensure cellular settings open
        """
        self.indicator_page.open_cellular_settings()
        settings_app = self.get_system_settings_proxy()
        system_settings.ensure_settings_page_visible(settings_app,
                                                     CELLULAR_SETTINGS_PAGE)

    def test_wifi_settings_item_open_correct_page(self):
        """
        1. Drag down the network indicator
        2. Tap 'Wifi settings...'
        3. Ensure Wifi settings open
        """
        self.indicator_page.open_wifi_settings()
        settings_app = self.get_system_settings_proxy()
        system_settings.ensure_settings_page_visible(settings_app,
                                                     WIFI_SETTINGS_PAGE)
