# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2014, 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# Steps
# --------
# 1. Navigate to the Apps Scope
# * apps scope is displayed
# 2. Scroll to the bottom of the scope
# * Ubuntu apps store is displayed
# 3. Tap on The ubuntu store button
# * Ubuntu apps store opens
# 4. Tap on an uninstalled app
# * Main view for app appears
# 5. Tap on install
# * U1 login page appears
# 6. Login to u1
# * Login window goes app starts to download
# 7. Tap on Open
# * Application opens

import base64
import logging
import psutil
import subprocess
import uuid

import ubuntuuitoolkit
from fixtures import EnvironmentVariable
from autopilot.introspection import get_proxy_object_for_existing_process
from autopilot.matchers import Eventually
from testtools.matchers import Equals
from ubuntu_system_tests.helpers import (
    click,
    ubuntuone_credentials
)
from ubuntu_system_tests.helpers.unity8 import dash
from ubuntu_system_tests.tests import base

# XXX Imported to load the custom proxy objects. We need to import the dash
# before import things from the click scope.
from ubuntu_system_tests.helpers import unityclickscope  # NOQA
from unityclickscope import credentials


logger = logging.getLogger(__name__)


class AppStoreTestCase(base.BaseUbuntuSystemTestCase):

    OSM_TOUCH_APP_ID = 'me.yohanboniface.osmtouch_OSMTouch'
    OSM_PACKAGE_NAME = 'me.yohanboniface.osmtouch'

    def setUp(self):
        super().setUp()
        existing_account_ids = get_existing_account_ids()
        self.addCleanup(remove_accounts, ids_to_keep=existing_account_ids)

        # Make sure that online services is stopped and we have a testability
        # one running.
        kill_online_service_process()
        self.launch_online_services_with_testability()

    def launch_online_services_with_testability(self):
        self.useFixture(EnvironmentVariable('QT_LOAD_TESTABILITY', '1'))
        self.useFixture(EnvironmentVariable('OAU_DAEMON_TIMEOUT', '9999'))
        new_process = subprocess.Popen(
            ['online-accounts-service'],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE
        )
        self.addCleanup(new_process.kill)

    def test_install_free_app(self):
        """Create U1 account and install application from store (OSMTouch)"""
        unity_proxy = self.restart_unity()

        dash_proxy = dash.get_dash()
        apps_scope = dash_proxy.open_apps_scope()

        store_scope = apps_scope.go_to_store()
        store_scope.enter_search_query('OSMTouch')

        preview = store_scope.open_preview('appstore', 'OSMTouch')

        preview.install()
        self.addCleanup(self._ensure_osm_uninstalled)

        # Login into Ubuntu One dialog.
        self.log_into_u1_account()

        # Wait for the accounts dialog to disappear.
        self.assertThat(
            unity_proxy.get_number_of_sessions, Eventually(Equals(1)))

        # Wait for the application to start installing.
        dash_proxy.wait_for_processing_to_complete()
        preview.wait_until_package_installed()

        preview.open()
        self.addCleanup(self._ensure_osmtouch_closed)

        self.assertThat(
            unity_proxy.main_window.get_current_focused_app_id,
            Eventually(Equals(self.OSM_TOUCH_APP_ID)))

    def log_into_u1_account(self):
        """Log into u1 signon-u1 dialog that pops up."""
        account_id = generate_b64_uuid()
        email_address = 'u1test+{}@canonical.com'.format(account_id)
        user_name = 'Ubuntu Tester {}'.format(account_id)
        password = 'Hola123*'

        dialog_pid = self.get_signon_dialog_pid()
        dialog = get_proxy_object_for_existing_process(
            pid=dialog_pid,
            emulator_base=ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase)

        account_form = dialog.select_single(ubuntuone_credentials.NewAccount)
        account_form.register_new_account(
            email_address, user_name, password, password, True)

    def get_signon_dialog_pid(self):
        """Return the pid of the running signon dialog.

        :raises MismatchError: If the process cannot be found within the normal
          Eventually matcher time limit.

        """
        def get_process():
            return [x for x in psutil.process_iter()
                    if x.name() == 'online-accounts-ui']
        self.assertThat(
            lambda: len(get_process()),
            Eventually(Equals(1)),
            message='Unable to find the online-accounts-ui process')

        process = get_process()[0]
        return process.pid

    def _ensure_osmtouch_closed(self):
        """Stop the app before unity8 is restarted so it's not orphaned.

        Bug I came across (this should solve that)
        https://bugs.launchpad.net/ubuntu/+source/unity8/+bug/1401361

        Reference bug:
        https://bugs.launchpad.net/ubuntu/+source/unity8/+bug/1326513

        """
        try:
            app_triplet = subprocess.check_output(
                ['ubuntu-app-triplet', self.OSM_PACKAGE_NAME],
                universal_newlines=True).strip()
            subprocess.call(
                ['ubuntu-app-stop', app_triplet]
            )
        except subprocess.CalledProcessError:
            logger.debug('Error stoping app')

    def _ensure_osm_uninstalled(self):
        package_name = self.OSM_PACKAGE_NAME

        click.ensure_package_uninstalled(package_name)


def generate_b64_uuid():
    """Return a 22 character base 64 url-safe uuid."""
    # get a uid, convert to base 64 and truncate == from end
    uid = uuid.uuid1().bytes
    return base64.urlsafe_b64encode(uid).decode().strip('=')


def get_existing_account_ids():
    """Return a list of the ids of the existing accounts."""
    accounts_manager = credentials.AccountManager()
    account_services = accounts_manager._manager.get_account_services()
    return [service.get_account().id for service in account_services]


def remove_accounts(ids_to_keep=None):
    """Remove the accounts that are logged in on the device.

    :param ids_to_keep: list of accounts that will not be removed.

    """
    if ids_to_keep is None:
        ids_to_keep = []
    accounts_manager = credentials.AccountManager()
    account_services = accounts_manager._manager.get_account_services()
    for service in account_services:
        account = service.get_account()
        if account.id not in ids_to_keep:
            accounts_manager.delete_account(account)


def kill_online_service_process():
    try:
        service_process = [
            x for x in psutil.process_iter()
            if x.name() == 'online-accounts-service'
        ][0]
        service_process.kill()
    except IndexError:
        logger.debug('Error: Process is not running')
    except psutil.NoSuchProcess:
        logger.debug(
            'Error: Process died between finding it and trying to kill it')
