# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import time

from autopilot.exceptions import StateNotFoundError
from autopilot.matchers import Eventually
from datetime import datetime
from testtools import skipUnless
from testtools.matchers import Equals
from threading import Event

from ubuntu_system_tests.helpers.addressbook import(
    Contact,
    fixture_setup as address_book_fixture
)
from ubuntu_system_tests.helpers import indicators
from ubuntu_system_tests.helpers import ofono
from ubuntu_system_tests.helpers.dialer_app import fixture_setup
from ubuntu_system_tests.helpers.mediaplayer import start_media_player_observer
from ubuntu_system_tests.helpers.system_settings.fixture_setup import SilentMode  # NOQA
from ubuntu_system_tests.helpers.timedate import get_current_utc_time
from ubuntu_system_tests.helpers.vibration import start_vibration_observer
from ubuntu_system_tests.tests.base import wait_until
from ubuntu_system_tests.tests.base_telephony import BaseTelephonyTestCase


CONTACT_NAME_1 = 'Fulano de Tal'

CALL_URL_SHORT = 'http://people.canonical.com/~platform-qa/call_short.xml'
CALL_OUTGOING_API = 'outbound-api'
CALL_OUTGOING_DIAL = 'outbound-dial'
CALL_INCOMING = 'inbound'
CALL_NO_ANSWER = 'no-answer'
CALL_COMPLETED = 'completed'
CALLING_FROM = 'Calling from '
MISSED_CALL = 'missed call'

KEY_DEVICE_PASSWORD = 'device_password'

ERR_CALL_LOG_COUNT = ('Error: Expected 1 call to be present in call log '
                      'from: "{from_}" to: "{to}". {count} calls were '
                      'reported.')


class CallsTestCase(BaseTelephonyTestCase):

    @skipUnless(ofono.is_sim_inserted(), 'No SIM inserted')
    def setUp(self):
        super().setUp()
        self.useFixture(fixture_setup.DialerTestEnvironment())
        self.useFixture(SilentMode(False))
        self.contact_1 = Contact(name=CONTACT_NAME_1,
                                 number=self.get_telephony_service_number1())
        self.useFixture(
            address_book_fixture.AddTemporaryContacts([self.contact_1]))

    def _duration_to_datetime(self, duration):
        """
        Convert provided duration string into datetime object.

        :param duration: Duration string in MM:SS format
        :return: datetime object of specified duration

        """
        return datetime.strptime(duration, '%M:%S')

    def _is_greater_than(self, value, target):
        """Return true if value is greater than target."""
        return value > target

    def _validate_call_started(self, call_page, caller_id, second_count=2):
        """
        Validate that a call has started successfully. Check that the caller id
        is displayed, the hang-up button is enabled and that the call counter
        is counting.

        :param call_page: The LiveCall page object to validate
        :param caller_id: The expected name of the caller
        :param second_count: The number of timer second events to check. This
                             is the number of times the second counter should
                             be updated.

        """
        self.assertThat(
            call_page.caller, Eventually(Equals(caller_id)))

        call_page._get_hangup_button().enabled.wait_for(True)

        stop_watch = call_page.get_stop_watch()
        for count in range(0, second_count):
            start = self._duration_to_datetime(stop_watch.elapsed_time())
            wait_until(
                self._is_greater_than,
                timeout=3,
                period=1,
                value=self._duration_to_datetime(stop_watch.elapsed_time()),
                target=start)

    def _validate_call_ended(self, call_page, timeout=30):
        """
        Validate that a call ends successfully.

        :param call_page: The LiveCall page object to validate
        :param timeout: How long to wait for the call to end

        """
        try:
            call_page.visible.wait_for(False, timeout=timeout)
        except StateNotFoundError:
            # This just means the call page has already closed
            pass

    def _validate_telephony_call(self, call_id, from_, to, direction,
                                 status=CALL_COMPLETED):
        """
        Validate the call logs on the telephony service.

        :param call_id: Id value for the call
        :param from_: Phone number used by telephony service to make call
        :param to: Phone number of device receiving the call
        :param direction: Direction of the call
        :param status: Status of the call

        """
        call = self.telephony.get_call(call_id)
        self.assertEqual(call['status'], status)
        self.assertEqual(call['direction'], direction)
        self.assertEqual(call['to'], to)
        self.assertEqual(call['from'], from_)

    def _call_contact_from_address_book(self, required_contact):
        """
        Helper to launch the address book app and call a contact.

        :param required_contact: Contact object holding name and number to call
        :return: Dialer app live call page proxy object.

        """
        address_book = self.launch_address_book()
        contacts_page = address_book.get_contact_list_page()
        contact = contacts_page.open_contact_by_name(required_contact.name)
        contact.call(required_contact.number)
        dialer_app = self.get_dialer_app_proxy()
        dialer_app.dialer_page.click_call_button()
        return dialer_app.live_call_page

    def _clear_notifications(self):
        """Clear any existing notification messages displayed in indicator."""
        self.shell.clear_notifications()
        self.assertFalse(self.shell.is_new_notification_displayed())

    def _get_incoming_call_notification(self, timeout=30):
        """Wait for incoming call notification to appear and return it."""
        notifications_list = self.shell.get_notifications_list()
        return notifications_list.get_incoming_call_notification(timeout)

    def _validate_call_notification(self, call_notifier, contact):
        """
        Validate that the call notifier displays correctly and return the
        caller's phone number as displayed in the notification.
        :param call_notifier: Call notifier object to validate
        :param contact: Expected contact details
        :return: Phone number as displayed by the notification

        """
        self.assertThat(call_notifier.get_caller_name(),
                        Eventually(Equals(contact.name)))
        caller_msg = call_notifier.get_caller_message()
        self.assertTrue(caller_msg.startswith(CALLING_FROM))
        caller_number = caller_msg[len(CALLING_FROM):]
        return caller_number

    def _get_missed_call_message(self, contact):
        """
        Validate that there is a missed call message from specified contact
        displayed in the messages indicator and return it.
        :param contact: The contact who made the missed call.
        :return: Message object for the missed call.

        """
        self.assertTrue(self.shell.is_new_notification_displayed())
        messages = self.shell.open_notifications_indicator()
        message = messages.get_message(sender=contact.name)
        self.assertIn(MISSED_CALL, message.body)
        return message

    def _create_observers(self):
        """Create observers used to watch for sound and vibrate events."""
        self.sound_played = Event()
        self.mediaplayer_observer = start_media_player_observer(
            self.sound_played)
        self.addCleanup(self.mediaplayer_observer.stop)
        self.vibra_used = Event()
        self.vibra_observer = start_vibration_observer(self.vibra_used)
        self.addCleanup(self.vibra_observer.stop)

    def _reset_observers(self):
        """Reset the sound and vibration observer results."""
        self.sound_played.clear()
        self.vibra_used.clear()

    def _receive_missed_call_and_wait_for_hangup(self):
        """Receive call and wait for remote end to hangup without answering."""
        self.telephony.make_phonecall(
            to=self.get_device_phone_number(),
            from_=self.contact_1.number,
            hangup_timeout=5,
            call_url=CALL_URL_SHORT)
        self._get_incoming_call_notification().wait_until_destroyed()

    def test_receive_incoming_call_from_contact(self):
        """
        Test to recive an incoming call from a contact.
        1. Receive incoming call from a contact
        2. Swipe to answer the call
        3. Validate that the call information is correct
        4. Wait for call to be terminated from remote end

        """
        call = self.telephony.make_phonecall(
            to=self.get_device_phone_number(),
            from_=self.contact_1.number,
            hangup_timeout=10,
            call_url=CALL_URL_SHORT)

        call_notifier = self._get_incoming_call_notification()
        self._validate_call_notification(call_notifier, self.contact_1)

        # Let the phone ring briefly
        time.sleep(1)
        call_notifier.start_call()

        call_page = self.get_dialer_app_proxy().live_call_page
        self._validate_call_started(call_page, self.contact_1.name)
        self._validate_call_ended(call_page)
        self._validate_telephony_call(
            call['sid'],
            from_=self.contact_1.number,
            to=self.get_device_phone_number(),
            direction=CALL_OUTGOING_API)

    def test_return_missed_call_using_indicator(self):
        """
        Test to reply to a missed call from a contact.
        1. Receive incoming call from a contact.
        2. Do not answer call and wait for caller to hang up.
        3. Check that the messages notifier is updated.
        4. Swipe down messages notifier and press call icon to return call.
        5. Ensure that dialer app loads with the contact's name and number
           ready to be called.

        """
        self._clear_notifications()
        call = self.telephony.make_phonecall(
            to=self.get_device_phone_number(),
            from_=self.contact_1.number,
            hangup_timeout=5,
            call_url=CALL_URL_SHORT)

        call_notifier = self._get_incoming_call_notification()
        caller_number = self._validate_call_notification(
            call_notifier, self.contact_1)
        call_notifier.wait_until_destroyed()

        self._get_missed_call_message(self.contact_1).call()
        dialer_page = self.get_dialer_app_proxy().dialer_page
        self.assertThat(dialer_page.get_entered_phone_number(),
                        Eventually(Equals(caller_number)))

        self._validate_telephony_call(
            call['sid'],
            from_=self.contact_1.number,
            to=self.get_device_phone_number(),
            direction=CALL_OUTGOING_API,
            status=CALL_NO_ANSWER)

    def test_return_missed_call_device_locked(self):
        """
        Test to reply to a missed call from a contact when device is locked.
        1. Receive incoming call from a contact.
        2. Do not answer call and wait for caller to hang up.
        3. Check that the messages notifier is updated.
        4. Swipe down messages notifier and press the contact avatar to
           display snap decision menu.
        5. Press 'Call back' button to return call.
        6. Unlock the greeter by entering passphrase.
        7. Check that the dialer-app is loaded showing the correct phone
           number already inserted.

        """
        self._clear_notifications()
        call = self.telephony.make_phonecall(
            to=self.get_device_phone_number(),
            from_=self.contact_1.number,
            hangup_timeout=5,
            call_url=CALL_URL_SHORT)

        call_notifier = self._get_incoming_call_notification()
        caller_number = self._validate_call_notification(
            call_notifier, self.contact_1)
        call_notifier.wait_until_destroyed()

        self._get_missed_call_message(self.contact_1).press_avatar()
        self.shell.get_missed_call_snap_decision().call_back()
        self.shell.get_passphrase_unlock_screen().unlock(
            self.config_stack.get(KEY_DEVICE_PASSWORD))

        dialer_page = self.get_dialer_app_proxy().dialer_page
        self.assertThat(dialer_page.get_entered_phone_number(),
                        Eventually(Equals(caller_number)))

        self._validate_telephony_call(
            call['sid'],
            from_=self.contact_1.number,
            to=self.get_device_phone_number(),
            direction=CALL_OUTGOING_API,
            status=CALL_NO_ANSWER)

    def test_call_contact_using_address_book(self):
        """
        Test to call a contact from the address book application.
        1. Open address book app and open contact page
        2. Press call button on required phone number
        3. In dialer-app press call button
        4. After call has started hang-up the call by pressing hang up button
        5. Check logs from telephony service to ensure they are correct

        """
        start_time = get_current_utc_time()
        call_page = self._call_contact_from_address_book(self.contact_1)
        self._validate_call_started(call_page, self.contact_1.name)

        self.start_notification_observer()
        call_page.click_hangup_button()
        self._validate_call_ended(call_page)
        self.stop_notification_observer()
        end_time = get_current_utc_time()

        call_log = self.telephony.get_calls(
            from_=self.get_device_phone_number(),
            to=self.contact_1.number,
            time_from=start_time,
            time_to=end_time)

        self.assertEqual(len(call_log), 1, ERR_CALL_LOG_COUNT.format(
            from_=start_time, to=end_time, count=len(call_log)))

        self._validate_telephony_call(
            call_id=call_log[-1]['sid'],
            from_=self.get_device_phone_number(),
            to=self.contact_1.number,
            direction=CALL_INCOMING)

    def test_incoming_call_silent_mode(self):
        """
        Test to receive an incoming call whilst silent mode is activated.
        1. Open sound indicator.
        2. Turn silent mode on and close indicator.
        3. Receive incoming phone call.
        4. Ensure no sound is played but the phone still vibrates.
        5. Open sound indicator.
        6. Turn off silent mode and close the indicator.
        7. Receive incoming phone call.
        8. Ensure ringtone is played and the phone vibrates.

        """
        self._create_observers()
        indicator = indicators.Indicator(
            self.shell, indicators.SOUND_INDICATOR,
            indicators.SOUND_INDICATOR_PAGE)
        indicator_page = indicator.open()
        self.addCleanup(indicator.ensure_closed)

        self.assertFalse(indicator_page.get_silent_mode_switcher().checked)
        indicator_page.enable_silent_mode()
        self.assertTrue(indicator_page.get_silent_mode_switcher().checked)
        indicator.close()

        self._reset_observers()
        self._receive_missed_call_and_wait_for_hangup()
        self.assertFalse(self.sound_played.is_set())
        self.assertTrue(self.vibra_used.is_set())

        indicator_page = indicator.open()
        self.addCleanup(indicator.ensure_closed)
        indicator_page.disable_silent_mode()
        self.assertFalse(indicator_page.get_silent_mode_switcher().checked)
        indicator.close()

        self._reset_observers()
        self._receive_missed_call_and_wait_for_hangup()
        self.assertTrue(self.sound_played.is_set())
        self.assertTrue(self.vibra_used.is_set())
