# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from datetime import datetime, timedelta
from threading import Event

from ubuntu_system_tests.tests import base

from ubuntu_system_tests.helpers.calendar.fixture_setup import (
    BackupRestoreCalendarFixture)
from ubuntu_system_tests.helpers.clock.fixture_setup import (
    SetClockAccessRequests)
from ubuntu_system_tests.helpers.vibration import start_vibration_observer


class ClockTestCase(base.BaseUbuntuSystemTestCase):
    """This Class tests the clock app"""

    def setUp(self):
        super().setUp()

        self.useFixture(SetClockAccessRequests(location=True))
        self.useFixture(BackupRestoreCalendarFixture())
        self.unity_proxy = self.restart_unity()
        self.clock = self.launch_clock_app()

    def _create_vibration_observer(self):
        """ Create observer used to watch for vibration event """
        self.vibra_used = Event()
        self.vibra_observer = start_vibration_observer(self.vibra_used)
        self.addCleanup(self.vibra_observer.stop)

    def _reset_vibration_observer(self):
        self.vibra_used.clear()

    def test_phone_trigger_alarm_using_clock_app(self):
        """ Test That an alarm can be set in the clock app and the alarm is
        triggered correctly showing the time information in a notification
        """
        date = datetime.now() + timedelta(minutes=2)
        day = date.strftime("%A")

        # Prepare the observers
        self._create_vibration_observer()

        # Configure the alarm for 2 minutes in advance
        alarm = self.clock.open_alarm()
        alarm.add_single_alarm('test', [day], date, 'Bliss')

        # Reset the observers
        self._reset_vibration_observer()

        # Wait until the alarm is triggered and a notification is displayed
        notifications = self.unity_proxy.main_window.get_notifications_list()
        notif = notifications.get_alarm_notification(timeout=120)
        self.assertIn(date.strftime("%-I:%M %p"), notif.summary)

        # Validate the notification is dismissed
        notif.dismiss()
        notif.wait_until_destroyed()

        # Validate the vibration event was triggered
        self.assertTrue(self.vibra_used.is_set)
