# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import glob
import logging
import os
import subprocess

from ubuntuuitoolkit import fixture_setup

from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.support.ui import WebDriverWait

from testtools.matchers import GreaterThan

from ubuntu_system_tests.helpers.unity8.dash import get_dash
from ubuntu_system_tests.tests import base

DEFAULT_WEBVIEW_INSPECTOR_IP = '127.0.0.1'
DEFAULT_WEBVIEW_INSPECTOR_PORT = 9221


logger = logging.getLogger(__name__)


class HTML5ApplicationTest(base.BaseUbuntuSystemTestCase):

    def setUp(self):
        super().setUp()
        self.setup_environment()

    def test_html5_application(self):
        """The EBay application opens and all content is properly displayed."""
        unity_proxy = self.launch_unity()
        unity_proxy.unlock()

        self.launch_webapp('eBay')
        driver = self.setup_webdriver()
        homepage = Homepage(driver)
        search_results = homepage.search_for('ubuntu')
        total_results = search_results.results_count()

        self.assertThat(total_results, GreaterThan(1))

    def setup_environment(self):
        self.useFixture(fixture_setup.InitctlEnvironmentVariable(
            global_=True,
            UBUNTU_WEBVIEW_DEVTOOLS_HOST=DEFAULT_WEBVIEW_INSPECTOR_IP,
            UBUNTU_WEBVIEW_DEVTOOLS_PORT=str(DEFAULT_WEBVIEW_INSPECTOR_PORT)))

    def launch_webapp(self, app_name):
        apps_scope = get_dash().open_apps_scope()
        apps_scope.click_scope_item('local', app_name)

        self.addCleanup(ensure_webapp_closed)

    def setup_webdriver(self):
        options = get_driver_options()
        chromedriver_exec_path = get_chromedriver_exec_path()

        logger.debug(
            'Connecting to webdriver, using {}.'.format(
                chromedriver_exec_path))
        driver = webdriver.Chrome(
            executable_path=chromedriver_exec_path,
            chrome_options=options)

        self.addCleanup(driver.quit)

        return driver


def ensure_webapp_closed():
    try:
        webapp_id = subprocess.check_output(
            ['ubuntu-app-triplet',
             'com.ubuntu.developer.webapps.webapp-ebay'],
            universal_newlines=True).strip()

        subprocess.call(['ubuntu-app-stop', webapp_id])
    except subprocess.CalledProcessError:
        pass


def get_driver_options():
    options = Options()
    options.binary_location = ''
    options.debugger_address = '{ip}:{port}'.format(
        ip=DEFAULT_WEBVIEW_INSPECTOR_IP,
        port=DEFAULT_WEBVIEW_INSPECTOR_PORT)
    return options


def get_chromedriver_exec_path():
    """
    Returns the chromedriver excutable path

    Depends on the base installation directory and the current
    architecture

    :returns: a string containing the chromedriver executable path
    """
    base_deps_dir = os.path.join(
        os.path.dirname(os.environ['ADT_ARTIFACTS']),
        'deps')

    path_glob = '{base_dir}/usr/lib/*/oxide-qt/chromedriver'.format(
        base_dir=base_deps_dir)

    return glob.glob(path_glob)[0]


class BasePage():
    def __init__(self, driver):
        self.driver = driver


class Homepage(BasePage):
    _search_box_id = 'kw'

    def search_for(self, term):
        self.search_field = self.driver.find_element_by_id(
            self._search_box_id)
        self.search_field.clear()
        self.driver.execute_script(
            "document.getElementById("
            "'{}').setAttribute('value', '{}')".format(
                self._search_box_id, term)
        )
        self.search_field.submit()
        return SearchResults(self.driver)


class SearchResults(BasePage):
    _product_list_css = 'div.itemDiv pre.preTxt'

    def results_count(self):
        # the default wait isn't enough sometimes
        WebDriverWait(self.driver, 100).until(
            lambda driver: driver.find_element_by_css_selector(
                self._product_list_css))

        results = self.driver.find_elements_by_css_selector(
            self._product_list_css)
        return len(results)
