# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3,
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from autopilot.matchers import Eventually
from testtools import skipUnless
from testtools.matchers import Equals

from ubuntu_system_tests.tests.base_messaging import MessagingBase
from ubuntu_system_tests.helpers.gallery import (
    fixture_setup as gallery_fixture
)

from ubuntu_system_tests.helpers import file_system
from ubuntu_system_tests.helpers import ofono
from ubuntu_system_tests.helpers import web
from ubuntu_system_tests.helpers.messaging import Message, Attachment
from ubuntu_system_tests.helpers.uid import b64_uuid

SAMPLE_FILE = 'sample01.jpg'
TIMEOUT_5_MIN = 300


class MMSTestCase(MessagingBase):

    @skipUnless(ofono.is_sim_inserted(), 'No SIM inserted')
    def setUp(self):
        super().setUp()
        self.useFixture(gallery_fixture.GalleryTestEnvironment())

    def test_receive_mms_from_contact(self):
        """
        Prerequisite: A contact in the address book with the number you
        are sending from
        1. From another device send an MMS to the phone number of a SIM
           card that is in the SUT
           - A notification appears containing the senders
             name and the message content.
        2. Tap on the message notification before it disappears
           - The messaging application is opened showing all
             messages received by that sender. The message
             just sent should appear at the bottom.
           - An image should be contained within the message
        """
        message_body = b64_uuid()
        media_url = 'http://people.canonical.com/~platform-qa/mms-ubuntu.jpg'
        mms_media_file = web.download_file(media_url)
        notification = self.receive_message_and_get_notification(
            self.get_device_phone_number(),
            self.get_telephony_service_number1(),
            message_body,
            media_url=media_url,
            timeout=TIMEOUT_5_MIN)

        self.assertThat(notification.summary,
                        Eventually(Equals('Message from {contact}'.format(
                                          contact=self.contact_1.name))))
        self.assertThat(notification.body, Eventually(Equals(message_body)))
        notification.tap_on()

        self.get_messaging_app()
        self.assertThat(self.main_view.get_header().title,
                        Eventually(Equals(self.contact_1.name)))

        mms_message = self.main_view.get_message(message_body)
        photo_path = mms_message.get_mms_content_file_path()
        self.assertTrue(file_system.compare_files(mms_media_file, photo_path))

    def test_send_mms_with_photo_from_gallery(self):
        """
        Prerequisite: A contact in the address book with the number you
        are sending to and a photo in the gallery
        1. Open the messaging application
            - The messaging application is opened to the main screen
        2. Swipe up to create a new message
            - The keyboard appears with the message composition controls
              above it (add photo button, 'Write a message...' field)
              and a Send button. The To.. field has focus.
        3. Type the name of the contact to send to in the To.. field
            - The contacts name is stored in the To... field
        4. Type the message text in the Write a message..
           field - e.g. 'Hello World'
            - The message text is entered into the Write a message.. field
        5  Tap the "attach" button on the button left of the screen
            - Content hub opens with list of Apps we can select content from
        6. Tap 'Gallery' button
            - Gallery App opens
        7  On the main camera screen select a photo
           - Messaging App is brought to front with the newly selected
             image being shown besides the message text field
        8. Press the Send button
            - The message is added to the conversation and is shown in green
              containing the newly selected photo's thumbnail
        9. Confirm the MMS was received on the other device
            - The message was received with the correct text and photo
        """
        self.launch_messaging()
        attachment = Attachment(
            attach_method=self._add_attachment_from_gallery,
            identifier=SAMPLE_FILE)
        message = Message(body=b64_uuid(), attachments=[attachment])
        recipients = [self.contact_1]
        self.send_message_to_contacts_and_validate_replies(recipients, message,
                                                           SAMPLE_FILE)

    def test_send_mms_with_photo_from_camera(self):
        """
        Prerequisite: A contact in the address book with the number you
        are sending to
        1. Open the messaging application
            - The messaging application is opened to the main screen
        2. Swipe up to create a new message
            - The keyboard appears with the message composition controls
              above it (add photo button, 'Write a message...' field)
              and a Send button. The To.. field has focus.
        3. Type the name of the contact to send to in the To.. field
            - The contacts name is stored in the To... field
        4. Type the message text in the Write a message..
           field - e.g. 'Hello World'
            - The message text is entered into the Write a message.. field
        5  Tap the "attach" button on the button left of the screen
            - Content hub opens with list of Apps we can select content from
        6. Tap 'Camera' button
            - Camera App opens
        7  On the main camera screen tap the exposure button
           - New photo is taken and the exposure button is changed to
             "affirmative" icon
        8  Tap the affirmative icon
           - Messaging App is brought to front with the newly selected
             image being shown besides the message text field
        9. Press the Send button
            - The message is added to the conversation and is shown in green
              containing the newly selected photo's thumbnail
        10. Confirm the MMS was received on the other device
            - The message was received with the correct text and photo
        """
        self.launch_messaging()
        attachment = Attachment(
            attach_method=self._add_attachment_from_camera)
        message = Message(body=b64_uuid(), attachments=[attachment])
        recipients = [self.contact_1]
        self.send_message_to_contacts_and_validate_replies(recipients, message)

    def test_send_mms_with_photo_from_camera_photoroll(self):
        """
        Prerequisite: A contact in the address book with the number you
        are sending to and a photo in the camera roll
        1. Open the messaging application
            - The messaging application is opened to the main screen
        2. Swipe up to create a new message
            - The keyboard appears with the message composition controls
              above it (add photo button, 'Write a message...' field)
              and a Send button. The To.. field has focus.
        3. Type the name of the contact to send to in the To.. field
            - The contacts name is stored in the To... field
        4. Type the message text in the Write a message..
           field - e.g. 'Hello World'
            - The message text is entered into the Write a message.. field
        5  Tap the "attach" button on the button left of the screen
            - Content hub opens with list of Apps we can select content from
        6. Tap 'Camera' button
            - Camera App opens
        7  On the main camera screen swipe left
           - Photo roll opens with the a photo visible
        8  Select the sample photo
           - Messaging App is brought to front with the newly selected
             image being shown besides the message text field
        9. Press the Send button
            - The message is added to the conversation and is shown in green
              containing the newly selected photo's thumbnail
        10. Confirm the MMS was received on the other device
            - The message was received with the correct text and photo
        """
        self.launch_messaging()
        attachment = Attachment(
            attach_method=self._add_attachment_from_camera_photoroll)
        message = Message(body=b64_uuid(), attachments=[attachment])
        recipients = [self.contact_1]
        self.send_message_to_contacts_and_validate_replies(recipients, message,
                                                           SAMPLE_FILE)
