# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging
import os

from testtools import skipUnless
from ubuntu_system_tests.helpers.data import load_test_metadata
from ubuntu_system_tests.helpers import file_system as fs
from ubuntu_system_tests.helpers.mtp import MTPObjFactory, MTPHelper
from ubuntu_system_tests.helpers import ssh
from ubuntu_system_tests.helpers import media
from ubuntu_system_tests.helpers.scopes import music
from ubuntu_system_tests.helpers.unity8.dash import get_dash

from ubuntu_system_tests.tests import base

logger = logging.getLogger(__name__)
songs_album_mp3 = ['song_1.mp3', 'song_2.mp3', 'song_3.mp3']
songs_album_ogg = ['song_1.ogg', 'song_2.ogg', 'song_3.ogg']

MUSIC_EXT = '.mp3'
VIDEO_EXT = '.avi'
PICT_EXT = '.jpg'
TMP_EXT = '.tmp'
SD_CARD_REASON = 'The SD card is not inserted in the device'

data = load_test_metadata()


class MTPTest(base.BaseUbuntuSystemTestCase):
    """This Class tests the mtp protocol"""

    def setUp(self):
        super().setUp()

        self.unity_proxy = self.restart_unity()
        self.remote_helper = ssh.LinuxRemoteHelper()
        self.addCleanup(self.remote_helper.close)
        self.factory = MTPObjFactory(self.remote_helper, self.addCleanup)
        self.mtp_helper = MTPHelper(self.remote_helper)

    def verify_music_artist_is_shown(self, artist):
        """ Determines if there is an artist with the desired name in the music
        scope
        :param album: The name ob the desired album
        :return: True is there is an artist with the desired name
        """
        music_scope = music.MusicScope(get_dash().open_music_scope())
        self.assertTrue(music_scope.is_artist(artist))

    def verify_same_files(self, src_files, dst_files):
        """ Verify both file lists contains the same files
        :param src_files: list of src file names
        :param dst_files: list of dst file names
        """
        self.assertEqual(sorted(src_files), sorted(dst_files))

    def verify_files_match(self, src_file, dst_dir, dst_files):
        """ Verify the src file is equal to the dst files in the dst_dir
        :param src_file: file from where the original files were copied
        :param dst_dir: name of the dir copied to the device
        :param dst_files: list of files copied to the copied in the device
        """
        for file in dst_files:
            self.assertTrue(src_file.compare(os.path.join(dst_dir, file)))

    def prepare_music_albums(self, mp3_file, ogg_file):
        """ Prepare the albums in the remote host with the song to be copied
         It creates one dir with two albums inside, each album has 3 songs with
         mp3 and ogg formats respectively
        """

        # Create music albums structure in the host machine
        albums_dir = self.factory.create_remote_dir()
        album_mp3_dir = self.factory.create_remote_dir(
            path=albums_dir.remote_src_path)
        album_ogg_dir = self.factory.create_remote_dir(
            path=albums_dir.remote_src_path)

        # Copy the music files to the ubuntu host machine
        mp3_file.send_to_host(fs.DIR_TEMP)
        ogg_file.send_to_host(fs.DIR_TEMP)

        # Copy the music files in the ubuntu host machine to the albums
        # several times with different names and formats
        self.mtp_helper.replicate_file(mp3_file, album_mp3_dir,
                                       songs_album_mp3)
        self.mtp_helper.replicate_file(ogg_file, album_ogg_dir,
                                       songs_album_ogg)

        return albums_dir, album_mp3_dir, album_ogg_dir

    def backup_local_media(self):
        """Backup the local media files which are removed during testing."""
        self.useFixture(media.BackupRestoreLocalMediaFixture())
        media.restart_media_scanner()

    def verify_copied_files(self, seed_file, original_files, copied_dir):
        """ Verify all the files have been copied correctly to the dst
        :param seed_file: this is the test data file that is used to generate
        the test data for the test. This is used to verify the integrity of
        the files after those are copied through mtp.
        :param original_files: list of files that were not copied
        :param copied_dir: mtp dir to where the original files were copied
        """
        copied_files = fs.files_in_dir(copied_dir)

        self.verify_same_files(original_files, copied_files)
        self.verify_files_match(seed_file, copied_dir, copied_files)

    def verify_songs(self, seed_song, original_songs, copied_album, artist):
        """ Verify all the songs have been copied for each album correctly and
        verify the artist is in the music scope
        """
        self.verify_copied_files(seed_song, original_songs, copied_album)
        self.verify_music_artist_is_shown(artist)

    def test_copy_music(self):
        """ Copy through mtp a file to the music dir and check the file has
        been copied to the correct dir and the content is the same to the
        original one

        This test initially creates through ssh two temporal music albums in
        the remote host machine with mp3 and ogg songs respectively. Then,
        these music albums are copied to the gvfs dir which maps the device
        music directory. Once those are copied, it is checked that the music
        albums have been copied properly to the device and the music appears
        in the music scope.
        """
        # Remove any existing local media from device
        self.backup_local_media()
        # Prepare the albums structure in the remote machine previous to be
        # copied to the device
        mp3_file = self.factory.create_file(
            os.path.join(fs.DIR_TEST_DATA_AUDIO, data.audio.mp3_file.file))
        ogg_file = self.factory.create_file(
            os.path.join(fs.DIR_TEST_DATA_AUDIO, data.audio.ogg_file.file))

        albums_dir, album_mp3_dir, album_ogg_dir = self.prepare_music_albums(
            mp3_file, ogg_file)

        # Copy the albums directory from the temporal dir in the ubuntu host
        # machine to the dir which maps the music dir in the device
        mtp_albums_path = self.mtp_helper.get_mtp_path(fs.DIR_MUSIC,
                                                       albums_dir.name)
        albums_dir.remote_copy(mtp_albums_path)

        # Verify the albums have ben copied to the music folder
        albums_dir.local_dst_path = os.path.join(fs.DIR_HOME_MUSIC,
                                                 albums_dir.name)
        self.assertTrue(os.path.isdir(albums_dir.local_dst_path))

        dst_path = os.path.join(albums_dir.local_dst_path, album_mp3_dir.name)
        self.verify_songs(mp3_file, songs_album_mp3, dst_path,
                          data.audio.mp3_file.artist)

        dst_path = os.path.join(albums_dir.local_dst_path, album_ogg_dir.name)
        self.verify_songs(ogg_file, songs_album_ogg, dst_path,
                          data.audio.ogg_file.artist)

    def test_copy_large_number_of_files(self):
        """ Copy big amount of files through mtp and check all the files have
        been copied to the correct dir and the content is the same to the
        original.
        """
        amount = 100
        myfile = self.factory.create_file(
            os.path.join(fs.DIR_TEST_DATA_TEXT, data.text.text_file))

        # Create the dir to store the files in the host machine
        mydir = self.factory.create_remote_dir()

        # Copy the src file to the ubuntu host machine
        myfile.send_to_host(fs.DIR_TEMP)

        # Replicate the src file 100 times
        original_files = [myfile.name + "_" + str(i) for i in range(amount)]
        self.mtp_helper.replicate_file(myfile, mydir, original_files)

        # Copy the directory from the temporal dir in the ubuntu host
        # machine to the dir which maps the documents dir in the device
        mydir.remote_copy(self.mtp_helper.get_mtp_path(fs.DIR_DOCUMENTS,
                                                       mydir.name))

        # Verify the files have ben copied correctly to the destiation
        mydir.local_dst_path = os.path.join(fs.DIR_HOME_DOCUMENTS, mydir.name)
        self.assertTrue(os.path.isdir(mydir.local_dst_path))
        self.verify_copied_files(myfile, original_files, mydir.local_dst_path)

    def _test_copy_big_file(self, kb=0, mb=0, gb=0):
        # Create the remote file with random content and get the sha1 digest
        file = self.factory.create_remote_file(ext=TMP_EXT, kb=kb, mb=mb,
                                               gb=gb)
        # Copy the directory from the temporal dir in the ubuntu host
        # machine to the dir which maps the documents dir in the device
        file.remote_copy(self.mtp_helper.get_mtp_path(fs.DIR_DOCUMENTS,
                                                      file.name))

        # Verify the files have ben copied correctly to the destiation
        file.local_dst_path = os.path.join(fs.DIR_HOME_DOCUMENTS, file.name)
        self.assertTrue(os.path.isfile(file.local_dst_path))

        copied_sha1 = fs.calculate_file_sha1(file.local_dst_path)
        self.assertEqual(file.sha1, copied_sha1)

    def test_copy_big_1gb_file(self):
        """ Copy a big file (1GB) through mtp and check the file has
        been copied to the correct dir and the content is the same to the
        original.
        """
        self._test_copy_big_file(gb=1)

    def test_copy_big_2gb_file(self):
        """ Copy a big file (2GB + 1KB) through mtp and check the file has
        been copied to the correct dir and the content is the same to the
        original.

        This test is failing and a bug has been created
        https://bugs.launchpad.net/ubuntu/+source/mtp/+bug/1472789
        """
        self._test_copy_big_file(gb=2, kb=1)

    def _test_copy_from_device(self, dir, dir_name, ext, mtp_path_fc):
        """ Copy a file from the device to the ubuntu host machine and
        validate the file has been copied successfully
        :param dir: The media dir which maps either to the internal or
        external storage
        :param dir_name: Media dir name such as Music, Videos, etc
        :param ext: Extension of the media file to be created with random
        content
        :param mtp_path_fc: Method to get the mtp media path in the ubuntu host
        """
        file = self.factory.create_random_file(dir, ext=ext)
        file.remote_src_path = mtp_path_fc(dir_name, file.name)

        # Set destination path to files
        file.remote_dst_path = os.path.join(fs.DIR_TEMP, file.name)

        # Copy the files from mtp to tmp directory and verify the sha1
        file.remote_copy()
        copied_sha1 = file.get_remote_dst_sha1()
        self.assertEqual(file.sha1, copied_sha1)

        # Remove remote files and verify the files were deleted in the device
        self.remote_helper.remove_file(file.remote_src_path)
        self.assertFalse(os.path.isfile(file.local_src_path))

    def test_copy_from_internal_storage(self):
        """ Copy Music, Videos and Pictures from the device to the ubuntu host
         machine and delete the files in the device from the host
        """
        self._test_copy_from_device(fs.DIR_HOME_MUSIC, fs.DIR_MUSIC,
                                    MUSIC_EXT, self.mtp_helper.get_mtp_path)
        self._test_copy_from_device(fs.DIR_HOME_VIDEOS, fs.DIR_VIDEOS,
                                    VIDEO_EXT, self.mtp_helper.get_mtp_path)
        self._test_copy_from_device(fs.DIR_HOME_PICTURES, fs.DIR_PICTURES,
                                    PICT_EXT, self.mtp_helper.get_mtp_path)

    @skipUnless(fs.is_media_folder_dir(fs.DIR_MUSIC), SD_CARD_REASON)
    def test_copy_from_sd_card(self):
        """ Copy Music, Videos and Pictures from the device sd card to the
        ubuntu host machine and delete the files in the device from the host

        Prerequisite: There is an sd card plugged in in the device

        The test is going to fail in case there is not any media storage
        connected to the device.
        """
        self._test_copy_from_device(fs.get_media_folder_dir(fs.DIR_MUSIC),
                                    fs.DIR_MUSIC, MUSIC_EXT,
                                    self.mtp_helper.get_mtp_media_path)
        self._test_copy_from_device(fs.get_media_folder_dir(fs.DIR_VIDEOS),
                                    fs.DIR_VIDEOS, VIDEO_EXT,
                                    self.mtp_helper.get_mtp_media_path)
        self._test_copy_from_device(fs.get_media_folder_dir(fs.DIR_PICTURES),
                                    fs.DIR_PICTURES, PICT_EXT,
                                    self.mtp_helper.get_mtp_media_path)

    def _test_copy_to_sd_card(self, ext, dir):
        file = self.factory.create_remote_file(ext=ext, sha1=True, kb=10)
        file.remote_copy(self.mtp_helper.get_mtp_media_path(dir))
        file.local_dst_path = os.path.join(fs.get_media_folder_dir(dir),
                                           file.name)
        self.assertTrue(os.path.isfile(file.local_dst_path))
        self.assertEqual(file.sha1,
                         fs.calculate_file_sha1(file.local_dst_path))

    @skipUnless(fs.is_media_folder_dir(fs.DIR_MUSIC), SD_CARD_REASON)
    def test_copy_to_sd_card(self):
        """ Copy Music, Videos and Pictures from the ubuntu host machine to the
        sd card on the device and check the media has been copied successfully

        Prerequisite: There is an sd card plugged in in the device

        The test is going to fail in case there is not any media storage
        connected to the device.
        """
        self._test_copy_to_sd_card(MUSIC_EXT, fs.DIR_MUSIC)
        self._test_copy_to_sd_card(VIDEO_EXT, fs.DIR_VIDEOS)
        self._test_copy_to_sd_card(PICT_EXT, fs.DIR_PICTURES)
