# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3,
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging
import os
import shutil

from testtools import skipUnless
from ubuntu_system_tests.tests import base
from ubuntu_system_tests.helpers import autopilot
from ubuntu_system_tests.helpers.data import load_test_metadata
from ubuntu_system_tests.helpers import file_system as fs
from ubuntu_system_tests.helpers import media
from ubuntu_system_tests.helpers import mediaplayer
from ubuntu_system_tests.helpers import music
from ubuntu_system_tests.helpers import processes
from ubuntu_system_tests.helpers import scopes
from ubuntu_system_tests.helpers.scopes import music as music_scope
from ubuntu_system_tests.helpers.scopes.apps import AppsScope, APP_STORE_TITLE
from ubuntu_system_tests.helpers.scopes.photos import PhotosScope
from ubuntu_system_tests.helpers.scopes.music import MusicScope
from ubuntu_system_tests.helpers.scopes.video import VideoScope
from ubuntu_system_tests.helpers.scopes import timedate
from ubuntu_system_tests.helpers.unity8.dash import get_dash
from ubuntu_system_tests.helpers import url_dispatcher
from ubuntu_system_tests.helpers import wait_until
from ubuntu_system_tests.helpers import web
from ubuntu_system_tests.helpers import webbrowser


data = load_test_metadata()

SRC_VIDEO_FILE_1 = os.path.join(fs.DIR_TEST_DATA_VIDEO, data.video.video_ogv)
SRC_VIDEO_FILE_2 = os.path.join(fs.DIR_TEST_DATA_VIDEO, data.video.video_avi)
SRC_AUDIO_FILE_1 = os.path.join(fs.DIR_TEST_DATA_AUDIO,
                                data.audio.ogg_file.file)
SRC_AUDIO_FILE_2 = os.path.join(fs.DIR_TEST_DATA_AUDIO,
                                data.audio.count20.file)
SRC_AUDIO_FILE_3 = os.path.join(fs.DIR_TEST_DATA_AUDIO,
                                data.audio.mp3_file.file)
SRC_IMAGE_FILE_1 = os.path.join(fs.DIR_TEST_DATA_IMAGES, data.images.img1)
SRC_IMAGE_FILE_2 = os.path.join(fs.DIR_TEST_DATA_IMAGES, data.images.img2)

LOCAL_VIDEO_FILE_1 = os.path.join(fs.DIR_HOME_VIDEOS, data.video.video_ogv)
LOCAL_VIDEO_FILE_2 = os.path.join(fs.DIR_HOME_VIDEOS, data.video.video_avi)
LOCAL_AUDIO_FILE_1 = os.path.join(fs.DIR_HOME_MUSIC, data.audio.ogg_file.file)
LOCAL_AUDIO_FILE_2 = os.path.join(fs.DIR_HOME_MUSIC, data.audio.count20.file)
LOCAL_AUDIO_FILE_3 = os.path.join(fs.DIR_HOME_MUSIC, data.audio.mp3_file.file)
LOCAL_IMAGE_FILE = os.path.join(fs.DIR_HOME_PICTURES, data.images.img1)

KEY_COMPARE_METHOD = 'compare_method'

FAIL_MSG_HEADERS = ('The number of headers in scope does not match expected '
                    'for scope id: {s} under condition: {c}.\n'
                    'Expected headers:\n{e}\nObserved headers:\n{o}')

FAIL_MSG_LABELS = ('Label: \'{l}\' not found for scope id: \'{s}\' under '
                   'condition: \'{c}\'.\n'
                   'Expected labels:\n{e}\n'
                   'Observed labels:\n{o}')

FAIL_MSG_ORDER = ('Label: \'{l}\' not in expected position for scope id: '
                  '\'{s}\' under condition: \'{c}\'.\n'
                  'Expected order:\n{e}\n'
                  'Observed order:\n{o}')
SKIP_REASON = 'Scope not required for the current device and channel'

THUMBNAILER_PREFIX = 'image://thumbnailer/file://'

channel = autopilot.get_channel_name()
device = autopilot.get_device_name()
expected_scopes = scopes.get_scope_list()
logger = logging.getLogger(__name__)


class ScopeSystemTest(base.BaseUbuntuSystemTestCase):
    """This Class tests the scopes"""

    def setUp(self):
        super().setUp()
        self.unity_proxy = self.restart_unity()
        self.dash = get_dash()
        logger.info('Running test using channel: {c} and device: {d}'.format(
            c=channel, d=device))
        self.assertTrue(
            len(expected_scopes) > 0,
            'No scopes defined for channel: {c} device: {d}'.format(
                c=channel, d=device))
        self.useFixture(media.BackupRestoreLocalMediaFixture())

        url_dispatcher.go_to_url(expected_scopes[0].url)
        self.addCleanup(url_dispatcher.go_to_url, expected_scopes[0].url)

    def _apply_condition_local_video(self):
        """Method to apply 'local_video' condition."""
        shutil.copy2(SRC_VIDEO_FILE_1, LOCAL_VIDEO_FILE_1)
        media.restart_media_scanner()

    def _apply_condition_no_local_video(self):
        """Method to apply 'no_local_video' condition."""
        fs.clean_dir(fs.DIR_HOME_VIDEOS)
        media.restart_media_scanner()

    def _apply_condition_local_photo(self):
        """Method to apply 'local_photo' condition."""
        shutil.copy2(SRC_IMAGE_FILE_1, LOCAL_IMAGE_FILE)
        media.restart_media_scanner()

    def _apply_condition_no_local_photo(self):
        """Method to apply 'no_local_photo' condition."""
        fs.clean_dir(fs.DIR_HOME_PICTURES)
        media.restart_media_scanner()

    def _apply_condition_local_music(self):
        """Method to apply 'local_music' condition."""
        shutil.copy2(SRC_AUDIO_FILE_1, LOCAL_AUDIO_FILE_1)
        media.restart_media_scanner()

    def _apply_condition_no_local_music(self):
        """Method to apply 'no_local_music' condition."""
        fs.clean_dir(fs.DIR_HOME_MUSIC)
        media.restart_media_scanner()

    def _apply_condition_none(self):
        """Method to apply 'none' condition."""
        pass

    def _apply_condition(self, condition):
        """Call the method which applies the specified condition.

        :param condition: The name of the condition method to apply.

        """
        getattr(self, '_apply_condition_{c}'.format(c=condition))()

    def _find_matching_items_in_list(self, list, required_item):
        """Return a list of indexes for matching items in the list."""
        return [i for i, x in enumerate(list) if x == required_item]

    def _validate_label_in_list(self, expected_label, observed_labels,
                                scope_id, condition, expected_labels):
        """
        Validate that expected_label appears in observed_labels list.

        :return: list of index values where expected_label matches label
                 in observed_labels list.

        """
        indexs = self._find_matching_items_in_list(observed_labels,
                                                   expected_label)
        self.assertGreater(len(indexs), 0,
                           FAIL_MSG_LABELS.format(l=expected_label,
                                                  s=scope_id,
                                                  c=condition,
                                                  e=expected_labels,
                                                  o=observed_labels))
        return indexs

    def _validate_label_in_list_with_custom_validation(self, validation_method,
                                                       expected_label,
                                                       observed_labels,
                                                       scope_id, condition,
                                                       expected_labels):
        """
        Validate that a label in observed_labels list can be matched using
        the specified validation method.
        :param validation_method: startswith, endswith or contains
        :return: list of indexes of matching label in observed_labels list

        """
        indexs = []
        index = 0
        for observed_label in observed_labels:
            validator = getattr(self, '_validate_string_{v}'.format(
                v=validation_method))
            if validator(observed_label, expected_label):
                indexs.append(index)
            index += 1
        self.assertTrue(len(indexs) > 0,
                        FAIL_MSG_LABELS.format(l=expected_label,
                                               s=scope_id,
                                               c=condition,
                                               e=expected_labels,
                                               o=observed_labels))
        return indexs

    def _validate_string_startswith(self, string, expected):
        """Custom validation method to check if string starts with expected."""
        return string.startswith(expected)

    def _validate_string_endswith(self, string, expected):
        """Custom validation method to check if string ends with expected."""
        return string.endswith(expected)

    def _validate_string_contains(self, string, expected):
        """Custom validation method to check if string contains expected."""
        return string.find(expected) >= 0

    def _validate_labels(self, observed_labels, expected_sections,
                         scope_id, condition):
        """
        Assert that the expected labels appear in the observed labels
        in the order specified.

        The number of labels observed could be greater than the expected
        labels because of the content being displayed. So it is
        not possible to perform an exact match between observed and expected
        labels.

        """
        previous_index = -1
        expected_labels = scopes.get_section_names_from_list(
            expected_sections)
        for expected_section, expected_label in zip(expected_sections,
                                                    expected_labels):
            if KEY_COMPARE_METHOD in expected_section:
                index_list = \
                    self._validate_label_in_list_with_custom_validation(
                        expected_section[KEY_COMPARE_METHOD],
                        expected_label, observed_labels, scope_id, condition,
                        expected_labels)
            else:
                index_list = self._validate_label_in_list(
                    expected_label, observed_labels, scope_id, condition,
                    expected_labels)
            # To keep the required order, expect that at least 1 item in
            # index_list will be greater than previous index.
            # Find this item from the list of matching indexs.
            current_index = self._get_next_index(index_list, previous_index)
            self.assertTrue(current_index > previous_index,
                            FAIL_MSG_ORDER.format(l=expected_label,
                                                  s=scope_id,
                                                  c=condition,
                                                  e=expected_labels,
                                                  o=observed_labels))
            previous_index = current_index

    def _get_next_index(self, index_list, previous_index):
        """
        Choose the next index from index_list based on the value of
        previous_index.

        In order to keep the required order, the next index should be greater
        than the previous index.
        :param index_list: A list of index items
        :param previous_index: The value of the previous index used
        :return: The index value selected from index_list, or -1 if not found

        """
        for index in index_list:
            if index > previous_index:
                return index
        return -1

    def _validate_headers_if_required(self, expected_scope, expected_sections,
                                      observed_headers, condition):
        """
        Check the header count matches expected, if it is required for
        this scope.

        """
        if expected_scope.check_header_count:
            # check that there are the expected number of header items
            expected_headers = scopes.get_section_header_names_from_list(
                expected_sections)
            self.assertEqual(
                len(expected_headers), len(observed_headers),
                FAIL_MSG_HEADERS.format(s=expected_scope.id,
                                        c=condition,
                                        e=expected_headers,
                                        o=observed_headers))

    def _test_scope(self, test_scope, dismiss_setup=True):
        """
        Verify all the scopes are in the correct order, the titles are
        correct and there are subsections for each one.

        """
        expected_scope = scopes.get_scope_data(test_scope)

        for condition in expected_scope.sections:
            logger.info('Testing scope {s} under condition {c}.'.format(
                s=expected_scope.title, c=condition))

            expected_sections = expected_scope.sections[condition]
            # apply required condition to scope
            self._apply_condition(condition)
            scope = scopes.go_to_scope(test_scope, dismiss_setup=dismiss_setup)
            self.dash.wait_for_processing_to_complete()
            # check title is as expected
            self.assertEqual(expected_scope.title, scope.get_title())
            # get lists of observed and expected labels
            observed_headers, observed_labels = scope.get_all_content(
                scopes.is_image_expected_in_sections(expected_sections))
            # check that labels appear in order expected
            self._validate_labels(observed_labels,
                                  expected_sections,
                                  expected_scope.title,
                                  condition)
            # some scopes disable the header count check, so only check
            # the number of headers if requried
            self._validate_headers_if_required(
                expected_scope, expected_sections, observed_headers,
                condition)

    def test_scopes_navigation(self):
        """ Verify the navigation through the scopes and their order
        navigating from the left to the right and then from the right to the
        left. The total number of scopes is also validated.
        """
        scope_counter = 0
        expected_scope_count = len(expected_scopes)

        # repeat for each scope from left to right
        for expected_scope in expected_scopes:
            scope = self.dash.get_current_scope()
            self.assertEqual(expected_scope.title, scope.get_title())

            scope_counter += 1
            if expected_scope_count > scope_counter:
                self.dash.scroll_to_right_scope()

        self.assertEqual(expected_scope_count, self.dash.get_scope_count(),
                         'The observed scope count does not match expected.')

        # repeat for each scope from right to left avoiding the last one
        for expected_scope in reversed(expected_scopes[:-1]):
            self.dash.scroll_to_left_scope()

            scope = self.dash.get_current_scope()
            self.assertEqual(expected_scope.title, scope.get_title())

    @skipUnless(scopes.is_scope_included(
        expected_scopes, scopes.SCOPE_ID_TODAY), SKIP_REASON)
    def test_today_scope(self):
        """ Verify that for the today scope the titles the subsections are
        correct for the predefined conditions.
        """
        self._test_scope(scopes.SCOPE_ID_TODAY)

    @skipUnless(scopes.is_scope_included(
        expected_scopes, scopes.SCOPE_ID_NEARBY), SKIP_REASON)
    def test_nearby_scope(self):
        """ Verify that for the nearby scope the titles the subsections are
        correct for the predefined conditions.
        """
        self._test_scope(scopes.SCOPE_ID_NEARBY)

    @skipUnless(scopes.is_scope_included(
        expected_scopes, scopes.SCOPE_ID_APPS), SKIP_REASON)
    def test_apps_scope(self):
        """ Verify that for the apps scope the titles the subsections are
        correct for the predefined conditions.
        """
        self._test_scope(scopes.SCOPE_ID_APPS)

    @skipUnless(scopes.is_scope_included(
        expected_scopes, scopes.SCOPE_ID_NEWS), SKIP_REASON)
    def test_news_scope(self):
        """ Verify that for the news scope the titles the subsections are
        correct for the predefined conditions.
        """
        self._test_scope(scopes.SCOPE_ID_NEWS)

    @skipUnless(scopes.is_scope_included(
        expected_scopes, scopes.SCOPE_ID_PHOTOS), SKIP_REASON)
    def test_photos_scope(self):
        """ Verify that for the photos scope the titles the subsections are
        correct for the predefined conditions.
        """
        self._test_scope(scopes.SCOPE_ID_PHOTOS)

    @skipUnless(scopes.is_scope_included(
        expected_scopes, scopes.SCOPE_ID_MUSIC), SKIP_REASON)
    def test_music_scope(self):
        """ Verify that for the music scope the titles the subsections are
        correct for the predefined conditions.
        """
        self._test_scope(scopes.SCOPE_ID_MUSIC)

    @skipUnless(scopes.is_scope_included(
        expected_scopes, scopes.SCOPE_ID_VIDEO), SKIP_REASON)
    def test_video_scope(self):
        """ Verify that for the video scope the titles the subsections are
        correct for the predefined conditions.
        """
        self._test_scope(scopes.SCOPE_ID_VIDEO)

    def _prepare_data(self, src, dst, rescan=False):
        """ Copy test data to the dst and restart the media scanner if
        it is requested"""
        shutil.copy2(src, dst)
        self.addCleanup(os.remove, dst)
        if rescan:
            media.restart_media_scanner()

    def _check_pic_in_carousel(self, scope, pic_path):
        """ Check that a picture is in the carousel """
        pic = scope.get_picture_from_carousel(pic_path)
        self.assertIsNotNone(pic)
        return pic

    def _go_to_photo_preview(self, scope, pic_path):
        """ Go to the preview screen """
        scope.select_picture_from_carousel(pic_path)
        return scope.get_current_photo_preview()

    def _check_photo_preview(self, scope, pic, img_path):
        """ Check the open button, the date and time and that the picture
        which is being displayed is the correct one
        :param scope: The scope object
        :param pic: The picture object
        :param img_path: The path to the image file
        """
        # Go to the preview screen
        preview = self._go_to_photo_preview(scope, pic)

        # Verify the open button is present
        self.assertIsNotNone(preview.get_open_button())

        # Get last acess time for the picture
        last_access_time = timedate.get_utc_last_modification_time(img_path)

        # Verify the date is shown correctly
        self.assertEqual(last_access_time.date(), preview.get_date())

        # Verify the time is shown correctly
        self.assertEqual(last_access_time.time().replace(microsecond=0),
                         preview.get_time())

        # Verify the thumbnail is shown correctly
        self.assertEqual(pic.source, preview.get_thumbnail().source)

    def _go_back_and_validate_header_title(self, expected_title):
        """
        Press back button on current header and validate that the next
        header page has the expected title.
        :param expected_title: Expected title of new header.

        """
        self.dash.get_current_page_header().go_back()
        header = self.dash.get_current_page_header()
        self.assertEqual(header.title, expected_title)

    def test_search_apps_scope(self):
        """
        Test to search the apps scope.
        1) Search the apps scope for an app that is not installed on device.
        2) Check that app is not listed in search results.
        3) Open the app store.
        4) Check that the app can be found in the app store.
        5) Return to Apps scope and search for an app that is installed on
           the device.
        6) Check the app is listed in search results and return to app scope.

        """
        installed_app = 'Contacts'
        not_installed_app = 'OSMTouch'

        scope_data = scopes.get_scope_data(scopes.SCOPE_ID_APPS)
        apps_scope = scopes.go_to_scope(
            scopes.SCOPE_ID_APPS, scope_class=AppsScope)
        apps_scope.search(not_installed_app)
        self.assertFalse(apps_scope.is_app_displayed(not_installed_app))
        self.assertTrue(apps_scope.is_app_displayed(APP_STORE_TITLE))

        apps_scope.open_app_store()
        self.assertTrue(apps_scope.is_app_displayed(not_installed_app))

        self._go_back_and_validate_header_title(APP_STORE_TITLE)
        self._go_back_and_validate_header_title(scope_data.title)

        apps_scope.search(installed_app)
        self.assertTrue(apps_scope.is_app_displayed(installed_app))
        self.assertTrue(apps_scope.is_app_displayed(APP_STORE_TITLE))

        self._go_back_and_validate_header_title(scope_data.title)

    @skipUnless(fs.is_media_folder_dir(fs.DIR_PICTURES),
                'The SD card is not inserted in the device')
    @skipUnless(scopes.is_scope_included(
        expected_scopes, scopes.SCOPE_ID_PHOTOS), SKIP_REASON)
    def test_photos_displayed_from_sd_card(self):
        """ Verify that the pictures stored in the sd card are displayed in
        the photos scope and in the photo preview screen
        """
        sd_card_image = os.path.join(fs.get_media_folder_dir(fs.DIR_PICTURES),
                                     data.images.img2)

        self._prepare_data(SRC_IMAGE_FILE_1, LOCAL_IMAGE_FILE)
        self._prepare_data(SRC_IMAGE_FILE_2, sd_card_image, rescan=True)

        # Go to photos scope
        scope = scopes.go_to_scope(
            scopes.SCOPE_ID_PHOTOS, scope_class=PhotosScope)
        sd_card_pic_file_path = fs.get_file_path(sd_card_image)
        internal_pic_file_path = fs.get_file_path(LOCAL_IMAGE_FILE)

        # Check the two pictures are in the carousel
        self._check_pic_in_carousel(scope, internal_pic_file_path)
        pic = self._check_pic_in_carousel(scope, sd_card_pic_file_path)
        self._check_photo_preview(scope, pic, SRC_IMAGE_FILE_2)

    @skipUnless(scopes.is_scope_included(
        expected_scopes, scopes.SCOPE_ID_PHOTOS), SKIP_REASON)
    def test_photos_displayed_from_local_storage(self):
        """ Verify that the pictures stored on the device are displayed in the
        photos scope and in the photo preview screen, and that the
        gallery opens with a preview of the image

        """
        self._prepare_data(SRC_IMAGE_FILE_1, LOCAL_IMAGE_FILE, rescan=True)

        # Go to photos scope
        scope = scopes.go_to_scope(
            scopes.SCOPE_ID_PHOTOS, scope_class=PhotosScope)
        local_storage_pic_file_path = fs.get_file_path(LOCAL_IMAGE_FILE)

        # Check the local pic is in the carousel
        self._check_pic_in_carousel(scope, local_storage_pic_file_path)
        pic = self._check_pic_in_carousel(scope, local_storage_pic_file_path)

        self._check_photo_preview(scope, pic, SRC_IMAGE_FILE_1)
        preview = self._go_to_photo_preview(scope, pic)
        preview.click_open_button()

        # The gallery opens
        photo_viewer = self.get_gallery_photo_viewer()
        self.assertIn(local_storage_pic_file_path,
                      photo_viewer.get_image_source())

    @skipUnless(scopes.is_scope_included(
        expected_scopes, scopes.SCOPE_ID_VIDEO), SKIP_REASON)
    def test_search_local_video_content(self):
        """ Verify that it is possible to search videos stored in local
        storage from the videos scope
        """
        search_query = 'video2'
        video1_file = 'video1_test.ogv'
        video1_name = 'video1 test'
        video2_file = 'video2_test.ogv'
        video2_name = 'video2 test'

        video1 = os.path.join(fs.DIR_HOME_VIDEOS, video1_file)
        video2 = os.path.join(fs.DIR_HOME_VIDEOS, video2_file)

        self._prepare_data(SRC_VIDEO_FILE_1, video1)
        self._prepare_data(SRC_VIDEO_FILE_1, video2, rescan=True)

        scope = scopes.go_to_scope(scopes.SCOPE_ID_VIDEO,
                                   scope_class=VideoScope)

        scope.search(search_query)
        self.assertTrue(scope.is_in_search_result(video2_name))
        self.assertFalse(scope.is_in_search_result(video1_name))

    @skipUnless(scopes.is_scope_included(
        expected_scopes, scopes.SCOPE_ID_PHOTOS), SKIP_REASON)
    def test_search_local_photo_content(self):
        """ Verify that it is possible to search a picture stored in local
        storage from the photos scope
        """
        scope_data = scopes.get_scope_data(scopes.SCOPE_ID_PHOTOS)
        image_path = fs.get_file_path(LOCAL_IMAGE_FILE)
        self._prepare_data(SRC_IMAGE_FILE_1, LOCAL_IMAGE_FILE, rescan=True)
        scope = scopes.go_to_scope(
            scopes.SCOPE_ID_PHOTOS, scope_class=PhotosScope)

        # Get the creation date of the picture from its preview page
        pic1 = self._check_pic_in_carousel(scope, image_path)
        preview = self._go_to_photo_preview(scope, pic1)
        date = preview.get_date()

        # Search by date, the photo has to be part of the result
        self._go_back_and_validate_header_title(scope_data.title)
        scope.search(date.strftime(scope.SEARCH_FORMAT))
        self.assertTrue(scope.is_in_search_result(image_path))

        # Search by different date, the photo has not to be part of the result
        self._go_back_and_validate_header_title(scope_data.title)
        date = date.replace(year=date.year-1)
        scope.search(date.strftime(scope.SEARCH_FORMAT))
        self.assertFalse(scope.is_in_search_result(image_path))

    @skipUnless(scopes.is_scope_included(
        expected_scopes, scopes.SCOPE_ID_MUSIC), SKIP_REASON)
    def test_play_music_from_music_scope(self):
        """ Verify it is possible to reproduce music from the music scope """

        self._prepare_data(SRC_AUDIO_FILE_2, LOCAL_AUDIO_FILE_2, rescan=True)
        scope = scopes.go_to_scope(scopes.SCOPE_ID_MUSIC,
                                   scope_class=MusicScope)

        # Select the artist
        self.assertTrue(scope.is_artist(data.audio.count20.artist))
        artist_preview = scope.select_artist(data.audio.count20.artist)

        # Play and pause the song and check the song is being played
        track_preview = artist_preview.select_track(data.audio.count20.title)
        track_preview.play()
        self.assertTrue(track_preview.is_progress_bar_moving(),
                        'Progress bar is not moving while the song in playing')
        track_preview.pause()
        self.assertFalse(track_preview.is_progress_bar_moving(),
                         'Progress bar is moving while the song in paused')

        # Verify the music app is playing the selected song
        track_preview.play_in_music_app()
        self.assertTrue(wait_until(music.is_music_app_running))
        self.addCleanup(music.stop_music_app)
        self.assertEqual(LOCAL_AUDIO_FILE_2, music.get_current_song())

    @skipUnless(scopes.is_scope_included(expected_scopes,
                                         scopes.SCOPE_ID_VIDEO), SKIP_REASON)
    def test_display_local_video_content(self):
        """
        Verify local video content is displayed in the videos scope
        """
        video1_file = 'video1_test.ogv'
        video1_name = 'video1 test'
        video2_file = 'video2_test.ogv'
        video2_name = 'video2 test'

        video1 = os.path.join(fs.DIR_HOME_VIDEOS, video1_file)
        video2 = os.path.join(fs.DIR_HOME_VIDEOS, video2_file)

        self._prepare_data(SRC_VIDEO_FILE_1, video1)
        self._prepare_data(SRC_VIDEO_FILE_1, video2, rescan=True)

        scope = scopes.go_to_scope(scopes.SCOPE_ID_VIDEO,
                                   scope_class=VideoScope)
        self.dash.wait_for_dash_loaded()
        video_content = scope.get_my_videos_thumbnails()
        video_names = [video1_name, video2_name]
        video_files = [THUMBNAILER_PREFIX + video1,
                       THUMBNAILER_PREFIX + video2]
        self.assertCountEqual(video_names, video_content[0])
        self.assertCountEqual(video_files, video_content[1])

    def validate_video_information(self, summary, statistics, title, subtitle):
        """ Validates the video information """

        # Assert the summary, title and subtitle are not empty strings
        self.assertLess(1, len(summary))
        self.assertLess(1, len(title))
        self.assertLess(1, len(subtitle))

        # Assert the statistics are well formed
        stats = statistics.split()
        self.assertEqual(4, len(stats))
        self.assertEqual('☝', stats[0])
        self.assertEqual('☟', stats[2])
        self.assertLessEqual(0, int(stats[1]))
        self.assertLessEqual(0, int(stats[3]))

    @skipUnless(scopes.is_scope_included(expected_scopes,
                                         scopes.SCOPE_ID_VIDEO), SKIP_REASON)
    def test_play_online_video_from_the_video_scope(self):
        """
        Verify the video scope show different online videos in the youtube
        section and it is possible reproduce one in the webbrowser
        """

        scope = scopes.go_to_scope(scopes.SCOPE_ID_VIDEO,
                                   scope_class=VideoScope)

        # Validate there are videos displayed in the youtube section
        videos = scope.get_youtube_videos()
        self.assertLessEqual(0, len(videos))

        # Go to the video preview for the first youtube video in the scope
        preview = scope.select_video(videos[0])

        # Validate the information displayed about the video is correct
        summary = preview.get_summary()
        statistics = preview.get_statistics()
        title = preview.get_title()
        subtitle = preview.get_subtitle()
        self.validate_video_information(summary, statistics, title, subtitle)

        # Validate the video online thumbnail exist
        thumbnail = preview.get_thumbnail()
        self.assertTrue(web.exists(thumbnail))

        # Validate the webbrowser is launched by clicking in the play button
        preview.play_video()
        self.assertTrue(wait_until(webbrowser.is_webbrowser_running))

        # Validate the url opened in the webbrowser is youtube
        browser = self.get_webbrowser_app_proxy()
        self.assertIn('youtube', browser.get_current_url())

        # Validate the webbrowser was invoked with the correct video id
        external_youtube_id = preview.get_external_id()
        pid = webbrowser.get_webbrowser_process_id()
        self.assertIn(external_youtube_id, processes.get_process_args(pid))

    @skipUnless(scopes.is_scope_included(expected_scopes,
                                         scopes.SCOPE_ID_MUSIC), SKIP_REASON)
    def test_music_scope_displays_local_music(self):
        """
        Verify music stored in internal storage of the device is shown in the
        music scope and the music artist and albums are correctly displayed in
        my music screen
        """
        self._prepare_data(SRC_AUDIO_FILE_1, LOCAL_AUDIO_FILE_1)
        self._prepare_data(SRC_AUDIO_FILE_2, LOCAL_AUDIO_FILE_2)
        self._prepare_data(SRC_AUDIO_FILE_3, LOCAL_AUDIO_FILE_3, rescan=True)
        songs = [data.audio.ogg_file, data.audio.mp3_file, data.audio.count20]

        # Go to my music screen
        scope = scopes.go_to_scope(scopes.SCOPE_ID_MUSIC,
                                   scope_class=MusicScope)

        # Validate all the artists displayed are the expected
        expected_artists = music_scope.get_artists(songs)
        displayed_artists = sorted(
            scope.get_my_music_titles(len(expected_artists)))
        self.assertEqual(expected_artists, displayed_artists)

        artists_preview = scope.select_my_music()
        music_header = music_scope.get_page_header()

        # Validate all the artists displayed are the expected
        artists = sorted(artists_preview.get_displayed_titles())
        self.assertEqual(expected_artists, artists)

        # Validate all the albums displayed are the expected
        albums_preview = music_header.display_by(
            music_header.MY_MUSIC_ALBUMS_LABEL)
        albums = sorted(albums_preview.get_displayed_titles())
        self.assertEqual(music_scope.get_albums(songs), albums)

    def check_all_included(self, list1, list2):
        """ Verify all the elements in the list1 are includede in the list2 """
        for elem in list1:
            self.assertIn(elem, list2)

    @skipUnless(scopes.is_scope_included(expected_scopes,
                                         scopes.SCOPE_ID_MUSIC), SKIP_REASON)
    def test_music_scope_displays_online_music(self):
        """
        Verify the music scope show different online providers such as youtube
        and sound cloud. The songs that are shown in the music scope for those
        providers are also shown in their category previews
        """
        # Go to my music scope
        scope = scopes.go_to_scope(scopes.SCOPE_ID_MUSIC,
                                   scope_class=MusicScope)

        # Get the song titles shown in the music scope and validate at least
        # there are 2 songs for each
        digital_scope_titles = scope.get_7digital_titles()
        self.assertLessEqual(2, len(digital_scope_titles))

        sound_cloud_scope_titles = scope.get_sound_cloud_titles()
        self.assertLessEqual(2, len(sound_cloud_scope_titles))

        # Get the song titles shown in the 7digital category preview
        digital_preview_titles = scope.select_7digital().get_displayed_titles()

        # Check the 7digital albums can be displayed by top albums
        music_header = music_scope.get_page_header()
        music_header.display_by(music_header.DIGITAL_ALL_CATEGORIES_LABEL)
        top_albums_preview = music_header.display_by(
            music_header.DIGITAL_TOP_ALBUMS_LABEL)
        self.assertLessEqual(2, len(top_albums_preview.get_displayed_titles()))
        self.dash.get_current_page_header().go_back()

        # Get the song titles shown in the sound cloud category preview
        sound_cloud_preview_titles = \
            scope.select_sound_cloud().get_displayed_titles()
        self.dash.get_current_page_header().go_back()

        # Validate the songs in the scope appear in the category previews
        self.check_all_included(digital_scope_titles, digital_preview_titles)
        self.check_all_included(sound_cloud_scope_titles,
                                sound_cloud_preview_titles)

    def test_manage_scopes_displays_scopes(self):
        """
        The 'Manage' page displays an expected number of favorite and
        other scopes.

        """
        # Swipe in the launcher and press the Ubuntu button
        shell = self.unity_proxy.main_window
        shell.show_dash_from_launcher()
        scopes_list = self.dash.open_manage_scopes()
        # Check there are scopes visible here
        self.assertGreater(len(scopes_list.get_scopes_list()), 0)
        # The favourite scopes are all visible
        self.assertTrue(scopes_list.favorite_scopes_are_visible())
        # Swipe up and ensure there are scopes available
        # Multiple scopes are visible as you scroll
        self.assertTrue(scopes_list.other_scopes_are_visible())
        self.assertTrue(scopes_list.scrolling_down_shows_more_scopes())

    @skipUnless(scopes.is_scope_included(expected_scopes,
                                         scopes.SCOPE_ID_VIDEO), SKIP_REASON)
    def test_play_local_video_from_the_video_scope(self):
        """
        Verify the video scope show local videos and it is possible reproduce
        one in the media player
        """
        self._prepare_data(SRC_VIDEO_FILE_2, LOCAL_VIDEO_FILE_2, rescan=True)

        scope = scopes.go_to_scope(scopes.SCOPE_ID_VIDEO,
                                   scope_class=VideoScope)

        # Get the video from my videos section
        video = scope.get_my_videos()[0]

        # Go to the video preview and validate the thumbnail and title
        preview = scope.select_video(video)
        self.assertEqual(THUMBNAILER_PREFIX + LOCAL_VIDEO_FILE_2,
                         preview.get_thumbnail())
        self.assertEqual(data.video.video_avi.split('.')[0],
                         preview.get_title())

        # Play the video and validate it is reproduced in the media player
        preview.play_video()

        self.assertTrue(base.wait_until(mediaplayer.is_mediaplayer_running))
        media_player = self.get_media_player_proxy()
        self.assertEqual(fs.get_file_path(LOCAL_VIDEO_FILE_2),
                         media_player.get_player().source)

        # Validate the video is played and it finishes
        self.assertTrue(base.wait_until(media_player.is_playing))
        self.assertTrue(base.wait_until(media_player.is_play_finished))
