# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3,
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
from testtools import skipUnless

from ubuntu_system_tests.helpers.data import load_test_metadata
from ubuntu_system_tests.helpers import file_system as fs
from ubuntu_system_tests.helpers import indicators
from ubuntu_system_tests.helpers import media
from ubuntu_system_tests.helpers import scopes
from ubuntu_system_tests.helpers.scopes.music import MusicScope
from ubuntu_system_tests.helpers import sdcard
from ubuntu_system_tests.helpers import url_dispatcher

from ubuntu_system_tests.tests import base

data = load_test_metadata()
scopes_list = scopes.get_scope_list()

MESSAGES_INDICATOR = 'indicator-messages'
MESSAGES_INDICATOR_PAGE = 'indicator-messages-page'

SKIP_REASON = 'The SD card is not inserted in the device'
INSERT_MSG = 'detected'
REMOVE_MSG = 'removed'

SRC_AUDIO_FILE_1 = os.path.join(fs.DIR_TEST_DATA_AUDIO,
                                data.audio.count20.file)


class SDCardSystemTest(base.BaseUbuntuSystemTestCase):
    """This Class tests the sd card"""

    @skipUnless(fs.is_media_folder_dir(fs.DIR_PICTURES), SKIP_REASON)
    def setUp(self):
        super().setUp()
        self.unity_proxy = self.restart_unity()
        self.useFixture(media.BackupRestoreLocalMediaFixture())
        self.addCleanup(url_dispatcher.go_to_url, scopes_list[0].url)
        self.msg_number = 0

    def _get_message_indicator(self):
        return indicators.MessageIndicator(self.unity_proxy.main_window,
                                           MESSAGES_INDICATOR,
                                           MESSAGES_INDICATOR_PAGE)

    def _remove_sd_card(self):
        """
        Remove the sd card, validate the indicator is displayed and wait until
        the indicator disappears
        :return: The removed device path
        """
        device = sdcard.remove()
        self.addCleanup(sdcard.insert, device)
        self.msg_number += 1
        indicator = self._get_message_indicator()
        self.assertIn(REMOVE_MSG, indicator.get_last_message(self.msg_number))
        indicator.wait_while_indicator_visible()
        return device

    def _insert_sd_card(self, device):
        """
        Insert the sd card and validate the indicator is displayed
        :param number: The number of message indicators displayed in the
        current test counting this one
        """
        sdcard.insert(device)
        self.msg_number += 1
        indicator = self._get_message_indicator()
        self.assertIn(INSERT_MSG, indicator.get_last_message(self.msg_number))

    def test_insert_sd_card_and_explore(self):
        """ Verify music stored in the sd card is displayed and can be played
        in the music scope when the sd card is inserted.
        """

        # Prerequisite: Copy music and remove the sd card
        sdcard.copy_music(SRC_AUDIO_FILE_1)
        device = self._remove_sd_card()

        # Go to the music scope
        scope = scopes.go_to_scope(scopes.SCOPE_ID_MUSIC,
                                   scope_class=MusicScope)

        # Check there are not any music displayed in my music section
        self.assertFalse(scope.is_my_music_displayed())

        # Insert the sd card and validate the indicator is displayed
        self._insert_sd_card(device)

        # Check the song copied to the sd card is displayed in my music section
        self.assertEqual(1, len(scope.get_my_music_titles(count=1)))

        # Play the song and check the song is being played
        artist_preview = scope.select_artist(data.audio.count20.artist)
        track_preview = artist_preview.select_track(data.audio.count20.title)
        track_preview.play()
        self.assertTrue(track_preview.is_progress_bar_moving())
        track_preview.pause()
