# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2014, 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from collections import namedtuple
import logging
import os.path

from autopilot import exceptions
from autopilot.matchers import Eventually
from testtools.matchers import Equals

from ubuntu_system_tests.helpers import download_manager as dm
from ubuntu_system_tests.helpers import file_system as fs
from ubuntu_system_tests.helpers import gallery
from ubuntu_system_tests.helpers import wait_until
from ubuntu_system_tests.tests import base_webbrowser

logger = logging.getLogger(__name__)
PageUrls = namedtuple('PageUrls', ['entered_url',
                                   'title_url',
                                   'actual_url',
                                   'page_title'])
FileDesc = namedtuple('FileDesc', ['file_name',
                                   'sha1'])

APP = 'webbrowser-app'
PARAMS = '--desktop_file_hint=/usr/share/applications/webbrowser-app.desktop'
ARGS = ['--new-session']
QT = 'qt'

test_data = {
    'start_page': PageUrls(
        entered_url='start.ubuntu.com',
        title_url='start.ubuntu.com',
        actual_url='http://start.ubuntu.com/current/Google/?sourceid=hp',
        page_title='Ubuntu Start Page'),
    'page1': PageUrls(
        entered_url='people.canonical.com/~platform-qa/page1.html',
        title_url='people.canonical.com',
        actual_url='http://people.canonical.com/~platform-qa/page1.html',
        page_title='First page'),
    'page2': PageUrls(
        entered_url='lillypilly.canonical.com/~platform-qa/page2.html',
        title_url='lillypilly.canonical.com',
        actual_url='http://lillypilly.canonical.com/~platform-qa/page2.html',
        page_title='Second page'),
    'page3': PageUrls(
        entered_url='lillypilly.canonical.com/~platform-qa/page3.html',
        title_url='lillypilly.canonical.com',
        actual_url='http://lillypilly.canonical.com/~platform-qa/page3.html',
        page_title='Third page'),
    'ubuntu': PageUrls(
        entered_url='www.ubuntu.com',
        title_url='ubuntu.com',
        actual_url='http://www.ubuntu.com/',
        page_title='The leading OS for PC, tablet, phone and cloud | Ubuntu'),
    'canonical': PageUrls(
        entered_url='www.canonical.com',
        title_url='canonical.com',
        actual_url='http://www.canonical.com/',
        page_title='Canonical | The company behind Ubuntu'),
    'linux': PageUrls(
        entered_url='http://www.linux.org/',
        title_url='linux.org',
        actual_url='http://www.linux.org/',
        page_title='Linux.org'),
    'ubuntu_img': FileDesc(
        file_name='ubuntu.jpg',
        sha1='429cbd692c4abd3e5c785edf5dcf0993d21076b1')
    }


class WebBrowserSystemTest(base_webbrowser.WebBrowserBase):
    """This Class tests the web browser app"""

    def setUp(self):
        super().setUp()

    def get_download_notification(self):
        notifications = self.unity_proxy.main_window.get_notifications_list()
        return notifications.get_download_finished_notification()

    def ensure_browser_actually_loaded(self, unity_proxy):
        """Fail the test if the browser app doesn't get past loading.

        Due to this bug[1] after a reboot or 2 of unity8 the browser is no
        longer launchable from the apps scope.
        Check for when this happens and if it does, fail the test with a nice
        message.

        [1] https://bugs.launchpad.net/ubuntu/+source/unity8/+bug/1401361

        """
        try:
            spinner = unity_proxy.select_single('ActivityIndicatorStyle')
            spinner.wait_until_destroyed()
        except exceptions.StateNotFoundError:
            # This is fine, it's already loaded
            pass
        except RuntimeError:
            self.fail("""Test failed due to browser not loading.
            Please see this bug for further information:
            https://bugs.launchpad.net/ubuntu/+source/unity8/+bug/1401361
            """)

    def test_navigate_forward_back(self):
        """Test the back and forward mechanism provided by the web browser"""
        self.browser.go_to_url(test_data.get('page1').entered_url)
        self.browser.wait_until_page_loaded(test_data.get('page1').actual_url)

        self.browser.go_to_url(test_data.get('page2').entered_url)
        self.browser.wait_until_page_loaded(test_data.get('page2').actual_url)

        self.browser.go_back()
        self.browser.wait_until_page_loaded(test_data.get('page1').actual_url)

        self.browser.go_forward()
        self.browser.wait_until_page_loaded(test_data.get('page2').actual_url)

    def test_navigate_backwards_quickly_no_crash(self):
        """Test quickly going back a few pages does not crash the web
        browser."""
        for page in ['page1', 'page2', 'ubuntu', 'canonical']:
            self.browser.go_to_url(test_data.get(page).entered_url)
            self.browser.wait_until_page_loaded(test_data.get(page).actual_url)

        for i in range(4):
            self.browser.go_back()

        self.browser.wait_until_page_loaded(
            test_data.get('start_page').actual_url)

    def test_history_view(self):
        """Test that after visit three web pages, all of them are listed in the
        history page of the web browser"""
        self.browser.go_to_url(test_data.get('ubuntu').entered_url)
        self.browser.wait_until_page_loaded(test_data.get('ubuntu').actual_url)

        self.browser.go_to_url(test_data.get('linux').entered_url)
        self.browser.wait_until_page_loaded(test_data.get('linux').actual_url)

        self.browser.go_to_url(test_data.get('canonical').entered_url)
        self.browser.wait_until_page_loaded(
            test_data.get('canonical').actual_url)

        self.browser.open_history_view()

        titles = [url.title for url in self.browser.get_history_urls()]
        self.assertEqual(len(titles), 3)
        self.assertIn(test_data.get('canonical').title_url, titles)
        self.assertIn(test_data.get('linux').title_url, titles)
        self.assertIn(test_data.get('ubuntu').title_url, titles)

    def test_create_bookmark_and_navigate(self):
        """Test that the user is able to create a bookmark and then navigate
        to it"""
        self.browser.go_to_url(test_data.get('ubuntu').entered_url)
        self.browser.wait_until_page_loaded(
            test_data.get('ubuntu').actual_url)

        bookmark_toggle = self.browser.address_bar.get_bookmark_toggle()
        self.browser.pointing_device.click_object(bookmark_toggle)
        self.assertThat(self.browser.chrome.bookmarked,
                        Eventually(Equals(True)))

        self.browser.go_to_url(test_data.get('canonical').entered_url)
        self.browser.wait_until_page_loaded(
            test_data.get('canonical').actual_url)
        self.assertThat(self.browser.chrome.bookmarked,
                        Eventually(Equals(False)))

        self.browser.open_new_tab()

        titles = [url.title for url in self.browser.get_tab_preview_urls()]

        self.assertEqual(len(titles), 5)
        self.assertEqual(titles.count(test_data.get('ubuntu').page_title), 2)
        self.assertIn(test_data.get('start_page').page_title, titles)
        self.assertIn(test_data.get('canonical').page_title, titles)

    def test_delete_bookmark(self):
        """Test that the user is able to create a bookmark and then delete
        it"""
        self.browser.go_to_url(test_data.get('ubuntu').entered_url)
        self.browser.wait_until_page_loaded(test_data.get('ubuntu').actual_url)

        bookmark_toggle = self.browser.address_bar.get_bookmark_toggle()
        self.browser.pointing_device.click_object(bookmark_toggle)
        self.assertThat(self.browser.chrome.bookmarked,
                        Eventually(Equals(True)))

        self.browser.go_to_url(test_data.get('ubuntu').entered_url)
        self.browser.wait_until_page_loaded(test_data.get('ubuntu').actual_url)

        self.browser.pointing_device.click_object(bookmark_toggle)
        self.assertThat(self.browser.chrome.bookmarked,
                        Eventually(Equals(False)))

        self.browser.open_new_tab()

        titles = [url.title for url in self.browser.get_tab_preview_urls()]

        self.assertEqual(len(titles), 3)
        self.assertEqual(titles.count(test_data.get('ubuntu').page_title), 1)
        self.assertIn(test_data.get('start_page').page_title, titles)

    def test_download_image(self):
        """Test to download an image from internet an save it locally"""
        img = test_data.get('ubuntu_img')

        # Clean up the pictures directory
        pics_dir = os.path.join(os.path.expanduser("~"), "Pictures")
        fs.clean_files(pics_dir)

        # Go to page with the picture to download
        self.browser.go_to_url(test_data.get('page3').entered_url)
        self.browser.wait_until_page_loaded(test_data.get('page3').actual_url)

        # Click on the picture
        webview = self.browser.get_current_webview()
        self.browser.pointing_device.move_to_object(webview)
        self.browser.pointing_device.move(self.browser.pointing_device.x,
                                          self.browser.pointing_device.y / 2)
        self.browser.pointing_device.click(press_duration=3)

        # Select to save the image
        save_image = self.browser.get_popover_foreground_dialog_item(
            'Save image')
        self.browser.pointing_device.click_object(save_image)

        # Pick the gallery to see the picture after it is downlaoded
        gallery_button = self.browser.get_gallery_button()
        self.browser.pointing_device.click_object(gallery_button)

        # Once it is downloaded, press open button in the notification
        notification = self.get_download_notification()
        notification.open()

        # Clean the image after the test case ends
        downloaded_file = os.path.join(pics_dir, img.file_name)
        self.addCleanup(fs.delete_file, downloaded_file)
        self.addCleanup(dm.clear_download_manager_data())
        self.addCleanup(gallery.clear_gallery_data)

        # Validates the file downloaded exists
        self.assertTrue(wait_until(os.path.isfile, 10, 0.5, downloaded_file))

        # Validate the gallery-app is displaying the downloaded imgage
        events_view = self.get_gallery_events_view()
        self.assertTrue(events_view.is_image_displayed(img.file_name))

        # Validates the file is the same than the original
        # Note: The verification fail due to bug #1463559 in the gallery-app
        actual_sha1 = fs.calculate_file_sha1(downloaded_file)
        self.assertEqual(img.sha1, actual_sha1)
